/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/**
 * 設定管理
 *   Inherits From: nsIObserver
 */
function UnMHTConfigManager () {
  this.loadPrefBranch ();
}
UnMHTConfigManager.prototype = {
  prefBranch : null,    /* nsIPrefBranch/nsIPrefBranch2  pref サービス */
    
  listener : null,      /* UnMHTConfigListener  設定変更イベントリスナ */
    
  /**
   * リスナ登録
   *
   * @param  UnMHTConfigListener listener
   *         設定変更イベントリスナ
   */
  setListener : function (listener) {
    if (Components.interfaces.nsIPrefBranch2) {
      this.listener = listener;
            
      this.prefBranch.addObserver ("unmht.savepref",
                                   this,
                                   false);
    }
  },
    
  /**
   * リスナ削除
   */
  resetListener : function () {
    if (Components.interfaces.nsIPrefBranch2) {
      /* 設定の変更の監視を解除する */
      this.prefBranch.removeObserver ("unmht.savepref",
                                      this);
    }
  },
    
  /**
   * prefBranch を設定し直す
   */
  loadPrefBranch : function () {
    if (Components.interfaces.nsIPrefBranch2) {
      this.prefBranch
      = Components.classes ["@mozilla.org/preferences-service;1"]
      .getService (Components.interfaces.nsIPrefBranch2);
    }
    else {
      this.prefBranch
      = Components.classes ["@mozilla.org/preferences-service;1"]
      .getService (Components.interfaces.nsIPrefBranch);
    }
  },
    
  /**
   * 設定を読み込む
   * 設定が無ければ既定値を書き込む
   *
   * @param  String type
   *         設定の種類
   *           "bool": bool
   *           "char": 文字列
   *           "int":  数値
   * @param  String name
   *         設定名
   * @param  Boolean/String/Number value
   *         既定値
   * @return Boolean/String/Number
   *         取得した値
   *         設定が無ければ既定値
   */
  initPref : function (type, name, value) {
    if (this.prefBranch.prefHasUserValue (name)) {
      ; /* switch のインデント用 */
      switch (type) {
        case "bool":
          value = this.prefBranch.getBoolPref (name);
          break;
        case "char":
          value = this.prefBranch.getCharPref (name);
          break;
        case "int":
          value = this.prefBranch.getIntPref (name);
          break;
      }
    }
    else {
      ; /* switch のインデント用 */
      switch (type) {
        case "bool":
          this.prefBranch.setBoolPref (name, value);
          break;
        case "char":
          this.prefBranch.setCharPref (name, value);
          break;
        case "int":
          this.prefBranch.setIntPref (name, value);
          break;
      }
    }
        
    return value;
  },
    
  /**
   * 設定を書き込む
   *
   * @param  String type
   *         設定の種類
   *           "bool": bool
   *           "char": 文字列
   *           "int":  数値
   * @param  String name
   *         設定名
   * @param  Boolean/String/Number value
   *         既定値
   */
  setPref : function (type, name, value) {
    ; /* switch のインデント用 */
    switch (type) {
      case "bool":
        this.prefBranch.setBoolPref (name, value);
        break;
      case "char":
        this.prefBranch.setCharPref (name, value);
        break;
      case "int":
        this.prefBranch.setIntPref (name, value);
        break;
    }
  },
    
  /**
   * 設定変更イベント
   *   nsIObserver.observe
   *
   * @param  nsISupports subject
   *         不明
   * @param  String topic
   *         通知の対象
   * @param  String data
   *         通知の内容
   */
  observe : function (subject, topic, data) {
    if (topic == "nsPref:changed") {
      this.listener.onConfigChange ();
    }
  },
  
  /**
   * 設定が監視可能かどうかを返す
   *
   * @return Boolean
   *         設定が監視可能かどうか
   */
  isObservable : function () {
    return Components.interfaces.nsIPrefBranch2 != undefined;
  },
  
  defaultTime : -2,
  
  /**
   * スクリプトの実行時間を設定する
   *
   * @param Number t
   *        時間
   */
  setTime : function (t) {
    if (this.defaultTime == -2) {
      if (this.prefBranch
          .prefHasUserValue ("dom.max_chrome_script_run_time")) {
        this.defaultTime
        = this.prefBranch
        .getIntPref ("dom.max_chrome_script_run_time");
      }
      else {
        this.defaultTime = -1;
      }
    }
    this.prefBranch
    .setIntPref ("dom.max_chrome_script_run_time", t);
  },
  
  /**
   * スクリプトの実行時間を元に戻す
   */
  restoreTime : function () {
    if (this.defaultTime == -2) {
    }
    else if (this.defaultTime == -1) {
      if (this.prefBranch
          .prefHasUserValue ("dom.max_chrome_script_run_time")) {
        this.prefBranch
        .clearUserPref ("dom.max_chrome_script_run_time");
      }
    }
    else {
      this.prefBranch
      .setIntPref ("dom.max_chrome_script_run_time",
                   this.defaultTime);
    }
  }
};
