/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/**
 * 文字列変換器
 */
function UnMHTConverter () {
  this.converter
    = Components.classes ["@mozilla.org/intl/scriptableunicodeconverter"]
    .getService (Components.interfaces.nsIScriptableUnicodeConverter);
}
UnMHTConverter.prototype = {
  converter : null, /* nsIScriptableUnicodeConverter  文字コード変換器 */
    
  /**
   * 初期化
   */
  init : function () {
  },
    
  /**
   * UTF-16 から ISO-2022-JP に変換する
   *
   * @param  String text
   *         UTF-16 の文字列
   * @return String
   *         ISO-2022-JP に変換した文字列
   */
  convertToISO2022JP : function (text) {
    this.converter.charset = "iso-2022-jp";
    return this.converter.ConvertFromUnicode (text);
  },
    
  /**
   * HTML に使えない文字をエスケープする
   * 
   * @param  String text
   *         エスケープする文字列
   * @return String
   *         エスケープした文字列
   */
  escapeEntity : function (text) {
    return text
    .replace (/&/g, "&amp;")
    .replace (/\"/g, "&quot;")
    .replace (/\'/g, "&#x27;")
    .replace (/</g, "&lt;")
    .replace (/>/g, "&gt;")
    .replace (/\xa0/g, "&nbsp;");
  },
    
  /**
   * HTML に使えない文字のエスケープを解除する
   * 
   * @param  String text
   *         エスケープを解除する文字列
   * @return String
   *         エスケープを解除した文字列
   */
  unescapeEntity : function (text) {
    return text
    .replace (/&gt;/g, ">")
    .replace (/&lt;/g, "<")
    .replace (/&quot;/g, "\"")
    .replace (/&#x27;/g, "\'")
    .replace (/&nbsp;/g, " ")
    .replace (/&amp;/g, "&");
  },
    
  /**
   * Quoted-Printable にエンコードする
   *
   * @param  String text
   *         エンコードする文字列
   * @return String
   *         エンコードした文字列
   */
  encode_qp : function (aOctets) {
    // Encode the mandatory characters. Octets with decimal values of
    // 33 through 60, inclusive, and 62 through 126, inclusive,
    // are not encoded. Spaces, tabs, and line breaks will be
    // encoded or normalized later, if necessary.
    var aEncodedLines
    = aOctets.replace
    (/[^\t\r\n \x21-\x3C\x3E-\x7E]/g,
     function (aMatch) {
      // Convert the octet to hexadecimal representation
      var hexString
      = "0" + aMatch.charCodeAt(0).toString(16).toUpperCase();
      // Consider only the last two digits of the number
      return "=" + hexString.slice (-2);
    });

    // Limit the final line length to 76 characters,
    // adding soft line breaks if necessary. Also convert every type of
    // line break to CRLF. Since the regular expression used for
    // the task cannot handle strings that don't end with a line break,
    // add one now and remove it at the end.
    return (aEncodedLines + "\r\n").replace
    /*
     * The regular expression below is composed of the following parts:
     *
     * aMain   ( [^\r\n]{0,73} )
     *
     * The first 73 characters of each line, up to and excluding the end of
     *  line character, if present.
     *
     * aLastThree   ( =.. or [ \t] or [^\r\n=]{2}[^\t\r\n =] )
     *
     * This group will match only if followed by a line ending. Can be an
     *  encoded octet representation, a space or tab (that will be encoded
     *  to three characters) or a sequence of three characters that does not
     *  contain the beginning of an encoded sequence ("=") and does not end
     *  with a space or tab.
     *
     * aLastThreeEOL   ( $ or \r?\n or \r )
     *
     * Line ending after aLastThree. This group is empty only if the end of
     *  the string is reached.
     *
     * aLastTwo   ( [^\t\r\n =]{0,2} )
     *
     * Up to two characters that normally precede a soft line break. None of
     *  these characters will need further encoding.
     *
     * aLastTwoEOL   ( \r?\n? )
     *
     * Optional line ending. If this group matches,
     * then no soft line break is needed.
     */
    (/([^\r\n]{0,73})(?:(=..|[ \t]|[^\r\n=]{2}[^\t\r\n =])($|\r?\n|\r)|([^\t\r\n =]{0,2})(\r?\n?))/g,
     function (aAll, aMain,
               aLastThree, aLastThreeEOL,
               aLastTwo, aLastTwoEOL,
               aOffset) {
      // Compose the main text of the line
      var line = aMain + aLastThree + aLastTwo;
      // If a line break was found in the original string
      if (aLastThreeEOL || aLastTwoEOL) {
        // If the last character in the line is a tab or a space
        // and no soft line break will be added, encode the character
        if (line) {
          var lastChar = line [line.length - 1];
          if (lastChar === " "
              || lastChar === "\t") {
            line
              = line.slice (0, -1)
              + (lastChar === " " ? "=20" : "=09");
          }
        }
        // Return the line followed by a hard line break
        return line + "\r\n";
      }
      // Return the line followed by a soft line break. Since the regular
      // expression also matches the empty string, this function is called
      // one last time with empty parameters. In that case, do not add the
      // soft line break.
      return line ? (line + "=\r\n") : "";
    }).slice (0, -2);
  },
    
  /**
   * Quoted-Printable をデコードする
   *
   * @param  String text
   *         デコードする文字列
   * @return String
   *         デコードした文字列
   */
  decode_qp : function (aAsciiString) {
    // Replace every soft line break and encoded character in the string.
    // Soft line breaks are represented by an equal sign ("=") followed by
    // any valid end-of-line sequence, while encoded characters are
    // represented by an equal sign immediately followed by two hexadecimal
    // digits, either uppercase or lowercase.
    return aAsciiString.replace
    (/=(?:\r?\n|\r|([A-Fa-f0-9]{2}))/g,
     function (aAll, aEncodedOctet) {
      return (aEncodedOctet ?
              String.fromCharCode (parseInt (aEncodedOctet, 16)) : "");
    });
  },
    
  /**
   * Unicode から変換する
   *
   * @param  String text
   *         変換する文字列
   * @param  String charset
   *         変換先の文字コード
   * @return String
   *         変換した文字列
   */
  convertFromUnicode : function (text, charset) {
    var text2;
            
    try {
      this.converter.charset = charset;
      text2 = this.converter.ConvertFromUnicode (text);
    }
    catch (e) {
      text2 = text;
    }
            
    return text2;
  },
  
  /**
   * Unicode から変換する
   * 変換できないものは実体参照に置き換える
   *
   * @param  String text
   *         変換する文字列
   * @param  String charset
   *         変換先の文字コード
   * @return String
   *         変換した文字列
   */
  convertFromUnicodeWithEntity : function (text, charset) {
    var text2
    = this.convertToUnicode (this.convertFromUnicode (text, charset), charset);
    var result = "";
    
    for (;;) {
      var p = text2.indexOf ("?");
      if (p == -1) {
        result += text2;
        break;
      }
      
      if (text.charAt (p) != "?") {
        /* 変換できなかったため ? になった */
        
        result += text2.substr (0, p);
        
        var code1 = text.charCodeAt (p);
        if (code1 >= 0xd800 && code1 <= 0xdbff) {
          /* サロゲートペア */
          var code2 = text.charCodeAt (p + 1);
          var code = (code1 - 0xd800) * 0x400 + (code2 - 0xdc00) + 0x10000;
          
          result += "&#" + code + ";";
          
          text = text.substr (p + 2);
          text2 = text2.substr (p + 1);
        }
        else {
          result += "&#" + code1 + ";";
          text = text.substr (p + 1);
          text2 = text2.substr (p + 1);
        }
      }
      else {
        /* 本当に ? だった */
        result += text2.substr (0, p + 1);
        text = text.substr (p + 1);
        text2 = text2.substr (p + 1);
      }
    }
    
    return this.convertFromUnicode (result, charset);
  },
  
  /**
   * Unicode に変換する
   *
   * @param  String text
   *         変換する文字列
   * @param  String charset
   *         変換元の文字コード
   * @return String
   *         変換した文字列
   */
  convertToUnicode : function (text, charset) {
    var text2;
    try {
      this.converter.charset = charset;
      text2 = this.converter.ConvertToUnicode (text);
    }
    catch (e) {
      text2 = text;
    }
            
    return text2;
  },
    
  /**
   * encoded word をデコードする
   *
   * @param  String text
   *         デコードする文字列
   * @param  Boolean utf16
   *         text が UTF-16 であるかどうか
   * @return String
   *         デコードした文字列
   */
  decode_ew : function (text, utf16) {
    if (text.match (/^=\?([^\?]+)\?([^\?])\?([^\?]+)\?=$/)) {
      var encodedCharset = RegExp.$1;
      var encodedType = RegExp.$2;
      var encodedText = RegExp.$3;
            
      if (encodedType == "Q") {
        encodedText
          = encodedText.replace
          (/=([A-Fa-f0-9][A-Fa-f0-9])/g,
           function (dummy, matched) {
            return String.fromCharCode ("0x" + matched);
          });
      }
      else if (encodedType == "B") {
        encodedText = atob (encodedText);
      }
            
      return this.convertToUnicode (encodedText,
                                    encodedCharset);
    }
        
    if (utf16) {
      return text;
    }
        
    return this.convertToUnicode (text, "shift_jis");
  },
    
  /**
   * Base64 をデコードする
   *
   * @param  String text
   *         デコードする文字列
   * @return String
   *         デコードした文字列
   */
  atob : function (text) {
    var b
    = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
        
    var result = "";
    var c, part1, part2, part3, part4;

    for (var i = 0; i < text.length; i += 4) {
      part1 = b.indexOf (text.substr (i, 1));
      part2 = b.indexOf (text.substr (i + 1, 1));
      part3 = b.indexOf (text.substr (i + 2, 1));
      part4 = b.indexOf (text.substr (i + 3, 1));
            
      if (part1 != -1 && part2 != -1) {
        result += String.fromCharCode ((part1 << 2)
                                       | (part2 >> 4));
      }
      if (part2 != -1 && part3 != -1 && part3 != 64) {
        result += String.fromCharCode (((part2 & 0x0f) << 4)
                                       | (part3 >> 2));
      }
      if (part3 != -1 && part4 != -1 && part4 != 64) {
        result += String.fromCharCode (((part3 & 0x03) << 6)
                                       | (part4));
      }
    }
        
    return result;
  },
    
  /**
   * Base64 にエンコードする
   *
   * @param  String text
   *         エンコードする文字列
   * @return String
   *         エンコードした文字列
   */
  btoa8 : function (text) {
    var b
    = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
        
    var result = "";
    var c;
    var part1, part2, part3, part4;
        
    for (var i = 0; i < text.length; i += 3) {
      c = text.charCodeAt (i) & 0xff;
      part1 = c >> 2;
      part2 = (c & 0x3) << 4;
      if (i + 1 == text.length) {
        part3 = 64;
        part4 = 64;
      }
      else {
        c = text.charCodeAt (i + 1) & 0xff;
        part2 |= (c & 0xf0) >> 4;
        part3 = (c & 0x0f) << 2;
        if (i + 2 == text.length) {
          part4 = 64;
        }
        else {
          c = text.charCodeAt (i + 2) & 0xff;
          part3 |= (c & 0xc0) >> 6;
          part4 = c & 0x3f;
        }
      }
      result
        += b.substr (part1, 1)
        + b.substr (part2, 1)
        + b.substr (part3, 1)
        + b.substr (part4, 1);
    }
    return result;
  },
    
  /**
   * UTF-16 を Base64 にエンコードする
   *
   * @param  String text
   *         エンコードする文字列
   * @return String
   *         エンコードした文字列
   */
  btoa16 : function (text) {
    var b
    = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
        
    var result = "";
    var c;
    var part1a, part2a, part3a, part4a;
    var part1b, part2b, part3b, part4b;
        
    for (var i = 0; i < text.length; i += 3) {
      c = (text.charCodeAt (i) >> 8) & 0xff;
      part1a = c >> 2;
      part2a = (c & 0x3) << 4;
      c = text.charCodeAt (i) & 0xff;
      part2a |= (c & 0xf0) >> 4;
      part3a = (c & 0x0f) << 2;
      if (i + 1 == text.length) {
        part4a = 64;
        partab = -1;
      }
      else {
        c = (text.charCodeAt (i + 1) >> 8) & 0xff;
        part3a |= (c & 0xc0) >> 6;
        part4a = c & 0x3f;
        c = text.charCodeAt (i + 1) & 0xff;
        part1b = c >> 2;
        part2b = (c & 0x3) << 4;
        if (i + 2 == text.length) {
          part3b = 64;
          part4b = 64;
        }
        else {
          c = (text.charCodeAt (i + 2) >> 8) & 0xff;
          part2b |= (c & 0xf0) >> 4;
          part3b = (c & 0x0f) << 2;
          c = text.charCodeAt (i + 2) & 0xff;
          part3b |= (c & 0xc0) >> 6;
          part4b = c & 0x3f;
        }
      }
            
      result
        += b.substr (part1a, 1)
        + b.substr (part2a, 1)
        + b.substr (part3a, 1)
        + b.substr (part4a, 1);
            
      if (part1b != -1) {
        result
          += b.substr (part1b, 1)
          + b.substr (part2b, 1)
          + b.substr (part3b, 1)
          + b.substr (part4b, 1);
      }
    }
    return result;
  }
};
