/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/**
 * mht ファイルに含まれるファイル1つ当たりの情報
 */
function UnMHTExtractorFile () {
}
UnMHTExtractorFile.prototype = {
  charset : "",         /* String  Content-Type のうち、文字コード */
  filetype : "",        /* String  Content-Type のうち、MIME タイプ */
  
  parentFile : null,    /* UnMHTExtractorFile  親のファイル */
    
  cid : "",             /* String  Content-ID の値 */
    
  location : "",        /* String  ファイルのアドレス */
  contentLocation : "", /* String  Content-Location の値 */
  contentBase : "",     /* String  Content-Base の値 */
    
  baseDir : "",         /* String  親ディレクトリ */
  referredBaseDir : "", /* String  参照元のファイルの親ディレクトリ
                         *   IE で保存されたファイルの場合
                         *   CSS から CSS が相対パスで参照されると
                         *   パスがおかしくなるため、これを修正するために用いる */
    
  leafName : "",        /* String  ファイル名 */
  extension : "",       /* String  拡張子 (. も含む) */
    
  disposition : "",     /* String  Content-Disposition の filename の値 */
  dispositionType : "", /* String  Content-Disposition のタイプ */
    
  encoding : 0,         /* Number  Content-Encoding の値
                         *   1: quoted-printable
                         *   2: base64
                         *   3: 7bit
                         *   0: 8bit */
    
  content : "",         /* String  ファイルの中身 */
  originalContent : "", /* String  ファイルの元の中身 */
    
  deleted : false,      /* Boolean  展開しない事を示すフラグ */
    
  nativeFilename : "",  /* String  保存したファイル名 */
  size : 0,             /* Number  ファイルサイズ */
    
  refName : "",         /* String  参照用のファイル名 */
    
  headerMap : null,     /* Object  ヘッダ
                         *   <String フィールド名, String フィールド値> */
  headerList : null,    /* Array  ヘッダ (小文字化しない)
                         *   [[String フィールド名, String フィールド値], ...] */
    
  boundary : "",        /* String  boundary パラメータ */
  childFiles : null     /* Array  含むファイル (multipart の場合)
                         *   [UnMHTExtractorFile, ...] */
};

/*
 * ファイル展開器
 */
function UnMHTExtractor () {
  this.conv = new UnMHTConverter ();
  
  var isThunderbird = false;
  try {
    var id
      = Components.classes ["@mozilla.org/xre/app-info;1"]
      .getService (Components.interfaces.nsIXULAppInfo).ID;
    if (id == "{3550f703-e582-4d05-9a08-453d09bdfdc6}") {
      /* Thunderbird */
      isThunderbird = true;
    }
  }
  catch (e) {
  }
    
  if (isThunderbird) {
    /* Thunderbird */
    this.protocolHandler = {
      enUnMHTURI : function (uri) {
        return uri;
      },
      isUnMHTURI : function (uri) {
        return false;
      },
      deUnMHTURI : function (uri) {
        return uri;
      },
      getUnMHTURIParam : function (uri) {
        return null;
      }
    };
  }
  else {
    this.protocolHandler
      = Components.classes ["@mozilla.org/network/protocol;1?name=unmht"]
      .getService (Components.interfaces.arIUnMHTProtocolHandler);
  }
}
UnMHTExtractor.prototype = {
  protocolHandler : null,  /* arUnMHTMhtmlProtocolHandler
                            *   unmht プロトコルハンドラ */
  
  conv : null,           /* UnMHTConverter 文字列変換器 */
    
  original : "",         /* String  展開したファイル名の URI */
  rootURI : "",          /* String  ルートドキュメントの URI */
    
  redirect : "",         /* String  最初のドキュメントを置き換える場合の参照名 */
    
  subject : "",          /* String  Subject の値 */
  date : "",             /* String  Date の値 */
  headerMap : null,      /* Object  ヘッダ
                          *   <String フィールド名, String フィールド値> */
    
  start : "",            /* String  ルートドキュメントの CID */
    
  rootFile : null,       /* UnMHTExtractorFile  ルートドキュメントのファイル */
  alternative : false,   /* Boolean  multipert/alternative に入ったか */
  files : null,          /* Array  mht ファイルの中にあるファイル
                          *   [UnMHTExtractorFile ファイル情報,
                          *    ...] */
  refFiles : null,       /* Object  参照できるファイル
                          *   <String 参照用のファイル名,
                          *    UnMHTExtractorFile ファイル情報> */
    
  firstFile : null,      /* UnMHTExtractorFile  MHT ファイルの
                          *   直下のファイル */
    
  extractTime : 0,       /* Number  展開時刻 */
  extractName : "",      /* String  展開先のディレクトリの名前 */
  lastAccessTime : 0,    /* Number  最終アクセス時刻 */
    
  count : 0,             /* Number  展開したファイルの数 */
  size : 0,              /* Number  全キャッシュのサイズ */
  hasCache : false,      /* Boolean mht のキャッシュを持っているか */
  cacheFilename : "",    /* String  mht のキャッシュのファイル名 */
  cacheSize : 0,         /* Number  mht のキャッシュのサイズ */
    
  retcode : "",          /* String  mht ファイルの改行コード */
  headerRegExp : null,   /* RegExp  ヘッダを分割する正規表現 */
  valuesRegExp : null,   /* RegExp  ヘッダの値を分割する正規表現 */
    
  /**
   * mht ファイルを展開する
   *
   * @param  String originalURISpec
   *         展開したファイル名の URI 表記
   * @param  String text
   *         mht ファイルの内容
   * @param  Boolean utf16
   *         text が UTF-16 であるかどうか
   * @param  String extractName
   *         展開先のディレクトリ名
   *           一旦ダウンロードする場合には先に作られているので
   *           その値を渡す
   *           "" の場合は自動的に作られる
   * @param  Boolean extract
   *         展開するかどうか
   *           false の場合、text で与えられた内容のファイルを作る
   * @return String
   *         Firefox 版
   *           ルートドキュメントの URL
   *         Opera/Safari 版
   *           ファイルの data 形式の URI
   *           展開できなかった場合は ""
   */
  extractMHT : function (originalURISpec, text, utf16,
                         extractName, extract) {
    if (UnMHT.mode == 1) {
      this.original = originalURISpec
      .replace (/\(/g, "%28")
      .replace (/\)/g, "%29")
      .replace (/\'/g, "%27")
      .replace (/\"/g, "%22")
      .replace (/\[/g, "%5B")
      .replace (/\]/g, "%5D")
      .replace (/ /g, "%20");
            
      this.rootURI
      = this.protocolHandler.enUnMHTURI (this.original);
            
      this.original
      = this.protocolHandler.deUnMHTURI (this.rootURI);
      
      this.extractTime = (new Date ()).getTime ();
                
      /* UnMHT.enableNolimit を取得する */
      if (UnMHT.config.isObservable ()) {
        /* 監視していない場合にのみ設定を取得する */
        UnMHT.getConfigurationFromPreferences ();
      }
            
      if (UnMHT.enableNolimit) {
        UnMHT.config.setTime (UnMHT.nolimitTime);
      }
            
      /* 一定時間以上経ったキャッシュを削除する */
      UnMHT.clearCache (UnMHT.cacheExpire, false);
            
      if (!UnMHT.memoryCacheMode) {
        var extractDirectory = "";
        if (extractName == "") {
          /* 展開先のディレクトリを作る */
          extractName = UnMHT.createExtractName ();
                    
          extractDirectory
            = UnMHTFileManager.cacheDirectory
            + UnMHTFileManager.separator + extractName;
          UnMHTFileManager.createDirectory (extractDirectory);
        }
        else {
          extractDirectory
            = UnMHTFileManager.cacheDirectory
            + UnMHTFileManager.separator + extractName;
        }
                
        this.extractName = extractName;
      }
    }
    else if (UnMHT.mode == 2) {
      this.original = escape (originalURISpec);
      this.rootURI = this.original;
    }
        
    try {
      this.rootFile = null;
      this.alternative = false;
            
      if (extract) {
        /* 展開する */
                
        /* 改行コードのチェック */
        this.retcode = "";
        if (text.search (/(\r\n|\r|\n)/) != -1) {
          this.retcode = RegExp.$1;
          this.headerRegExp
            = new RegExp ("([^:]+): *([^" + this.retcode + "]*"
                          + this.retcode
                          + "([\t ]+[^" + this.retcode + "]*"
                          + this.retcode + ")*)",
                          "g");
          this.valuesRegExp
            = new RegExp ("[\t ]*([^" + this.retcode + "]*)"
                          + this.retcode,
                          "g");
        }
                
        if (this.retcode) {
          this.files = new Array ();
          this.refFiles = new Object ();
                    
          this.firstFile = this.parseMHT (text, utf16, null);
        }
      }
      
      if (this.rootFile == null) {
        /* HTML が含まれない場合 */
        
        if (this.files
            && this.files.length > 0) {
          this.rootFile = this.files [0];
        }
      }
            
      if (this.rootFile != null) {
        /* 解析に成功した場合 */
                
        /* 無効なファイルを削除する */
        this.deleteInvalidFile ();
                
        /* 参照名を重複しないように設定する */
        this.setRefName ();
                
        if (UnMHT.enableFilterExtract) {
          /* フィルタをかける設定の場合 */
          this.applyFilter ();
        }
        
        /* 内容をデコードする */
        this.decodeContent (utf16);
                
        /* URI を置き換える */
        this.modifyAttribute ();
        
        if (UnMHT.mode == 1) {
          if (!UnMHT.enableFastExtract) {
            /* CSS 中の URI を置き換える */
            this.replaceCSS ();
          }
        }
                
        if (UnMHT.mode == 2) {
          /* 文字コードを変換する */
          var charset = "";
          if (text.search (new RegExp ("[\u0100-\uffff]")) != -1) {
            text = this.conv.btoa16 (this.rootFile.content);
            charset = "charset=utf-16;";
          }
          else {
            text = this.conv.btoa8 (this.rootFile.content);
            if (this.rootFile.charset) {
              charset = "charset=" + this.rootFile.charset + ";";
            }
          }
                    
          return "data:text/html;"
          + charset
          + "unmht=" + this.rootURI + ";"
          + "base64," + text;
        }
      }
      else {
        /* 解析に失敗した場合、もしくは展開しなかった場合 */
        if (UnMHT.mode == 1) {
          var file = new UnMHTExtractorFile ();
                    
          if (extract) {
            /* 展開するはずならば、中身が HTML のファイルとみなす */
            file.extension = ".html";
            file.refName = "index.html";
            file.filetype = "text/html";
          }
          else {
            /* mht のソースを出力する */
            file.extension = ".txt";
            file.refName = "index.txt";
            file.filetype = "text/plain";
          }
                    
          if (utf16) {
            text
            = this.conv.convertFromUnicode (text,
                                            "shift_jis");
          }
                    
          this.rootFile = file;
          file.location = originalURISpec;
          file.cid = "";
          file.encoding = 0;
          file.charset = "shift_jis";
          file.content = text;
          this.splitLocation (file);
                    
          this.files = new Array ();
          this.refFiles = new Object ();
          this.files.push (file);
                    
          this.firstFile = file;
        }
        else {
          return "";
        }
      }
            
      /* ファイルを作成する */
      this.createFile (extractDirectory);
    }
    catch (e) {
      throw e;
    }
        
    if (UnMHT.enableNolimit) {
      UnMHT.config.restoreTime ();
    }
        
    this.lastAccessTime = (new Date ()).getTime ();
        
    return this.rootURI + this.redirect;
  },
    
  /**
   * mht ファイルを解析する
   *
   * @param  String text
   *         mht ファイルの内容
   * @param  Boolean utf16
   *         text が UTF-16 であるかどうか
   * @param  UnMHTExtractorFile parentFile
   *         親のファイル
   * @return UnMHTExtractorFile
   *         直下のファイル
   */
  parseMHT : function (text, utf16, parentFile) {
    var start_pos, end_pos;
    
    start_pos = 0;
        
    /* ヘッダの終わりの検索 */
    end_pos = text.indexOf (this.retcode + this.retcode, start_pos);
    if (end_pos == -1) {
      /* ヘッダが存在しない */
      return null;
    }
        
    var header = text.substr (start_pos,
                              end_pos + this.retcode.length - start_pos);
    var body = text.substr (end_pos + this.retcode.length * 2);
    
    /* ヘッダを解析するだけなので代入はしない */
    var headerMap = new Object ();
    var headerList = new Array ();
    header.replace
    (this.headerRegExp,
     (function (self) {
       return function (matched, name, tmp) {
         var value = "";
         /* 値を解析するだけなので代入はしない */
         tmp.replace
           (self.valuesRegExp,
            function (matched, part1) {
             value += self.conv.decode_ew (part1, utf16);
                    
             return "";
           });
         headerMap [name.toLowerCase ()] = value;
         headerList.push ([name, value]);
            
         return "";
       };
     })(this));
    
    var file = this.parseHeader (headerMap, headerList, parentFile);
    if (file == null) {
      /* ヘッダが異常な場合、終了する */
      return null;
    }
    if (file.filetype.match (/^multipart\//i)) {
      /* multipart の場合 */
      if (file.filetype.match (/^multipart\/alternative/i)) {
        this.alternative = true;
      }
      var b = file.boundary
      .replace (/([\\\[\]\(\)\^\,\.\{\}\|\?\!\-\*\+\$])/g,"\\$1");
      var boundaryRegExp = new RegExp ("--"
                                       + b
                                       + "(--)?(" + this.retcode + ")?");
      var bodies = body.split (boundaryRegExp);
      file.childFiles = new Array ();
      for (var i = 1; i < bodies.length - 1; i ++) {
        /* 最初と最後は boundary に挟まれていないので無効 */
        if (bodies [i]) {
          var file2 = this.parseMHT (bodies [i], utf16, file);
          if (file2) {
            file.childFiles.push (file2);
          }
        }
      }
      this.alternative = false;
            
      return file;
    }
        
    file.content = body;
        
    if (file.filetype == "application/octet-stream") {
      /* CGI の応答等の HTML かもしれない */
      var tmp;
      if (file.content.length > 32) {
        tmp = file.content.substr (0, 32);
      }
      else {
        tmp = file.content;
      }
      if (tmp.search (/</) != -1) {
        /* 先頭がタグっぽい */
        if (file.content.search (/<html/i) != -1) {
          file.filetype = "text/html";
        }
      }
    }
        
    return file;
  },
    
  /**
   * ヘッダを解析して UnMHTExtractorFile オブジェクトを作成する
   *
   * @param  Object headerMap
   *         ヘッダ
   *           <String フィールド名, String フィールド値>
   * @param  Array headerList
   *         ヘッダ (小文字化しない)
   *           [[String フィールド名, String フィールド値], ...]
   * @param  UnMHTExtractorFile parentFile
   *         親のファイル
   * @return UnMHTEnxtractorFile
   *         作成されたファイル情報
   */
  parseHeader : function (headerMap, headerList, parentFile) {
    var i;
    var name, value, valueMap;
    var value2Lower;
    var file;
    var boundary = "";
        
    file = new UnMHTExtractorFile ();
    file.parentFile = parentFile;
        
    file.charset = "";
    file.headerMap = headerMap;
    file.headerList = headerList;
        
    /* Content-Type を解析 */
    if ("content-type" in file.headerMap) {
      value
        = this.conv
        .convertFromUnicode (file.headerMap ["content-type"],
                             "shift_jis");
    }
    else {
      value = "text/html";
    }
    valueMap = this.parseValue (value, true);
    for (name in valueMap) {
      if (name == "_") {
        file.filetype = valueMap [name].toLowerCase ();
      }
      if (name == "charset") {
        file.charset = valueMap [name];
      }
      else if (name == "boundary") {
        boundary = valueMap [name];
      }
      else if (name == "start") {
        this.start
          = valueMap [name].replace (/^</, "").replace (/>$/, "");
      }
    }
    if (file.filetype != "") {
      /* Subject を解析 */
      if ("subject" in file.headerMap) {
        this.subject = file.headerMap ["subject"];
      }
        
      /* Date を解析 */
      if ("date" in file.headerMap) {
        this.date = file.headerMap ["date"];
      }
      
      /* Content-Base を解析 */
      if ("content-base" in file.headerMap) {
        file.contentBase = file.headerMap ["content-base"];
      }
      else {
        file.contentBase = undefined;
      }
      if ("content-location" in file.headerMap) {
        file.contentLocation = file.headerMap ["content-location"];
      }
      else {
        file.contentLocation = undefined;
      }
      if (file.contentBase) {
        if (file.contentLocation) {
          file.location
          = UnMHTFileManager.jointPath
          (file.contentBase, file.location)
        }
        else {
          file.location = file.contentBase;
        }
      }
      else {
        file.location = file.contentLocation;
      }
      
      if (file.location) {
        if (UnMHT.mode == 1) {
          file.location
          = file.location.replace
          (/^file:(\/\/)?(\/\/)?(\/)?/,
           function (matched, sep1, sep2, sep3) {
            var n = 0;
            if (sep1) {
              n |= 1;
            }
            if (sep2) {
              n |= 2;
            }
            if (sep3) {
              n |= 4;
            }
            return "unmht:///dummy." + n + "/";
          });
        }
        else if (UnMHT.mode == 2) {
          file.location
          = file.location.replace (/^file:\/\/(\/\/)?(\/)?/,
                                   "data:text/plain,dummy:");
        }
      }
      
      if (file.location == undefined) {
        /* ルートドキュメントの Content-Location が
         * 指定されない場合があるので index.html とする */
        file.location = "";
        
        var loc = "";
        var f;
        f = parentFile;
        while (f) {
          if (f.location) {
            loc = f.location;
            break;
          }
          f = f.parentFile;
        }
        
        if (loc) {
          file.location = loc;
          this.splitLocation (file);
          file.location = undefined;
        }
        else {
          if (file.filetype == "text/html") {
            file.location = "index.html";
          }
        }
      }
      
      if (!parentFile) {
        this.location = file.location;
      }
      
      if (file.location) {
        this.splitLocation (file);
      }
      
      /* Content-Disposition を解析 */
      file.disposition = "";
      if ("content-disposition" in file.headerMap) {
        var disposition = file.headerMap ["content-disposition"];
        if (disposition != undefined
            && disposition.toLowerCase ()
            != "inline") {
                        
          valueMap
            = this.parseValue (disposition,
                               false);
                        
          for (name in valueMap) {
            if (name == "_") {
              file.dispositionType = valueMap [name];
            }
            else if (name == "filename") {
              file.disposition = valueMap [name];
            }
          }
        }
      }
                
      /* Content-ID を解析 */
      if ("content-id" in file.headerMap) {
        file.cid = file.headerMap ["content-id"];
        if (file.cid) {
          file.cid = file.cid.replace (/^<([^>]+)>$/, "$1");
        }
      }
      else {
        file.cid = "";
      }
                
      /* ファイル名を調べる */
      var leafName = "";
      if (file.disposition == "") {
        /* Content-Disposition が存在しない場合 */
                    
        leafName = file.leafName;
      }
      else {
        /* Content-Disposition が存在する場合 */
                    
        leafName = file.disposition;
      }
        
      var leafName2 = leafName.replace (/\?.+$/, "");
      if (leafName2.match (/(\.[A-Za-z0-9]+)$/)) {
        /* ファイル名に拡張子が付いている場合はそれを付ける */
        file.extension = RegExp.$1;
      }
      else {
        file.extension = "";
      }
      file.referredBaseDir = "";
        
      if (file.filetype.match (/^multipart\//i)) {
        /* マルチパート */
        if (boundary) {
          file.boundary = boundary;
        }
        else {
          /* ヘッダが異常 */
          return null;
        }
      }
      else {
        /* 通常のファイルならば */
                
        /* Content-Transfer-Encoding を解析する */
        if ("content-transfer-encoding" in file.headerMap) {
          value = file.headerMap ["content-transfer-encoding"];
          value
          = this.conv.convertFromUnicode (value,
                                          "shift_jis");
        }
        else {
          value = "";
        }
        var value2Lower = value.toLowerCase ();
        if (value2Lower == "quoted-printable") {
          file.encoding = 1;
        }
        else if (value2Lower == "base64") {
          file.encoding = 2;
        }
        else if (value2Lower == "7bit") {
          file.encoding = 3;
        }
        else {
          file.encoding = 0;
        }
        
        if (file.filetype == "text/html") {
          if ((this.rootFile == null
               || this.alternative)
              || (this.start && this.start == file.cid)) {
            /* 最初の HTML ファイルか、指定された Content-ID ならば、
             * これをルートドキュメントとする */
            this.rootFile = file;
            this.alternative = false;
          }
        }
                
        this.files.push (file);
      }
    }
        
    return file;
  },
    
  /**
   * Content-Type/Content-Disposition の値を解析する
   *
   * @param  String value
   *         Content-Type/Content-Disposition の値
   * @param  Boolean mimeType
   *         値のみのフィールドは MIME Type か
   * @return Object
   *         解析した値のマップ
   *           <String 名前, String 値>
   */
  parseValue : function (value, mimeType) {
    var parts;
    var i;
    var map;
    var name;
    var tmp;
    var realName, realValue;
        
    map = new Object ();
        
    /* 値を解析するだけなので代入はしない */
    value
    .replace (/[ \t]*=[ \t]*/g, "=")
    .replace
    (/(?:\"(?:\\\"|[^\"])*\"|\'(?:\\\'|[^\'])*\'|[^\"\'\\ \t;])+/g,
     function (matched) {
      if (matched.match (/^([^=]*)=(.*)$/)) {
        name = RegExp.$1.toLowerCase ();
        value = RegExp.$2;
        if (value.match (/^[\"\'](.*)[\"\']$/)) {
          value = RegExp.$1;
        }
        map [name] = value;
      }
      else if ((matched.indexOf ("/") != -1
                || !mimeType)
               && matched.indexOf ("=") == -1) {
        map ["_"] = matched;
      }
      return "";
    });
    
    for (var name in map) {
      if (name.match (/^([^\*]+)\*(([0-9]+))?\*?$/)) {
        /* RFC2231 */
                
        realName = RegExp.$1;
        if (RegExp.$2) {
          /* 複数の値 */
          if (RegExp.$3 == "0") {
            /* 最初の値 */
                        
            realValue = "";
            i = 0;
            for (;;) {
              if ((realName + "*" + i + "*") in map) {
                /* エンコードあり */
                tmp = map [realName + "*" + i + "*"];
                if (tmp.match (/^([^\']+)\'([^\']+)?\'(.+)$/)) {
                  try {
                    tmp
                      = this.conv.convertToUnicode
                      (unescape (RegExp.$3), RegExp.$1);
                  }
                  catch (e) {
                    tmp = unescape (RegExp.$3);
                  }
                }
                realValue += tmp;
              }
              else if ((realName + "*" + i) in map) {
                /* エンコードなし */
                realValue += map [realName + "*" + i];
              }
              else {
                break;
              }
              i ++;
            }
          }
          else {
            continue;
          }
        }
        else {
          /* 単体でエンコードあり */
          tmp = map [name];
          if (tmp.match (/^([^\']+)\'([^\']+)?\'(.+)$/)) {
            try {
              tmp
                = this.conv.convertToUnicode
                (unescape (RegExp.$3), RegExp.$1);
            }
            catch (e) {
              tmp = unescape (RegExp.$3);
            }
          }
          realValue = tmp;
        }
                
        map [realName] = realValue;
      }
    }
        
    return map;
  },
    
  /**
   * 無効なファイルを削除する
   */
  deleteInvalidFile : function () {
    var file;
        
    for (var i = 0; i < this.files.length; i ++) {
      file = this.files [i];
      if (file.location == ""
          && file.content == "") {
        this.files.splice (i, 1);
        i --;
      }
    }
  },
    
  /**
   * 参照名を重複しないように設定する
   */
  setRefName : function () {
    var refName, refBase;
    var i, j, suffix;
    var file, file2;
    
    for (i = 0; i < this.files.length; i ++) {
      file = this.files [i];
            
      var leafName = file.leafName;
      var p = leafName.indexOf ("?");
      if (p != -1) {
        leafName = leafName.substr (0, p);
      }
      if (leafName) {
        if (leafName.match (/([^\/\\:\!]+)(\.[A-Za-z0-9]+)$/)) {
          refBase = RegExp.$1;
        }
        else {
          refBase = file.leafName;
        }
      }
      else if (file.cid) {
        refBase = file.cid;
      }
      else if (file.filetype == "text/html") {
        refBase = "index";
      }
      else {
        refBase = i;
      }
      refBase = escape (refBase).replace (/\//g, "_");
            
      suffix = 0;
      file.refName = "";
      while (file.refName == "") {
        if (suffix == 0) {
          refName = refBase + file.extension;
        }
        else {
          refName = refBase + "." + suffix + file.extension;
        }
                
        for (j = 0; j < i; j ++) {
          file2 = this.files [j];
                    
          if (refName == file2.refName) {
            break;
          }
        }
        if (j == i) {
          file.refName = refName;
        }
        suffix ++;
      }
    }
  },
    
  /**
   *  フィルタをかける
   */
  applyFilter : function () {
    var uri
    = Components.classes ["@mozilla.org/network/standard-url;1"]
    .createInstance (Components.interfaces.nsIURI);
            
    for (var i = 0; i < this.files.length; i ++) {
      var file = this.files [i];
            
      if (file.deleted) {
        continue;
      }
      
      for (var j = 0; j < UnMHT.filterExtractList.length; j ++) {
        var text = "";
        if (UnMHT.filterExtractList [j][0] == "host") {
          try {
            uri.spec = file.location;
                        
            text = uri.host;
          }
          catch (e) {
            text = "";
          }
        }
        else if (UnMHT.filterExtractList [j][0] == "leafname") {
          text = file.leafName;
        }
        else if (UnMHT.filterExtractList [j][0] == "extension") {
          text = file.extension;
        }
        else if (UnMHT.filterExtractList [j][0] == "location") {
          text = file.location;
        }
        else if (UnMHT.filterExtractList [j][0] == "mimetype") {
          text = file.filetype;
        }
                
        var pos = -1;
        if (UnMHT.filterExtractList [j][2]) {
          pos = text.search (UnMHT.filterExtractList [j][1]);
        }
        else {
          pos = text.indexOf (UnMHT.filterExtractList [j][1]);
        }
                
        if (pos != -1) {
          /* フィルタに引っかかった場合、展開しない設定にする */
          file.deleted = true;
          file.extension = "";
          file.encoding = 0;
          file.content = "";
          break;
        }
      }
    }
  },
    
  /**
   * 内容をデコードする
   *
   * @param  Boolean utf16
   *         text が UTF-16 であるかどうか
   */
  decodeContent : function (utf16) {
    var i;
    var file;
        
    for (i = 0; i < this.files.length; i ++) {
      file = this.files [i];
            
      if (file.deleted) {
        continue;
      }
            
      if (file.encoding == 1) {
        file.content
          = this.conv.decode_qp (file.content);
      }
      else if (file.encoding == 2) {
        if (file.content.length % 4 != 0) {
          file.content
            = file.content.substr
            (0, parseInt (file.content.length / 4) * 4);
        }
        if (UnMHT.mode == 1) {
          try {
            file.content
              = atob
              (file.content.replace(/[^A-Za-z0-9+\/=]+/g, ""));
          }
          catch (e) {
            file.content
              = this.conv.atob
              (file.content.replace(/[^A-Za-z0-9+\/=]+/g, ""));
          }
        }
        else if (UnMHT.mode == 2) {
          if (file == this.rootFile) {
            file.content
              = this.conv.atob
              (file.content.replace(/[^A-Za-z0-9+\/=]+/g, ""));
          }
        }
      }
      else if (utf16) {
        if (file.filetype == "text/html"
            && file.charset) {
          file.content
            = this.conv
            .convertFromUnicode (file.content, file.charset);
        }
      }
      file.originalContent = file.content;
    }
  },
  
  cssImportRe : /(@import[ \t\r\n]*\(?[ \t\r\n]*)(\"(?:\\\"|[^\"])*\"|\'(?:\\\'|[^\'])*\'|[^\"\'\\;\)]+)/ig,
  /**
   * Style 要素内の CSS を変換する
   *
   * @param  UnMHTExtractorFile file
   *         対象のファイル
   * @param  String matched
   *         マッチした全体
   *           prev + path
   * @param  String prev
   * @param  String path
   *         以下参照
   *         path
   *         -----------
   * @import "12345.jpg";
   * --------
   * prev
   *          path
   *          -----------
   * @import ("12345.jpg");
   * ---------
   * prev
   * @return String
   *         変換した結果
   */
  cssImportFunc : function (file,
                            matched, prev, path) {
    var next = "";
    if (path.match (/^\"(.*)\"$/)) {
      path = RegExp.$1;
      prev += "\"";
      next = "\"";
    }
    else if (path.match (/^\'(.*)\'$/)) {
      path = RegExp.$1;
      prev += "\'";
      next = "\'";
    }
    path = path.replace (/^ +/, "").replace (/ +$/, "");
    
    if (path.match (/^[\'\";\+]/)) {
      return matched;
    }
    
    if (!path.match (/^[a-z]+:/)) {
      /* 相対パスは newURI の方で変える */
      return matched;
    }
    
    if (UnMHT.mode == 1) {
      path
      = path
      .replace (/^file:\/\/\/\/\//, "unmht:///dummy.7/")
      .replace (/^file:\/\/\//, "unmht:///dummy.5/")
      .replace (/^file:\/\//, "unmht:///dummy.1/")
      .replace (/^cid:/, "unmht:///cid.0/");
    }
    else {
      path
      = path
      .replace (/^file:\/\/\/\/\//, "data:text/plain,dummy:")
      .replace (/^file:\/\/\//, "data:text/plain,dummy:")
      .replace (/^file:\/\//, "data:text/plain,dummy:")
      .replace (/^cid:/, "data:text/plain,cid:");
    }
        
    if (path.match (/^url/)) {
      /* 変換済み */
      return matched;
    }
    var list
    = UnMHT.checkFile (this, file, path, false);
        
    if (list [0] != -1) {
      /* 含まれていた場合 */
      if (list [1]) {
        /* 展開した場合、参照関係を設定してパスを置き換える */
        list [1].referredBaseDir = file.baseDir;
        if (UnMHT.mode == 1) {
          return prev + this.rootURI + list [1].refName + next;
        }
      }
      else {
        /* 展開しなかった場合、削除する */
        return prev + next;
      }
    }
    /* 含まれていない場合 */
        
    if (this.protocolHandler.isUnMHTURI (path)) {
      var param = this.protocolHandler.getUnMHTURIParam (path);
      if (param.protocol == "dummy") {
        /* 参照先はローカルファイルの場合、削除する */
        return prev + next;
      }
    }
                            
    if (path.match (/^[A-Za-z0-9\-]+:/)) {
      /* 絶対パスの場合そのままにする */
      return matched;
    }
        
    if (file.baseDir == "") {
      /* 元のアドレスが不明な場合、削除する */
      return prev + next;
    }
        
    /* 相対パスの場合、元の場所へ取りに行くようにする */
    var dir
    = UnMHT.conv.convertFromUnicode (file.baseDir, file.charset);
    return prev
    + UnMHTFileManager.jointPath (dir, path)
    + next;
  },
  
  unCssImportFunc : function (file,
                              matched, prev, path) {
    var next = "";
    if (path.match (/^\"(.*)\"$/)) {
      path = RegExp.$1;
      prev += "\"";
      next = "\"";
    }
    else if (path.match (/^\'(.*)\'$/)) {
      path = RegExp.$1;
      prev += "\'";
      next = "\'";
    }
    path = path.replace (/^ +/, "").replace (/ +$/, "");
    
    if (this.protocolHandler.isUnMHTURI (path)) {
      var param = this.protocolHandler.getUnMHTURIParam (path);
      var file2 = null;
      if (param.refName) {
        if (param.refName in this.refFiles) {
          file2 = this.refFiles [param.refName];
        }
      }
      else {
        file2 = this.rootFile;
      }
      
      if (file2) {
        var location = file2.location
        .replace
        (/^unmht:\/\/([^\/]*)\/([^\.]+)\.([0-9]+)\//,
         function (matched, host, proto, num) {
          var sep1;
          var sep2;
          var sep3;
          if (num & 1) {
            sep1 = "//";
          }
          if (num & 2) {
            sep2 = "//";
          }
          if (num & 4) {
            sep3 = "/";
          }
          return proto + ":" + sep1 + host + sep2 + sep3;
        });
        
        return prev + location + next;
      }
      else {
        return prev + next;
      }
    }
    
    return matched;
  },
  
  cssUrlRe : /([^a-zA-Z0-9_]url[ \r\n\t]*\([ \r\n\t]*)(\"(?:\\\"|[^\"])*\"|\'(?:\\\'|[^\'])*\'|[^\"\'\\\)]+)/ig,
  /**
   * Style 要素内の CSS を変換する
   *
   * @param  UnMHTExtractorFile file
   *         対象のファイル
   * @param  String matched
   *         マッチした全体
   *           prev + path
   * @param  String prev
   * @param  String path
   *         以下参照
   *                        path
   *                        -----------
   * background-image: url ("12345.jpg");
   *                   -----
   *                   prev
   * @param  Boolean isCSS
   *         CSS ファイルかどうか
   *         CSS ファイルの場合 referredBaseDir をチェックする
   * @return String
   *         変換した結果
   */
  cssUrlFunc : function (file,
                         matched, prev, path, isCSS) {
    var next = "";
    if (path.match (/^\"(.*)\"$/)) {
      path = RegExp.$1;
      prev += "\"";
      next = "\"";
    }
    else if (path.match (/^\'(.*)\'$/)) {
      path = RegExp.$1;
      prev += "\'";
      next = "\'";
    }
    else if (path.match (/^&quot;(.*)&quot;$/)) {
      path = RegExp.$1;
      prev += "&quot;";
      next = "&quot;";
    }
    else if (path.match (/^&apos;(.*)&apos;$/)) {
      path = RegExp.$1;
      prev += "&apos;";
      next = "&apos;";
    }
    else if (path.match (/^&#39;(.*)&#39;$/)) {
      path = RegExp.$1;
      prev += "&#39;";
      next = "&#39;";
    }
    else if (path.match (/^&#x27;(.*)&#x27;$/)) {
      path = RegExp.$1;
      prev += "&#x27;";
      next = "&#x27;";
    }
    path = path.replace (/^ +/, "").replace (/ +$/, "");
    
    if (path.match (/^[\'\";\+]/)) {
      return matched;
    }
    
    if (!path.match (/^[a-z]+:/)) {
      /* 相対パスは newURI の方で変える */
      return matched;
    }
    
    if (UnMHT.mode == 1) {
      path
      = path
      .replace (/^file:\/\/\/\/\//, "unmht:///dummy.7/")
      .replace (/^file:\/\/\//, "unmht:///dummy.5/")
      .replace (/^file:\/\//, "unmht:///dummy.1/")
      .replace (/^cid:/, "unmht:///cid.0/");
    }
    else {
      path
      = path
      .replace (/^file:\/\/\/\/\//, "data:text/plain,dummy:")
      .replace (/^file:\/\/\//, "data:text/plain,dummy:")
      .replace (/^file:\/\//, "data:text/plain,dummy:")
      .replace (/^cid:/, "data:text/plain,cid:");
    }
        
    var list
    = UnMHT.checkFile (this, file, path, isCSS);
        
    if (list [0] != -1) {
      /* 含まれていた場合 */
      if (list [1]) {
        /* 展開した場合、参照関係を設定してパスを置き換える */
        list [1].referredBaseDir = file.baseDir;
        if (UnMHT.mode == 1) {
          return prev + this.rootURI + list [1].refName + next;
        }
      }
      else {
        /* 展開しなかった場合、削除する */
        return prev + next;
      }
    }
    /* 含まれていない場合 */
        
    if (this.protocolHandler.isUnMHTURI (path)) {
      var param = this.protocolHandler.getUnMHTURIParam (path);
      if (param.protocol == "dummy") {
        /* 参照先はローカルファイルの場合、削除する */
        return prev + next;
      }
    }
                            
    if (path.match (/^[A-Za-z0-9\-]+:/)) {
      /* 絶対パスの場合そのままにする */
      return matched;
    }
        
    if (file.baseDir == "") {
      /* 元のアドレスが不明な場合、削除する */
      return prev + next;
    }
        
    /* 相対パスの場合、元の場所へ取りに行くようにする */
    var dir
    = UnMHT.conv.convertFromUnicode (file.baseDir, file.charset);
    return prev
    + UnMHTFileManager.jointPath (dir, path)
    + next;
  },
    
  unCssUrlFunc : function (file,
                         matched, prev, path, isCSS) {
    var next = "";
    if (path.match (/^\"(.*)\"$/)) {
      path = RegExp.$1;
      prev += "\"";
      next = "\"";
    }
    else if (path.match (/^\'(.*)\'$/)) {
      path = RegExp.$1;
      prev += "\'";
      next = "\'";
    }
    path = path.replace (/^ +/, "").replace (/ +$/, "");
    
    if (this.protocolHandler.isUnMHTURI (path)) {
      var param = this.protocolHandler.getUnMHTURIParam (path);
      var file2 = null;
      if (param.refName) {
        if (param.refName in this.refFiles) {
          file2 = this.refFiles [param.refName];
        }
      }
      else {
        file2 = this.rootFile;
      }
      
      if (file2) {
        var location = file2.location
        .replace
        (/^unmht:\/\/([^\/]*)\/([^\.]+)\.([0-9]+)\//,
         function (matched, host, proto, num) {
          var sep1;
          var sep2;
          var sep3;
          if (num & 1) {
            sep1 = "//";
          }
          if (num & 2) {
            sep2 = "//";
          }
          if (num & 4) {
            sep3 = "/";
          }
          return proto + ":" + sep1 + host + sep2 + sep3;
        });
        
        return prev + location + next;
      }
      else {
        return prev + next;
      }
    }
    
    return matched;
  },
  
  /**
   * CSS 中の URI を置き換える
   */
  replaceCSS : function () {
    var i;
    var file;
        
    for (i = 0; i < this.files.length; i ++) {
      file = this.files [i];
                
      if (file.deleted) {
        continue;
      }
      
      if (file.filetype.substr (0, 6) != "image/") {
        var isCSS = (file.filetype == "text/css");
                
        file.content
          = file.content
          .replace
          (this.cssUrlRe,
           (function (self) {
             return function (matched, prev, path) {
               return self.cssUrlFunc
               (file,
                matched, prev, path, isCSS);
             };
           })(this))
          .replace
          (this.cssImportRe,
           (function (self) {
             return function (matched, prev, path) {
               return self.cssImportFunc
               (file,
                matched, prev, path);
             };
           })(this));
      }
    }
  },
    
  elementModRe : /(<(img)[ \r\n\t](?:\"(?:\\\"|[^\"])*\"|\'(?:\\\'|[^\'])*\'|[^\"\'\\>])*?(__unmht_modified)[ \r\n\t]*=[ \r\n\t]*)(\"(?:\\\"|[^\"])*\"|\'(?:\\\'|[^\'])*\'|[^\"\'\\ >]+)/ig,
    
  /**
   * unmht_modified 属性を削除する
   *
   * @param  UnMHTExtractorFile file
   *         対象のファイル
   * @param  String matched
   *         マッチした全体
   *           prev + path + next
   * @param  String prev
   * @param  String name
   * @param  String attr
   * @param  String path
   *         以下参照
   *  name            attr path
   *  ---             --- -----------
   * <img width="100" src="12345.png"  />
   * ---------------------
   * prev
   */
  elementModFunc : function (file,
                             matched, prev, name, attr, path) {
    var next = "";
    if (path.match (/^\"(.*)\"$/)) {
      path = RegExp.$1;
      prev += "\"";
      next = "\"";
    }
    else if (path.match (/^\'(.*)\'$/)) {
      path = RegExp.$1;
      prev += "\'";
      next = "\'";
    }
    path = path.replace (/^ +/, "").replace (/ +$/, "");
        
    return prev + next;
  },
  
  elementRe : /(<([\?A-Za-z0-9_:\-]+)[ \r\n\t](?:\"(?:\\\"|[^\"])*\"|\'(?:\\\'|[^\'])*\'|[^\"\'\\>])*?(src|href|background|action)[ \r\n\t]*=[ \r\n\t]*)(\"(?:\\\"|[^\"])*\"|\'(?:\\\'|[^\'])*\'|[^\"\'\\ >]+)/ig,
    
  /**
   * HTML から各ファイルへの参照を変換する
   *
   * @param  UnMHTExtractorFile file
   *         対象のファイル
   * @param  String matched
   *         マッチした全体
   *           prev + path + next
   * @param  String prev
   * @param  String name
   * @param  String attr
   * @param  String path
   *         以下参照
   *  name            attr path
   *  ---             --- -----------
   * <img width="100" src="12345.png"  />
   * ---------------------
   * prev
   */
  elementFunc : function (file,
                          matched, prev, name, attr, path) {
    var next = "";
    if (path.match (/^\"(.*)\"$/)) {
      path = RegExp.$1;
      prev += "\"";
      next = "\"";
    }
    else if (path.match (/^\'(.*)\'$/)) {
      path = RegExp.$1;
      prev += "\'";
      next = "\'";
    }
    path = path.replace (/^ +/, "").replace (/ +$/, "");
    
    if (path.match (/^[\'\";\+]/)) {
      return matched;
    }
    
    if (UnMHT.mode == 1) {
      path
      = path
      .replace (/^file:\/\/\/\/\//, "unmht:///dummy.7/")
      .replace (/^file:\/\/\//, "unmht:///dummy.5/")
      .replace (/^file:\/\//, "unmht:///dummy.1/")
      .replace (/^cid:/, "unmht:///cid.0/");
    }
    else {
      path
      = path
      .replace (/^file:\/\/\/\/\//, "data:text/plain,dummy:")
      .replace (/^file:\/\/\//, "data:text/plain,dummy:")
      .replace (/^file:\/\//, "data:text/plain,dummy:")
      .replace (/^cid:/, "data:text/plain,cid:");
    }
    
    if (name.match (/base/i)) {
      var path2;
      path2
      = UnMHT.conv.convertToUnicode (path, file.charset);
      if (path2.match (/\//)) {
        path2 = path2.replace (/\/[^\/]+$/, "");
      }
      if (file.baseDir) {
        file.baseDir
        = UnMHTFileManager.jointPath (file.baseDir, path2);
      }
      else {
        file.baseDir = path2;
      }
    }
    
    var list
    = UnMHT.checkFile (this, file, path, false);
    
    if (list [0] != -1) {
      /* 含まれていた場合 */
      if (list [1]) {
        /* 展開した場合 */
        if (name == "link") {
          /* 参照関係を設定する */
          list [1].referredBaseDir = file.baseDir;
        }
                
        if (UnMHT.mode == 1) {
          return prev + this.rootURI + list [1].refName
            + list [2] + next;
        }
        else if (UnMHT.mode == 2) {
          return prev
            + "data:" + list [1].filetype + ";"
            + "unmht=" + this.rootURI + list [1].refName + ";"
            + "base64,"
            + list [1].content + next;
        }
      }
      else {
        /* 展開しなかった場合、削除する */
        return prev + next;
      }
    }
    /* 含まれていない場合 */
        
    if (path.match (/^file:\/\/\/?/)) {
      /* 参照先がローカルファイルの場合、削除する */
      return prev + next;
    }
                            
    if (path.match (/^[A-Za-z0-9\-]+:/)) {
      /* 絶対パスの場合そのままにする */
      return matched;
    }
    
    if (file.baseDir == "") {
      /* 元のアドレスが不明な場合、削除する */
      return prev + next;
    }
    
    if (file.baseDir.match (/^file:\/\/\/?/)
        || file.baseDir.match (/^\//)
        || file.baseDir.match (/^[A-Za-z]:\//)) {
      /* 参照先がローカルファイルの場合、削除する */
      return prev + next;
    }
        
    /* 相対パスの場合、元の場所へ取りに行くようにする */
    var dir
    = UnMHT.conv.convertFromUnicode (file.baseDir, file.charset);
    return prev
    + UnMHTFileManager.jointPath (dir, path)
    + next;
  },
  
  unElementFunc : function (file,
                            matched, prev, name, attr, path) {
    var next = "";
    if (path.match (/^\"(.*)\"$/)) {
      path = RegExp.$1;
      prev += "\"";
      next = "\"";
    }
    else if (path.match (/^\'(.*)\'$/)) {
      path = RegExp.$1;
      prev += "\'";
      next = "\'";
    }
    path = path.replace (/^ +/, "").replace (/ +$/, "");
    
    if (path.match (/^\'/)
      || path.match (/^\"/)) {
      return matched;
    }
    
    if (this.protocolHandler.isUnMHTURI (path)) {
      var param = this.protocolHandler.getUnMHTURIParam (path);
      var file2 = null;
      if (param.refName) {
        if (param.refName in this.refFiles) {
          file2 = this.refFiles [param.refName];
        }
      }
      else {
        file2 = this.rootFile;
      }
      
      if (file2) {
        var location = file2.location
        .replace
        (/^unmht:\/\/([^\/]*)\/([^\.]+)\.([0-9]+)\//,
         function (matched, host, proto, num) {
          var sep1;
          var sep2;
          var sep3;
          if (num & 1) {
            sep1 = "//";
          }
          if (num & 2) {
            sep2 = "//";
          }
          if (num & 4) {
            sep3 = "/";
          }
          return proto + ":" + sep1 + host + sep2 + sep3;
        });
        
        return prev + location + next;
      }
      else {
        return prev + next;
      }
    }
    
    return matched;
  },
    
  /**
   * URI を置き換える
   */
  modifyAttribute : function () {
    var isPPT = false;
    var isExistsFrame = false;
        
    for (var i = 0; i < this.files.length; i ++) {
      var file = this.files [i];
            
      if (file.deleted) {
        continue;
      }
            
      if (file.filetype != "text/css"
          && file.filetype.substr (0, 6) != "image/") {
        if (file.refName == "frame.htm") {
          isExistsFrame = true;
        }
        
        file.content
          = file.content
          .replace
          (this.elementModRe,
           (function (self) {
             return function (matched,
                              prev, name, attr, path) {
               return self.elementModFunc
               (file,
                matched, prev, name, attr, path);
             };
           })(this))
          .replace
          (this.elementRe,
           (function (self) {
             return function (matched, prev, name, attr, path) {
               return self.elementFunc
               (file,
                matched, prev, name, attr, path);
             };
           })(this));
                
        if (file.content.indexOf
            ("xmlns:v=\"urn:schemas-microsoft-com:vml\"") != -1) {
          file.content
            = file.content
            .replace (/<(o|p|oa|v):(?:\"(?:\\\"|[^\"])*\"|\'(?:\\\'|[^\'])*\'|[^\"\'\\>])*?>/g, "");
          isPPT = true;
        }
      }
    }
        
    if (isPPT && UnMHT.enableSkipPPTWarning && isExistsFrame) {
      this.redirect = "frame.htm";
    }
  },

  /**
   * URI の置き換えを元に戻す
   */
  unModifyAttribute : function (file, filetype, content) {
    if (filetype.substr (0, 6) != "image/") {
      if (filetype != "text/css") {
        content
        = content
        .replace
        (this.elementRe,
         (function (self) {
           return function (matched, prev, name, attr, path) {
             return self.unElementFunc
             (file,
              matched, prev, name, attr, path);
           };
         })(this));
      }
      
      var isCSS = (filetype == "text/css");
      
      content
      = content
      .replace
      (this.cssUrlRe,
       (function (self) {
         return function (matched, prev, path) {
           return self.unCssUrlFunc
           (file,
            matched, prev, path, isCSS);
         };
       })(this))
      .replace
      (this.cssImportRe,
       (function (self) {
         return function (matched, prev, path) {
           return self.unCssImportFunc
           (file,
            matched, prev, path);
         };
       })(this));
      
      return content;
    }
    else {
      return content;
    }
  },
    
  /**
   * 完了
   *
   * @param  String extractDirectory
   *         展開先のディレクトリ
   */
  createFile : function (extractDirectory) {
    var i;
    var file;
        
    for (i = 0; i < this.files.length; i ++) {
      file = this.files [i];
            
      if (file.deleted) {
        continue;
      }
            
      file.size = file.content.length
        if (!UnMHT.memoryCacheMode) {
          file.nativeFilename
          = extractDirectory + UnMHTFileManager.separator + i;
                
          UnMHTFileManager.createFile (file.nativeFilename,
                                       file.content);
        }
            
      this.refFiles [file.refName] = file;
            
      this.count ++;
      this.size += file.content.length;
    }
        
    if (!UnMHT.memoryCacheMode) {
      var cacheFile;
            
      var filename
      = UnMHTFileManager.cacheDirectory
      + UnMHTFileManager.separator + this.extractName
      + UnMHTFileManager.separator + "cache";
      cacheFile = UnMHTFileManager.initFile (filename);
      if (cacheFile.exists ()) {
        this.hasCache = true;
        this.cacheFilename = filename;
        this.cacheSize = cacheFile.fileSize;
                
        this.size += this.cacheSize;
      }
    }
  },
    
  /**
   * ファイルのアドレスを親ディレクトリとファイル名に分割する
   *
   * @param  UnMHTExtractorFile file
   *         対象のファイル情報
   */
  splitLocation : function (file) {
    var leafNameRe = null;
        
    var location = file.location;
    var postfix = "";
    
    var p = location.indexOf ("?");
    if (p != -1) {
      postfix = location.substr (p);
      location = location.substr (0, p);
    }
        
    if (location.indexOf ("mhtml:") != -1) {
      /* mhtml で始まる場合 (IE のプロトコル ?) */
            
      /* ! の前後に分割する */
      leafNameRe = /^(.*)\!([^\!]+)$/;
    }
    else if (location.indexOf ("\\") != -1) {
      /* Windows のパスの場合 */
      leagNameRe = /^(.*)\\([^\\]*)$/;
    }
    else if (location.indexOf ("/") != -1) {
      /* Mac OS X、Linux のパス、URI の場合 */
      leafNameRe = /^(.*)\/([^\/]*)$/;
    }
    else if (location.indexOf (":") != -1) {
      /* Mac OS 9 以前のパスの場合 */
      leafNameRe = /^(.*):([^:]*)$/;
    }
    
    if (leafNameRe != null) {
      /* 区切りが見付かった場合 */
      if (location.match (leafNameRe)) {
        /* ファイル名がある場合 */
        file.baseDir = RegExp.$1;
        file.leafName = RegExp.$2 + postfix;
      }
      else {
        /* ファイル名がない場合 */
        file.baseDir = location + postfix;
        file.leafName = "";
      }
    }
    else {
      /* ファイル名のみの場合 */
      file.baseDir = "";
      file.leafName = location + postfix;
    }
  }
};

