/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/**
 * ファイルの管理をする
 */
var UnMHTFileManager = {
  fileProtocolHandler : null, /* nsIFileProtocolHandler  プロトコルの変換器 */
    
  separator : "\\",           /* String  ネイティブパスのセパレータ */
  systemDirectory : "",       /* String  システムディレクトリ
                               *   Profile ディレクトリ/UnMHT となる */
  cacheDirectory : "",        /* String  キャッシュディレクトリ
                               *   Profile ディレクトリ/UnMHT/Cache となる */
    
  /**
   * 初期化
   */
  init : function () {
    UnMHTFileManager.fileProtocolHandler
    = Components.classes ["@mozilla.org/network/io-service;1"]
    .getService (Components.interfaces.nsIIOService)
    .getProtocolHandler ("file")
    .QueryInterface (Components.interfaces.nsIFileProtocolHandler);
        
    /* 各種ディレクトリを作る */
    UnMHTFileManager.makeSystemDirectory ();
    UnMHTFileManager.makeCacheDirectory ();
  },
    
  /**
   * 現在のユーザの Profile ディレクトリを取得する
   *
   * @return  String
   *          現在のユーザの Profile ディレクトリ
   */
  getProfileDirectory : function () {
    var dirname;
        
    try {
      dirname
        = Components.classes ["@mozilla.org/file/directory_service;1"]
        .getService (Components.interfaces.nsIProperties)
        .get ("ProfD", Components.interfaces.nsIFile).path;
    }
    catch (e) {
      dirname = "";
    }
        
    return dirname;
  },
    
  /**
   * システムディレクトリを作成する
   */
  makeSystemDirectory : function () {
    var dirname = UnMHTFileManager.getProfileDirectory ();
    if (dirname.indexOf ("\\") != -1) {
      UnMHTFileManager.separator = "\\";
    }
    else if (dirname.indexOf ("/") != -1) {
      UnMHTFileManager.separator = "/";
    }
    else if (dirname.indexOf (":") != -1) {
      UnMHTFileManager.separator = ":";
    }
    UnMHTFileManager.systemDirectory = dirname
    + UnMHTFileManager.separator + "UnMHT";
        
    UnMHTFileManager.createDirectory (UnMHTFileManager.systemDirectory);
  },
    
  /**
   * キャッシュディレクトリを作成する
   */
  makeCacheDirectory : function () {
    UnMHTFileManager.cacheDirectory
    = UnMHTFileManager.systemDirectory
    + UnMHTFileManager.separator + "Cache";
        
    UnMHTFileManager.createDirectory (UnMHTFileManager.cacheDirectory);
  },
    
  /**
   * キャッシュを削除する
   *
   * @param  Number expire
   *         ミリ秒単位の経過時間
   *         0 なら即削除
   * @param  Boolean deleteExtractor
   *         展開情報も削除するか
   */
  clearCacheDirectory : function (expire, deleteExtractor) {
    try {
      var dir
      = Components.classes ["@mozilla.org/file/local;1"]
      .createInstance (Components.interfaces.nsILocalFile);
      dir.initWithPath (UnMHTFileManager.cacheDirectory);
            
      var now = new Date ().getTime ();
            
      if (dir.exists ()) {
        var entries = dir.directoryEntries;
                
        while (entries.hasMoreElements ()) {
          var file = entries.getNext ();
          file.QueryInterface (Components.interfaces.nsILocalFile);
                    
          if (expire > 0) {
            var createTime
              = parseInt (file.leafName
                          .replace (/_[0-9]*$/g, ""));
                        
            if (now < createTime + expire) {
              /* 作成から指定時間経過していない */
              continue;
            }
          }
                    
          if (deleteExtractor) {
            var extractor = null;
            var delete_urispec = "";
            var extractName;
            for (var urispec in UnMHT.extractors) {
              extractName
                = UnMHT.extractors [urispec].extractName;
              if (extractName == file.leafName) {
                extractor = UnMHT.extractors [urispec];
                delete_urispec = urispec;
                break;
              }
            }
                        
            if (extractor) {
              if (now < extractor.lastAccessTime + expire) {
                /* 最終アクセスから指定時間経過していない */
                continue;
              }
                            
              UnMHT.extractors [delete_urispec] = null;
              delete UnMHT.extractors [delete_urispec];
            }
          }
          file.remove (true);
        }
      }
    }
    catch (e) {
    }
  },
    
  /**
   * nsILocalFile オプジェクトを作成する
   * ファイルの実体は作成しない
   *
   * @param  String filename
   *         ファイル名
   * @return nsILocalFile
   */
  initFile : function (filename) {
    var file = null;
    try {
      file
        = Components.classes ["@mozilla.org/file/local;1"]
        .createInstance (Components.interfaces.nsILocalFile);
      file.initWithPath (filename);
    }
    catch (e) {
      file = null;
    }
        
    return file;
  },
    
  /**
   * ファイルを作成する
   *
   * @param  String filename
   *         ファイル名
   * @param  String text
   *         ファイルの内容
   */
  createFile : function (filename, text) {
    var file = null;
    try {
      file
        = Components.classes ["@mozilla.org/file/local;1"]
        .createInstance (Components.interfaces.nsILocalFile);
      file.initWithPath (filename);
      file.create (0x00, 0644);
    }
    catch (e) {
      file = null;
    }
        
    if (file) {
      try {
        var fstream
        = Components
        .classes ["@mozilla.org/network/file-output-stream;1"]
        .createInstance (Components.interfaces.nsIFileOutputStream);
        fstream.init (file, 0x02 | 0x08 | 0x20, 0644, 0);
        fstream.write (text, text.length);
        fstream.close ();
      }
      catch (e) {
      }
    }
  },
    
  /**
   * ファイルを読み込む
   *
   * @param  String filename
   *         ファイル名
   * @return String
   *         ファイルの内容
   */
  readFile : function (filename) {
    var text = "";
    var file = null;
    try {
      file
        = Components.classes ["@mozilla.org/file/local;1"]
        .createInstance (Components.interfaces.nsILocalFile);
      file.initWithPath (filename);
    }
    catch (e) {
      file = null;
    }
        
    if (file) {
      try {
        var fstream
        = Components
        .classes ["@mozilla.org/network/file-input-stream;1"]
        .createInstance (Components.interfaces.nsIFileInputStream);
        var sstream
        = Components
        .classes ["@mozilla.org/scriptableinputstream;1"]
        .createInstance (Components.interfaces
                         .nsIScriptableInputStream);
        fstream.init (file, 0x01, 0444, 0);
        sstream.init (fstream);
        text = sstream.read (-1);
        sstream.close ();
        fstream.close ();
      }
      catch (e) {
        text = "";
      }
    }
        
    return text;
  },
    
  /**
   * ディレクトリを作成する
   *
   * @param  String dirname
   *         ディレクトリ名
   */
  createDirectory : function (dirname) {
    try {
      var dir
      = Components.classes ["@mozilla.org/file/local;1"]
      .createInstance (Components.interfaces.nsILocalFile);
      dir.initWithPath (dirname);
      if (!dir.exists ()) {
        dir.create (0x01, 0755);
      }
    }
    catch (e) {
    }
  },
    
  /**
   * ネイティブパスを file プロトコルに変換する
   *
   * @param  String filename
   *         ネイティブパス
   * @return String
   *         file プロトコル
   */
  getURLSpecFromFilename : function (filename) {
    var targetFile = UnMHTFileManager.initFile (filename);
        
    return UnMHTFileManager.fileProtocolHandler
    .getURLSpecFromFile (targetFile);
  },
    
  /**
   * ファイルを file プロトコルに変換する
   *
   * @param  nsILocalFile file
   *         ファイル
   * @return String
   *         file プロトコル
   */
  getURLSpecFromFile : function (file) {
    return UnMHTFileManager.fileProtocolHandler.getURLSpecFromFile (file);
  },
    
  /**
   * file プロトコルをネイティブパスに変換する
   *
   * @param  String uri
   *         file プロトコル
   * @return String
   *         ネイティブパス
   */
  getFilenameFromURLSpec : function (uri) {
    return UnMHTFileManager.fileProtocolHandler
    .getFileFromURLSpec (uri).path;
  },
    
  /**
   * file プロトコルをファイルに変換する
   *
   * @param  String uri
   *         file プロトコル
   * @return nsIFile
   *         ファイル、もしくは
   */
  getFileFromURLSpec : function (uri) {
    return UnMHTFileManager.fileProtocolHandler.getFileFromURLSpec (uri);
  },
    
  /**
   * パスを展開、連結する
   *
   * @param  String base
   *         親ディレクトリのパス
   * @param  String sub
   *         相対パス、もしくは絶対パス
   * @return String
   *         連結したパス
   */
  jointPath : function (base, sub) {
    var i;
    var path;
    var sep;
    
    var protocol;
    var host;
        
    if (base == null) {
      return sub;
    }
        
    /* セパレータを検出 */
    if (base.indexOf ("\\") != -1) {
      /* Windows のパスの場合 */
      sep = "\\";
    }
    else if (base.indexOf ("/") != -1) {
      /* Mac OS X、Linux のパス、URI の場合 */
      sep = "/";
    }
    else if (base.indexOf (":") != -1) {
      /* Mac OS 9 以前のパスの場合 */
      sep = ":";
    }
    else {
      /* 区切りが見付からない場合連結できないので
       * サブのみ返す */
      return sub;
    }
        
    if (base.match (/^([A-Za-z0-9\-]+:\/\/\/?)([^\/]+)(.*)$/)) {
      /* URI の場合 */
      protocol = RegExp.$1;
      host = RegExp.$2;
      base = RegExp.$3;
    }
    else if (base.match (/^([A-Za-z]+:)(.*)$/)) {
      /* Windows のネイティブなパスの場合 */
      protocol = RegExp.$1;
      host = "";
      base = RegExp.$2;
    }
    else if (base.match (/^(\\)(\\.*)$/)) {
      /* Windows のネットワークドライブの場合 */
      protocol = RegExp.$1;
      host = "";
      base = RegExp.$2;
    }
    else {
      /* その他のパスの場合 */
      protocol = "";
      host = "";
    }
    
    if (sub.match (/^(\/\/\/?)(.*)$/)) {
      /* プロトコルのみ継承する */
      path = protocol + RegExp.$2;
      return path;
    }
    
    if (sub.match (/^([A-Za-z0-9\-]+:\/\/\/?)([^\/]+)(.*)$/)
        || sub.match (/^([A-Za-z]+:)(.*)$/)
        || sub.match (/^(\\)(\\.*)$/)) {
      /* sub が URI、 Windows のネイティブな絶対パス、
       * ネットワークドライブの場合 */
      return sub;
    }
        
    if (sub.substring (0, 1) == sep) {
      /* sub が絶対パスの場合 */
      base = "";
    }
    
    if (sub.length > 5 && sub.substring (0, 5) == "data:") {
      /* data scheme の場合 */
      base = "";
    }
    
    path = base + sep + sub;
    var sepRegExp
    = new RegExp ("[^"
                  + sep.replace (/(\\\[\]\(\)\^\,\.\{\}\|\?\!\-\*\+\$)/,
                                 "\\$1")
                  + "]*",
                  "g");
    var newNames = [];
    var isDir = false;
    if (path.match
        (new RegExp ("["
                     + sep.replace (/(\\\[\]\(\)\^\,\.\{\}\|\?\!\-\*\+\$)/,
                                    "\\$1")
                     + "]$"))) {
      isDir = true;
    }
        
    /* パスを解析するだけなので代入はしない */
    path.replace
    (sepRegExp,
     function (matched) {
      if (matched == ".") {
      }
      else if (matched == "..") {
        newNames.pop ();
      }
      else if (matched == "") {
      }
      else {
        newNames.push (matched);
      }
      return "";
    });
        
    path = "";
    for (i = 0; i < newNames.length; i ++) {
      path += sep;
      path += newNames [i];
    }
    if (isDir) {
      path += sep;
    }
        
    path = protocol + host + path;
        
    return path;
  }
};
