/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/**
 * カスタムファイル名フォーマッタ
 */
var UnMHTFormatter = {
  messages : null,  /* nsIStringBundleService  ロケール */
  weekdays : null,  /* Array  曜日 (省略) */
  weekdays2 : null, /* Array  曜日 */
  
  format : function (pattern, TITLE, FILENAME, URL, am, pm) {
    if (UnMHTFormatter.messages == null) {
      UnMHTFormatter.messages
      = Components.classes ["@mozilla.org/intl/stringbundle;1"]
      .getService (Components.interfaces.nsIStringBundleService)
      .createBundle ("chrome://unmht/locale/unmht.properties");
      
      UnMHTFormatter.weekdays =
      [UnMHTFormatter.messages.GetStringFromName ("weekday_sun"),
       UnMHTFormatter.messages.GetStringFromName ("weekday_mon"),
       UnMHTFormatter.messages.GetStringFromName ("weekday_tue"),
       UnMHTFormatter.messages.GetStringFromName ("weekday_wed"),
       UnMHTFormatter.messages.GetStringFromName ("weekday_thu"),
       UnMHTFormatter.messages.GetStringFromName ("weekday_fri"),
       UnMHTFormatter.messages.GetStringFromName ("weekday_sat")];
      UnMHTFormatter.weekdays2 =
      [UnMHTFormatter.messages.GetStringFromName ("weekday2_sun"),
       UnMHTFormatter.messages.GetStringFromName ("weekday2_mon"),
       UnMHTFormatter.messages.GetStringFromName ("weekday2_tue"),
       UnMHTFormatter.messages.GetStringFromName ("weekday2_wed"),
       UnMHTFormatter.messages.GetStringFromName ("weekday2_thu"),
       UnMHTFormatter.messages.GetStringFromName ("weekday2_fri"),
       UnMHTFormatter.messages.GetStringFromName ("weekday2_sat")];
    }
    
    var prefBranch
    = Components.classes ["@mozilla.org/preferences-service;1"]
    .getService (Components.interfaces.nsIPrefBranch);
    
    var enableLimit = false, limitCount = 100;
    
    if (prefBranch.prefHasUserValue ("unmht.save.filename.custom.limit")) {
      enableLimit = prefBranch.getBoolPref ("unmht.save.filename.custom.limit");
    }
    if (prefBranch.prefHasUserValue ("unmht.save.filename.custom.limit.count")) {
      limitCount = prefBranch.getIntPref ("unmht.save.filename.custom.limit.count");
    }
    
    var text;
        
    var urlObj
    = Components.classes ["@mozilla.org/network/standard-url;1"]
    .createInstance (Components.interfaces.nsIURI);
    urlObj.spec = URL;
        
    var now = new Date ();
    var YYYY = now.getFullYear ();
    var Y = YYYY % 100;
    var YY = Y >= 10 ? Y : ("0" + Y);
    var M = now.getMonth () + 1;
    var MM = M >= 10 ? M : ("0" + M);
    var MMM = UnMHTFormatter.weekdays [now.getDay ()];
    var MMMM = UnMHTFormatter.weekdays2 [now.getDay ()];
    var D = now.getDate ();
    var DD = D >= 10 ? D : ("0" + D);
    var H = now.getHours ();
    var HH = H >= 10 ? H : ("0" + H);
    var h = now.getHours () % 12;
    var hh = h >= 10 ? h : ("0" + h);
    var m = now.getMinutes ();
    var mm = m >= 10 ? m : ("0" + m);
    var s = now.getSeconds ();
    var ss = s >= 10 ? s : ("0" + s);
    var ampm = H >= 12 ? pm : am;
    var port = urlObj.port;
    if (port == -1) {
      if (urlObj.scheme == "http") {
        port = 80;
      }
      else if (urlObj.scheme == "https") {
        port = 443;
      }
      else if (urlObj.scheme == "ftp") {
        port = 21;
      }
    }
    var path = urlObj.path.replace (/([^\?]*)\??(.*)?$/, "$1");
    var query = urlObj.path.replace (/([^\?]*)\??(.*)?$/, "$2");
    
    if (enableLimit) {
      TITLE = TITLE.substr (0, limitCount);
      FILENAME = FILENAME.substr (0, limitCount);
    }
    
    text = pattern
    .replace (/%YYYY%/g, YYYY)
    .replace (/%YY%/g, YY)
    .replace (/%M%/g, M)
    .replace (/%MM%/g, MM)
    .replace (/%MMM%/g, MMM)
    .replace (/%MMMM%/g, MMMM)
    .replace (/%D%/g, D)
    .replace (/%DD%/g, DD)
    .replace (/%H%/g, H)
    .replace (/%HH%/g, HH)
    .replace (/%h%/g, h)
    .replace (/%hh%/g, hh)
    .replace (/%m%/g, m)
    .replace (/%mm%/g, mm)
    .replace (/%s%/g, s)
    .replace (/%ss%/g, ss)
    .replace (/%ampm%/g, ampm)
    .replace (/%TITLE%/g, TITLE)
    .replace (/%FILENAME%/g, FILENAME)
    .replace (/%URL%/g, URL)
    .replace (/%URL2%/g, URL.replace (/^[A-Za-z0-9]+:\/\//, ""))
    .replace (/%SCHEME%/g, urlObj.scheme)
    .replace (/%HOST%/g, urlObj.host)
    .replace (/%PORT%/g, port)
    .replace (/%PATH%/g, path)
    .replace (/%QUERY%/g, query);
        
    return UnMHTFormatter.validateFileName (text);
  },
  
  /**
   * ファイル名に使えない文字を変換する
   *
   * @param  String filename
   *         元のファイル名
   * @return String
   *         変換したファイル名
   */
  validateFileName : function (filename) {
    var re = /\//g;
    if (navigator.appVersion.indexOf ("Windows") != -1) {
      re = /[\\\/\|]/g;
      filename = filename.replace (/\"/g, "'")
        .replace (/[\*\:\?#\!]/g, "_")
        .replace (/</g, "(")
        .replace (/>/g, ")");
    }
    else if (navigator.appVersion.indexOf ("Macintosh") != -1) {
      re = /[\:\/]/g;
    }
        
    filename = filename.replace (re, "_");
        
    return filename;
  }
};
