/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/**
 * ダウンロード履歴に追加するためのダミーのリスナ
 */
function UnMHTSaveDownloadListener () {
}
UnMHTSaveDownloadListener.prototype = {
  dl : null,    /* nsIDownload  対象のダウンロード */
  fileSize : 0, /* Number  ファイルサイズ */
    
  /**
   * インターフェースの要求
   *   nsISupports.QueryInterface
   *
   * @param  nsIIDRef iid
   *         インターフェース ID
   * @throws Components.results.NS_NOINTERFACE
   * @return nsICacheListener
   *         this
   */
  QueryInterface : function(iid) {
    if (iid.equals(Components.interfaces.nsISupports)
        || iid.equals(Components.interfaces.nsIStreamListener)
        || iid.equals(Components.interfaces.nsICancelable)) {
      return this;
    }
        
    throw Components.results.NS_NOINTERFACE;
  },
    
  /**
   * キャンセルイベント
   *   nsICancelable.cancel
   */
  cancel : function () {
  },
    
  /**
   * リクエスト開始のイベント
   *   nsIRequestObserver.onStartRequest
   *
   * @param  nsIRequest request
   *         対象のリクエスト
   * @param  nsISupports context
   *         ユーザ定義
   */
  onStartRequest : function (request, context) {
  },
    
  /**
   * データ到着のイベント
   *   nsIStreamListener.onDataAvailable
   *
   * @param  nsIRequest request
   *         対象のリクエスト
   * @param  nsISupports context
   *         ユーザ定義
   * @param  nsIInputStream inputStream
   *         データを取得するストリーム
   * @param  PRUint32 offset
   *         データの位置
   * @param  PRUint32 count 
   *         データの長さ
   */
  onDataAvailable : function (request, context, inputStream, offset, count) {
  },
    
  /**
   * リクエスト終了のイベント
   *   nsIRequestObserver.onStopRequest
   *
   * @param  nsIRequest request
   *         対象のリクエスト
   * @param  nsISupports context
   *         ユーザ定義
   * @param  Number statusCode
   *         終了コード
   */
  onStopRequest : function (request, context, statusCode) {
    this.dl.onProgressChange
    (null, request,
     this.fileSize, this.fileSize,
     this.fileSize, this.fileSize);
    this.dl.onStateChange
    (null, request,
     Components.interfaces.nsIWebProgressListener.STATE_STOP
     | Components.interfaces.nsIWebProgressListener.STATE_IS_NETWORK,
     Components.results.NS_OK);
  }
};

/**
 * 保存するファイルの情報
 *   Inherits From: nsIStreamListener, nsIRequestObserver
 *                  nsICacheListener
 *
 * @param  String loc
 *         Content-Location の値
 */
function UnMHTSaveFile (loc) {
  this.location = loc;
    
  this.status = UnMHTSave.STATUS_NA;
  this.originalContent = "";
  this.content = "";
  this.filetype = "";
  this.charset = "";
  this.isCSS = false;
  this.isHTML = false;
  this.isText = false;
  this.isIFrame = false;
    
  if (UnMHTSave.protocolHandler.isUnMHTURI (this.location)) {
    /* UnMHT の展開した結果らしいので
     * 本体を探す */
    var param = UnMHTSave.protocolHandler.getUnMHTURIParam (this.location);
    
    var entries
      = Components.classes ["@mozilla.org/appshell/window-mediator;1"]
      .getService (Components.interfaces.nsIWindowMediator)
      .getEnumerator ("navigator:browser");
        
    var original = param.original;
    var refName = param.refName;
        
    while (entries.hasMoreElements ()) {
      var w = entries.getNext ();
        
      var ex;
      if ((ex = w.UnMHT.getExtractor (original))) {
        var srcFile = null;
                
        for (var i = 0; i < ex.files.length; i ++) {
          srcFile = ex.files [i];
          if ((srcFile == ex.rootFile
               && refName == "")
              || refName == srcFile.refName) {
            this.headerList = srcFile.headerList;
            this.extractor = ex;
            this.srcFile = srcFile;
            break;
          }
        }
        break;
      }
    }
  }
}
UnMHTSaveFile.prototype = {
  context : null,       /* UnMHTSaveContext  保存コンテキスト */
  
  startTime : 0,        /* Number  開始時間 */
  status : 0,           /* Number  取得状況 */
  depth : 0,            /* Number 参照の深さ */
  base : "",            /* String  ベースディレクトリ */
  isCSS : false,        /* Boolean  CSS かどうか
                         *   CSS の import をチェックするかどうか */
  isHTML : false,       /* Boolean  HTML かどうか
                         *   コメント, リンクを入れるかどうか */
  isText : false,       /* Boolean  テキストかどうか */
  notEncode : false,    /* Boolean  エンコードしない */
  location : "",        /* String  Content-Location の値 */
  host : "",            /* String  Content-Location のホスト */
  originalContent : "", /* String  ファイルの中身 */
  encodedIndex : 0,     /* Number  エンコード済みの位置 */
  fileSize : "",        /* String  ファイルサイズ 不明の場合 unknown */
  recvSize : 0,         /* Number  取得したファイルサイズ */
  content : "",         /* String  base 64 エンコードしたファイルの中身 */
  filetype : "",        /* String  Content-Type のうち、MIME タイプ */
  charset : "",         /* String  Content-Type のうち、文字コード */
  channel : null,       /* nsIChannel  ネットワーク取得用チャネル */
  httpChannel : null,   /* nsIChannel  HTTP ヘッダ取得用チャネル */
  importFiles : null,   /* Array  @import しているファイル
                         *   [String  URL, ...] */
  headerList : null,    /* Array  ヘッダ
                         *   [[String フィールド名,
                         String フィールド値], ...] */
  extractor : null,     /* UnMHTExtractor  MHT ファイルの保持者 */
  srcFile : null,       /* UnMHTExtractorFile  MHT ファイル情報 */
  childFiles : null,    /* Array  含むファイル (multipart の場合)
                         *   [UnMHTSaveFile, ...] */
  isIFrame : false,     /* Boolean  インラインフレームか */
    
  referrer : "",        /* String  リファラ */
    
  /**
   * インターフェースの要求
   *   nsISupports.QueryInterface
   *
   * @param  nsIIDRef iid
   *         インターフェース ID
   * @throws Components.results.NS_NOINTERFACE
   * @return nsICacheListener
   *         this
   */
  QueryInterface : function (iid) {
    if (iid.equals (Components.interfaces.nsISupports)
        || iid.equals (Components.interfaces.nsISupportsWeakReference)
        || iid.equals (Components.interfaces.nsIStreamListener)
        || iid.equals (Components.interfaces.nsIRequestObserver)
        || iid.equals (Components.interfaces.nsICacheListener)) {
      return this;
    }
        
    throw Components.results.NS_NOINTERFACE;
  },
    
  /**
   * リクエスト開始のイベント
   *   nsIRequestObserver.onStartRequest
   *
   * @param  nsIRequest request
   *         対象のリクエスト
   * @param  nsISupports context
   *         ユーザ定義
   */
  onStartRequest : function (request, context) {
    this.originalContent = "";
    this.content = "";
    this.encodedIndex = 0;
    try {
      if (this.channel) {
        this.httpChannel
          = this.channel.QueryInterface
          (Components.interfaces.nsIHttpChannel);
                
        if (this.httpChannel.responseStatus >= 400) {
          this.status = UnMHTSave.STATUS_NG;
                    
          var NS_BINDING_ABORTED = 0x804b0002;
          try {
            this.channel.cancel (NS_BINDING_ABORTED);
          }
          catch (e) {
          }
                    
          return;
        }
                
        this.fileSize = "";
        try {
          this.fileSize
            = this.httpChannel.getResponseHeader ("Content-Length");
                
        }
        catch (e) {
        }
        if (this.fileSize && parseInt (this.fileSize) > 0) {
          this.fileSize += " bytes";
        }
        else {
          this.fileSize = "unknown";
        }
      }
    }
    catch (e) {
    }
  },
    
  /**
   * リクエスト終了のイベント
   *   nsIRequestObserver.onStopRequest
   *
   * @param  nsIRequest request
   *         対象のリクエスト
   * @param  nsISupports context
   *         ユーザ定義
   * @param  Number statusCode
   *         終了コード
   */
  onStopRequest : function (request, context, statusCode) {
    try {
      if (this.status == UnMHTSave.STATUS_NG) {
        return;
      }
      if (statusCode != Components.results.NS_OK) {
        this.status = UnMHTSave.STATUS_NG;
        return;
      }
            
      try {
        this.httpChannel
        = request.QueryInterface
        (Components.interfaces.nsIHttpChannel);
      }
      catch (e) {
      }
            
      if (this.filetype == ""
          && this.httpChannel) {
        try {
          var tmp
          = this.httpChannel.getResponseHeader ("Content-Type");
          if (tmp.match (/([^ ;]+\/[^ ;]+)/)) {
            this.filetype = RegExp.$1;
          }
        }
        catch (e) {
        }
      }
      if (this.filetype == "") {
        if (this.location.match (/\.[sx]?html?(\?.+)?(#.+)?$/i)) {
          this.filetype = "text/html";
        }
        else if (this.location.match (/\.css(\?.+)?$/i)) {
          this.filetype = "text/css";
        }
        else if (this.location.match (/\.mht(ml)?(\?.+)?$/i)) {
          this.filetype = "message/rfc822";
        }
        else if (this.location.match (/\.eml(\?.+)?$/i)) {
          this.filetype = "message/rfc822";
        }
        else if (this.location.match (/\.js(\?.+)?$/i)) {
          this.filetype = "text/javascript";
        }
        else if (this.location.match (/\.txt(\?.+)?$/i)) {
          this.filetype = "text/plain";
        }
        else if (this.location.match (/\.jpe?g(\?.+)?$/i)) {
          this.filetype = "image/jpeg";
        }
        else if (this.location.match (/\.gif(\?.+)?$/i)) {
          this.filetype = "image/gif";
        }
        else if (this.location.match (/\.png(\?.+)?$/i)) {
          this.filetype = "image/png";
        }
        else if (this.location.match (/\.bmp(\?.+)?$/i)) {
          this.filetype = "image/bmp";
        }
        else if (this.location.match (/\.ico(\?.+)?$/i)) {
          this.filetype = "image/x-icon";
        }
        else if (this.location.match (/\.svg(\?.+)?$/i)) {
          this.filetype = "image/svg+xml";
        }
        else if (this.location.match (/\.swf(\?.+)?$/i)) {
          this.filetype = "application/swf";
        }
        else if (this.location.match (/\.jar(\?.+)?$/i)) {
          this.filetype = "application/java-archive";
        }
        else if (this.location.match (/\.class(\?.+)?$/i)) {
          this.filetype = "application/octet-stream";
        }
        else if (this.location.match (/^about:blank$/)) {
          this.filetype = "text/html";
        }
        else if (this.location.match (/\.([A-Za-z0-9])(\?.*)?$/i)) {
          var ext = RegExp.$1;
          var mime
          = Components.classes ["@mozilla.org/mime;1"]
          .getService
          (Components.interfaces.nsIMIMEService);
                
          try {
            this.filetype
              = mime.getTypeFromExtension (ext);
          }
          catch (e) {
            this.filetype = "application/octet-stream";
          }
        }
        else {
          this.filetype = "application/octet-stream";
        }
      }
            
      if (this.isCSS) {
        this.importFiles
        = UnMHTSave.checkCSS (this.originalContent,
                              this.base || this.location,
                              this.depth,
                              this.referrer);
      }
            
      if (!this.notEncode
          && !this.isText) {
        if (this.encodedIndex > 0) {
          var data
          = this.originalContent.substr (this.encodedIndex);
          this.content += btoa (data);
        }
        else {
          this.content = btoa (this.originalContent);
        }
      }
            
      this.channel = null;
      this.httpChannel = null;
      this.status = UnMHTSave.STATUS_OK;
      
      if (this.isHTML
          && !this.isIFrame) {
        if (this.context.enableInsertURL) {
          this.originalContent
            = UnMHTSave.insertComment (this.originalContent,
                                       this.location);
        }
        if (this.context.enableInsertLink) {
          var ok = true;
          if (UnMHTSave.enableInsertLinkNoLocal) {
            var loc = this.location;
            if (UnMHTSave.protocolHandler.isUnMHTURI (loc)) {
              loc = UnMHTSave.protocolHandler.deUnMHTURI (loc);
            }
            if (loc.match (/^file:\/\//)) {
              ok = false;
            }
          }
          if (ok) {
            this.originalContent
              = UnMHTSave.insertLink (this.originalContent,
                                      this.location,
                                      this.context.insertLinkPos,
                                      this.context.enableInsertLinkDate,
                                      this.context.enableInsertLinkVersion);
          }
        }
      }
        
      if (!this.notEncode
          && this.isText) {
        var bom = "";
        if (this.charset.match (/utf-8/i)) {
          if (!this.originalContent.match (/\xEF\xBB\xBF/)) {
            bom = "=EF=BB=BF";
          }
        }
        this.content
        = bom + UnMHTSave.conv.encode_qp (this.originalContent);
      }
    }
    catch (e) {
    }
  },
    
  /**
   * データ到着のイベント
   *   nsIStreamListener.onDataAvailable
   *
   * @param  nsIRequest request
   *         対象のリクエスト
   * @param  nsISupports context
   *         ユーザ定義
   * @param  nsIInputStream inputStream
   *         データを取得するストリーム
   * @param  PRUint32 offset
   *         データの位置
   * @param  PRUint32 count 
   *         データの長さ
   */
  onDataAvailable : function (request, context, inputStream, offset, count) {
    try {
      var bstream
      = Components.classes ["@mozilla.org/binaryinputstream;1"]
      .createInstance (Components.interfaces.nsIBinaryInputStream);
      bstream.setInputStream (inputStream);
      this.originalContent += bstream.readBytes (count);
      this.recvSize += count;
      if (!this.notEncode
          && !this.isText) {
        while (this.originalContent.length
               > this.encodedIndex + 12000) {
          var data
            = this.originalContent.substr
            (this.encodedIndex, 12000);
          this.encodedIndex += 12000;
          this.content += btoa (data);
        }
      }
    }
    catch (e) {
    }
  },
    
  /**
   * キャッシュエントリが使用可能になったイベント
   *   nsICacheListener.onCacheEntryAvailable
   *
   * @param  nsICacheEntryDescriptor descriptor
   *         キャッシュの情報
   * @param  nsCacheAccessMode accessGranted
   *         アクセス権限
   * @param  nsresult status
   *         不明
   */
  onCacheEntryAvailable : function (descriptor, accessGranted, status) {
    if (this.status != UnMHTSave.STATUS_CACHE) {
      return;
    }
    if (accessGranted) {
      var head = "";
      try {
        head = descriptor.getMetaDataElement ("response-head");
      }
      catch (e) {
      }
            
      if (head.match (/^HTTP\/[0-9\.]+ ([0-9]+) /)) {
        var code = RegExp.$1;
        if (code >= 400) {
          this.status = UnMHTSave.STATUS_CACHE_NG;
                    
          return;
        }
      }
      if (head.match (/(^|[\r\n])location[ \r\n\t]*:[ \r\n\t]*([^\r\n]+)/i)) {
        var url = RegExp.$2;
        this.importFiles = new Array ();
        this.importFiles.push (url);
        this.status = UnMHTSave.STATUS_CACHE_NG;
                
        return;
      }
      else if (head.match (/content-encoding[ \r\n\t]*:[ \r\n\t]*(compress|deflate|gzip|x-compress|x-gzip)/i)) {
        var encoding = (RegExp.$1).toLowerCase ();
        
        try {
          var converter
            = Components.classes
            ["@mozilla.org/streamconv;1?from="
             + encoding + "&to=uncompressed"]
            .createInstance
            (Components.interfaces.nsIStreamConverter);
          converter.asyncConvertData (encoding, "uncompressed",
                                      this, null);
                    
          var listener
            = converter.QueryInterface
            (Components.interfaces.nsIStreamListener);
          listener.onStartRequest (null, null);
          var istream = descriptor.openInputStream (0);
          listener.onDataAvailable
            (null, null,
             istream, 0, descriptor.dataSize);
          istream.close ();
          listener.onStopRequest (null, null,
                                  Components.results.NS_OK);
          
          if (this.originalContent.length == 0) {
            /* cache is empty */
            this.originalContent = "";
            this.content = "";
                    
            this.status = UnMHTSave.STATUS_CACHE_NG;
          }
          
          return;
        }
        catch (e) {
          this.originalContent = "";
          this.content = "";
                    
          this.status = UnMHTSave.STATUS_CACHE_NG;
        }
      }
      else {
        var istream = descriptor.openInputStream (0);
        var bstream
        = Components.classes ["@mozilla.org/binaryinputstream;1"]
        .createInstance (Components.interfaces.nsIBinaryInputStream);
        bstream.setInputStream (istream);
        this.originalContent
        = bstream.readBytes (descriptor.dataSize);
        bstream.close ();
        istream.close ();
        
        if (this.originalContent.length == 0) {
          /* cache is empty */
          this.originalContent = "";
          this.content = "";
          
          this.status = UnMHTSave.STATUS_CACHE_NG;
        }
        
        this.onStopRequest (null, null,
                            Components.results.NS_OK);
      }
    }
    else {
      this.status = UnMHTSave.STATUS_CACHE_NG;
    }
  }
};

/**
 * 保存コンテキスト
 */
function UnMHTSaveContext () {
  this.enableInsertURL = UnMHTSave.enableInsertURL;
  this.enableInsertLink = UnMHTSave.enableInsertLink;
  this.insertLinkPos = UnMHTSave.insertLinkPos;
  this.enableInsertLinkDate = UnMHTSave.enableInsertLinkDate;
  this.enableInsertLinkVersion = UnMHTSave.enableInsertLinkVersion;
  
  var prefBranch
    = Components.classes ["@mozilla.org/preferences-service;1"]
    .getService (Components.interfaces.nsIPrefBranch);
  
  try {
    this.maxConns
      = prefBranch.getIntPref ("network.http.max-connections");
  }
  catch (e) {
    this.maxConns = 30;
  }
  
  try {
    this.maxConnsPerServer
      = prefBranch.getIntPref ("network.http.max-connections-per-server");
  }
  catch (e) {
    this.maxConnsPerServer = 8;
  }
  
  this.maxConns = Math.ceil (this.maxConns / 2);
  this.maxConnsPerServer = Math.ceil (this.maxConnsPerServer / 2);
}
UnMHTSaveContext.prototype = {
  maxConns : 30,                   /* Number  最大リクエスト数 */
  maxConnsPerServer : 8,           /* Number  サーバあたりの最大リクエスト数 */
  
  enableInsertURL : false,         /* Boolean  URL をコメントとして挿入 */
  enableInsertLink : false,        /* Boolean  URL をリンクとして挿入 */
  insertLinkPos : "bottom",        /* String  ページ先頭/ページ末尾 */
  enableInsertLinkDate : false,    /* Boolean  保存日時 */
  enableInsertLinkVersion : false, /* Boolean  バージョン情報 */
  
  isCanceled : false,  /* Boolean  キャンセルされたか */ 
  isStopped : false,   /* Boolean  保存が停止したか */
    
  extraHeaders : null, /* Array  拡張ヘッダ */
    
  targetFile : null,          /* nsIFile  対象のファイル */
  subject : "",               /* String  ファイルの subject */
  location : "",              /* String  ルートドキュメントの URI */ 
    
  urls : null,                /* Object  収集した URL */
  files : null,               /* Array  収集したファイル
                               *   [UnMHTSaveFile, ...] */
  
  skipAlertValue : "",        /* String  現在の値
                               *   "": 未設定
                               *   "save": 保存する
                               *   "ignore": 無視する */
    
  listener : null,            /* UnMHTSaveListener  イベントリスナ */
    
  /**
   * 初期化処理
   *
   * @param  nsIFile targetFile
   *         対象のファイル,
   * @param  String location
   *         メールを保存する際のルートドキュメントの URI
   * @param  String subject
   *         ファイルの subject
   * @param  UnMHTSaveListener listener
   *         イベントリスナ
   */
  init : function (targetFile,
                   location, subject, listener) {
    this.location = location;
    this.subject = subject;
    this.listener = listener;
        
    this.extraHeaders = new Array ();
        
    this.targetFile = targetFile;
        
    this.urls = new Object ();
    this.files = new Array ();
  },
    
  /**
   * 保存開始
   */
  start : function () {
    if (UnMHTSave.enableNolimit) {
      UnMHTSave.config.setTime (UnMHTSave.nolimitTime);
    }
    
    if (this.files.length > 0) {
      setTimeout
      ((function (self) {
          return function () {
            self.checkFiles ();
          };
        })(this), 100);
    }
    else {
      setTimeout
      ((function (self) {
          return function () {
            self.storeToFile ();
          };
        })(this), 100);
    }
  },
    
  /**
   * 拡張ヘッダを追加する
   *
   * @param  String name
   *         フィールド名
   * @param  String vlaue
   *         フィールド値
   */
  addExtraHeader : function (name, value) {
    this.extraHeaders.push (new Array (name, value));
  },
    
  /**
   * 保存対象のファイルの URL を収集する
   *
   * @param  HTMLDocument currentDocument
   *         対象のドキュメント
   * @param  String location
   *         URL
   * @param  Boolean useCurrentState
   *         現在の状態を使用するか
   * @param  HTMLElement node
   *         対象の要素
   * @param  String base
   *         ベースディレクトリ
   * @param  Number depth
   *         現在の深さ
   * @param  nsISelection targetSelection
   *         対象の選択範囲もしくは null
   */
  gatherNormalDocumentFiles : function (currentDocument, location,
                                        useCurrentState,
                                        node, base, depth,
                                        targetSelection) {
    var url;
    var style;
    var file;
    var content;

    var referrer;

    try {
      if (currentDocument.referrer) {
        referrer = currentDocument.referrer;
      }
    }
    catch (e) {
    }
        
    if (targetSelection) {
      if (node.nodeName.toLowerCase () == "head") {
        targetSelection = null;
      }
    }
        
    if (targetSelection) {
      var ok = false;
      for (var i = 0; i < targetSelection.rangeCount; i ++) {
        range = targetSelection.getRangeAt (i);
        sourceRange = document.createRange();
        sourceRange.selectNode (node);
        var compareSS
          = range.compareBoundaryPoints
          (Range.START_TO_START, sourceRange);
        var compareSE
          = range.compareBoundaryPoints
          (Range.START_TO_END, sourceRange);
        var compareES
          = range.compareBoundaryPoints
          (Range.END_TO_START, sourceRange);
        var compareEE
          = range.compareBoundaryPoints
          (Range.END_TO_END, sourceRange);
                
        if (!((compareSS == -1
               && compareSE == -1
               && compareES == -1
               && compareEE == -1)
              || (compareSS == 1
                  && compareSE == 1
                  && compareES == 1
                  && compareEE == 1))) {
          ok = true;
        }
      }
      if (!ok) {
        return;
      }
    }
        
    style = null;
    try {
      style = currentDocument.defaultView.getComputedStyle (node, "");
    }
    catch (e) {
    }
    if (style) {
      url = style.getPropertyCSSValue ("background-image");
      if (url && url.primitiveType == CSSPrimitiveValue.CSS_URI) {
        url = url.getStringValue ();
        url = UnMHTFileManager.jointPath (base, url);
        
        if (!(url in this.urls)) {
          this.urls [url] = 1;
                    
          file = new UnMHTSaveFile (url);
          file.context = this;
          file.depth = depth + 1;
          file.referrer = referrer;
          this.files.push (file);
        }
      }
      url = style.getPropertyValue ("background-image");
      if (url && url.match (/url[ \r\n\t]*\([ \r\n\t]*(\"(?:\\\"|[^\"])+\"|\'(?:\\\'|[^\'])+\'|[^\"\'\\\)]+)/)) {
        var path = RegExp.$1;
        if (path.match (/^\"(.*)\"$/)) {
          path = RegExp.$1;
        }
        else if (path.match (/^\'(.*)\'$/)) {
          path = RegExp.$1;
        }
        path = path.replace (/^ +/, "").replace (/ +$/, "");
        
        url = UnMHTFileManager.jointPath (base, path);
        
        if (!(url in this.urls)) {
          this.urls [url] = 1;
                    
          file = new UnMHTSaveFile (url);
          file.context = this;
          file.depth = depth + 1;
          file.referrer = referrer;
          this.files.push (file);
        }
      }
    }
        
    if (node instanceof Components.interfaces.nsIDOMHTMLStyleElement) {
      content = UnMHTSave.convertToPlainText (this, node, null);
      var newURLs
      = UnMHTSave.checkCSS
      (content, base, depth, referrer);
      for (var i = 0; i < newURLs.length; i ++) {
        var url = newURLs [i];
        if (!(url in this.urls)) {
          this.urls [url] = 1;
                        
          var file = new UnMHTSaveFile (url);
          file.context = this;
          file.depth = depth + 1;
          file.isCSS = true;
          file.isText = true;
          file.filetype = "text/css";
          file.referrer = referrer;
          this.files.push (file);
        }
      }
    }
    else if (node instanceof Components.interfaces.nsIDOMHTMLImageElement) {
      url = node.src;
      if (url) {
        if (!(url in this.urls)) {
          this.urls [url] = 1;
                    
          file = new UnMHTSaveFile (url);
          file.context = this;
          file.depth = depth + 1;
          file.referrer = referrer;
          this.files.push (file);
        }
      }
    }
    else if ("nsIDOMHTMLVideoElement" in Components.interfaces
             && node
             instanceof Components.interfaces.nsIDOMHTMLVideoElement) {
      url = node.src;
      if (url) {
        if (!(url in this.urls)) {
          this.urls [url] = 1;
                    
          file = new UnMHTSaveFile (url);
          file.context = this;
          file.depth = depth + 1;
          file.referrer = referrer;
          this.files.push (file);
        }
      }
      url = node.poster;
      if (url) {
        if (!(url in this.urls)) {
          this.urls [url] = 1;
                    
          file = new UnMHTSaveFile (url);
          file.context = this;
          file.depth = depth + 1;
          file.referrer = referrer;
          this.files.push (file);
        }
      }
    }
    else if ("nsIDOMHTMLAudioElement" in Components.interfaces
             && node
             instanceof Components.interfaces.nsIDOMHTMLAudioElement) {
      url = node.src;
      if (url) {
        if (!(url in this.urls)) {
          this.urls [url] = 1;
                    
          file = new UnMHTSaveFile (url);
          file.context = this;
          file.depth = depth + 1;
          file.referrer = referrer;
          this.files.push (file);
        }
      }
    }
    else if ("nsIDOMHTMLSourceElement" in Components.interfaces
             && node
             instanceof Components.interfaces.nsIDOMHTMLSourceElement) {
      url = node.src;
      if (url) {
        if (!(url in this.urls)) {
          this.urls [url] = 1;
                    
          file = new UnMHTSaveFile (url);
          file.context = this;
          file.depth = depth + 1;
          file.referrer = referrer;
          this.files.push (file);
        }
      }
    }
    else if (node instanceof Components.interfaces.nsIDOMHTMLLinkElement
             && node.rel) {
      if (node.rel.match (/\bicon\b/i)) {
        url = node.href;
        if (url) {
          if (!(url in this.urls)) {
            this.urls [url] = 1;
                    
            file = new UnMHTSaveFile (url);
            file.context = this;
            file.depth = depth + 1;
            file.referrer = referrer;
            this.files.push (file);
          }
        }
      }
      else if (node.rel.match (/\bstylesheet\b/i)) {
        url = node.href;
        if (url) {
          if (!(url in this.urls)) {
            this.urls [url] = 1;
                    
            file = new UnMHTSaveFile (url);
            file.context = this;
            file.depth = depth + 1;
            file.isCSS = true;
            file.isText = true;
            file.filetype = "text/css";
            file.referrer = referrer;
            this.files.push (file);
          }
        }
      }
    }
    else if (node instanceof Components.interfaces.nsIDOMHTMLInputElement
             && node.type.toLowerCase () == "image") {
      url = node.src;
      if (url) {
        if (!(url in this.urls)) {
          this.urls [url] = 1;
                    
          file = new UnMHTSaveFile (url);
          file.context = this;
          file.depth = depth + 1;
          file.referrer = referrer;
          this.files.push (file);
        }
      }
    }
    else if (node
             instanceof Components.interfaces.nsIDOMHTMLScriptElement) {
      url = node.src;
      if (url) {
        if (!(url in this.urls)) {
          this.urls [url] = 1;
                    
          file = new UnMHTSaveFile (url);
          file.context = this;
          file.depth = depth + 1;
          file.isText = true;
          file.filetype = "text/javascript";
          file.referrer = referrer;
          this.files.push (file);
        }
      }
    }
    else if (node instanceof Components.interfaces.nsIDOMHTMLEmbedElement) {
      url = node.src;
      url = UnMHTFileManager.jointPath (base, url);
      if (url) {
        if (!(url in this.urls)) {
          this.urls [url] = 1;
                    
          file = new UnMHTSaveFile (url);
          file.context = this;
          file.depth = depth + 1;
                    
          var ignoreFiles = [
            "http://69.9.170.242/doubleagent.swf",
            "http://admin.brightcove.com/viewer/",
            "http://cdn.tvs.npowr.com/tvsplayer/Player.swf",
            "http://l.yimg.com/cosmos.bcst.yahoo.com/",
            "http://media1.break.com/static/app/v1/global/swf/ghost_player.swf",
            "http://i.current.com/swf/barca/vm2_cdc.swf",
            "http://lads.myspace.com/videos/vplayer.swf",
            "http://raguun.jp/flv",
            "http://s.mcstatic.com/f/fvp/",
            "http://static.vsocial.com/flash/",
            "http://video.google.com/googleplayer.swf",
            "http://video.stage6.com/",
            "http://videos.streetfire.net/flvplayer3.swf",
            "http://vm2-2.vision.ameba.jp/mcs.swf",
            "http://www.blastro.com/flashplayer/flvPlayer.swf",
            "http://www.dailymotion.com/swf/",
            "http://www.danerd.com/web_player.swf",
            "http://www.guba.com/f/root.swf",
            "http://www.lulu.tv/wp-content/flash_play/flvplayer-square-beta2.swf",
            "http://www.musicvideocodes.info/asx.php",
            "http://www.youtube.com/v/",
            "/",
            "."
            ];
          var ok = true;
                    
          for (var k = 0; k < ignoreFiles.length; k ++) {
            if (url.indexOf (ignoreFiles [k]) == 0) {
              ok = false;
              break;
            }
          }
                    
          if (ok) {
            file.status = UnMHTSave.STATUS_NA;
          }
          else {
            file.status = UnMHTSave.STATUS_NG;
          }
          file.filetype = node.getAttribute ("type");
          file.referrer = referrer;
          this.files.push (file);
        }
      }
    }
    else if (node
             instanceof Components.interfaces.nsIDOMHTMLFrameElement) {
      try {
        this.addNormalDocument
        (node.contentDocument, useCurrentState,
         targetSelection,
         node.contentDocument.location.href, false,
         depth, false);
      }
      catch (e) {
      }
    }
    else if (node
             instanceof Components.interfaces.nsIDOMHTMLIFrameElement) {
      try {
        this.addNormalDocument
        (node.contentDocument, useCurrentState,
         targetSelection,
         node.contentDocument.location.href, false,
         depth, true);
      }
      catch (e) {
      }
    }
  },
    
  /**
   * ドキュメントのデータを収集する
   *
   * @param  HTMLDocument currentDocument
   *         対象のドキュメント
   * @param  Boolean useCurrentState
   *         現在の状態を使用するか
   * @param  nsISelection targetSelection
   *         対象の選択範囲もしくは null
   * @param  String location
   *         URL
   * @param  Boolean useUnMHTHeader
   *         UnMHT の展開したヘッダを継承するか
   * @param  Number depth
   *         現在の深さ
   * @param  Boolean isIFrame
   *         インラインフレームか
   */
  addNormalDocument : function (currentDocument, useCurrentState,
                                targetSelection,
                                location, useUnMHTHeader,
                                depth, isIFrame) {
    this.urls [location] = 1;
        
    var file = new UnMHTSaveFile (location);
    file.context = this;
    file.depth = depth + 1;
    var nodes;
    var i;
        
    if (useUnMHTHeader && file.extractor) {
      /* UnMHT の展開した結果であり,
       * 最初のファイルならば展開結果のヘッダを継承する */
      this.addUnMHTHeader (file.extractor.firstFile.headerList);
    }
        
    var base = location;
    var baseElement = null;
    if (base.match (/^(.+\/)([^\/]+)$/)) {
      base = RegExp.$1;
    }
        
    nodes = currentDocument.getElementsByTagName ("base");
    for (i = 0; i < nodes.length; i ++) {
      if (nodes [i].hasAttribute ("href")
          && nodes [i].getAttribute ("href")) {
        try {
          var baseURI
            = Components.classes ["@mozilla.org/network/standard-url;1"]
            .createInstance (Components.interfaces.nsIURI);
          baseURI.spec = nodes [i].getAttribute ("href");
          base
            = UnMHTFileManager.jointPath
            (base, baseURI.resolve ("./"));
        }
        catch (e) {
        }
      }
    }
        
    file.base = base;
    file.isIFrame = isIFrame;
        
    if (useCurrentState || targetSelection) {
      if (currentDocument.doctype) {
        file.originalContent
          += "<!DOCTYPE "
          + currentDocument.doctype.name
          + " PUBLIC \""
          + UnMHTSave.conv.escapeEntity
          (currentDocument.doctype.publicId)
          + "\" \""
          + UnMHTSave.conv.escapeEntity
          (currentDocument.doctype.systemId)
          + "\">\n";
      }
            
      var head = null;
      var meta = null;
      var contentTypeExists = false;
      nodes = currentDocument.getElementsByTagName ("head");
      if (nodes.length > 0) {
        head = nodes [0];
      }
      nodes = head.getElementsByTagName ("meta");
      for (i = 0; i < nodes.length; i ++) {
        if (nodes [i].getAttribute ("http-equiv")
            && nodes [i].getAttribute ("http-equiv")
            .match (/Content-Type/i)
            && nodes [i].getAttribute ("content")
            && nodes [i].getAttribute ("content")
            .match (/charset=/i)) {
          contentTypeExists = true;
          break;
        }
      }
      if (!contentTypeExists && currentDocument.inputEncoding) {
        meta = currentDocument.createElement ("meta");
        meta.setAttribute ("http-equiv", "Content-Type");
        meta.setAttribute ("content",
                           "text/html; charset="
                           + currentDocument.inputEncoding
                           .toLowerCase ());
      }
            
      var tmpBaseElement = null;
      if (!baseElement) {
        tmpBaseElement = currentDocument.createElement ("base");
        tmpBaseElement.setAttribute ("href", base);
      }
            
      if (meta) {
        if (head.firstChild) {
          head.insertBefore (meta, head.firstChild);
        }
        else {
          head.appendChild (meta);
        }
      }
      if (tmpBaseElement) {
        head.appendChild (tmpBaseElement);
      }
      file.originalContent
        += UnMHTSave.convertToPlainText
        (this, currentDocument.documentElement, targetSelection);
      if (tmpBaseElement) {
        head.removeChild (tmpBaseElement);
      }
      if (meta) {
        head.removeChild (meta);
      }
            
      if (currentDocument.inputEncoding
          && (currentDocument.inputEncoding.match (/iso-8859/i)
              || currentDocument.inputEncoding.match (/us-ascii/i)
              || currentDocument.inputEncoding.match (/iso646-jp/i)
              || currentDocument.inputEncoding.match (/windows-1252/i)
              || currentDocument.inputEncoding.match (/macintosh/i)
              || currentDocument.inputEncoding.match (/koi8-r/i)
              || currentDocument.inputEncoding.match (/koi8-u/i)
              || currentDocument.inputEncoding.match (/ebcdic/i))) {
        file.originalContent
          = file.originalContent.replace
          (/[\u0100-\uffff]/g,
           function (matched) {
            return "&#" + matched.charCodeAt (0).toString () + ";";
          });
      }
            
      if (!file.isIFrame) {
        if (this.enableInsertURL) {
          file.originalContent
            = UnMHTSave.insertComment (file.originalContent,
                                       file.location);
        }
        if (this.enableInsertLink) {
          var ok = true;
          if (UnMHTSave.enableInsertLinkNoLocal) {
            var loc = file.location;
            if (UnMHTSave.protocolHandler.isUnMHTURI (loc)) {
              loc = UnMHTSave.protocolHandler.deUnMHTURI (loc);
            }
            if (loc.match (/^file:\/\//)) {
              ok = false;
            }
          }
          if (ok) {
            file.originalContent
              = UnMHTSave.insertLink
              (file.originalContent,
               file.location,
               this.insertLinkPos,
               this.enableInsertLinkDate,
               this.enableInsertLinkVersion);
          }
        }
      }
            
      file.status = UnMHTSave.STATUS_OK;
      file.isCSS = true;
      file.isHTML = true;
      file.isText = true;
      file.charset = currentDocument.inputEncoding || "utf-8";
      file.originalContent
        = UnMHTSave.conv.convertFromUnicodeWithEntity
        (file.originalContent,
         file.charset);
      var bom = "";
      if (file.charset.match (/utf-8/i)) {
        bom = "\xEF\xBB\xBF";
      }
      file.content
        = UnMHTSave.conv.encode_qp (bom + file.originalContent);
      file.filetype = "text/html";
      file.referrer = "";
    }
    else {
      file.status = UnMHTSave.STATUS_NA;
      file.isCSS = true;
      file.isHTML = true;
      file.isText = true;
      file.filetype = "text/html";
      file.charset = currentDocument.inputEncoding || "utf-8";
      try {
        if (currentDocument.referrer) {
          file.referrer = currentDocument.referrer;
        }
      }
      catch (e) {
      }
    }
    this.files.push (file);
    
    var node;
    var firstNode;
    node = currentDocument.documentElement;
    firstNode = node;
    
    while (node) {
      this.gatherNormalDocumentFiles
        (currentDocument, location, useCurrentState, node, base, depth,
         targetSelection);
      
      if (node.firstChild) {
        node = node.firstChild;
      }
      else {
        while (node) {
          if (node.nextSibling) {
            node = node.nextSibling;
            break;
          }
          node = node.parentNode;
          if (node == firstNode) {
            return;
          }
        }
      }
    }
  },

  /**
   * ファイルのデータを収集する
   *
   * @param  HTMLDocument currentDocument
   *         対象のドキュメント
   * @param  String location
   *         URL
   */
  addFile : function (currentDocument, location) {
    this.urls [location] = 1;
        
    var file = new UnMHTSaveFile (location);
    file.context = this;
    if (currentDocument) {
      file.filetype = currentDocument.contentType;
      file.charset = currentDocument.inputEncoding || "utf-8";
    }
        
    try {
      if (currentDocument.referrer) {
        file.referrer = currentDocument.referrer;
      }
    }
    catch (e) {
    }
    this.files.push (file);
  },
    
  /**
   * ドキュメントを追加する
   *
   * @param  HTMLDocument targetDocument
   *         対象のドキュメント
   * @param  Boolean useCurrentState
   *         現在の状態を使用するか
   * @param  Boolean isScrapBook
   *         ScrapBook か
   * @param  Object scrapBookData
   *         ScrapBook のデータ
   * @param  nsISelection targetSelection
   *         対象の選択範囲もしくは null
   * @param  String location
   *         URL
   * @param  Boolean isHTML
   *         HTML かどうか
   * @param  Boolean useUnMHTHeader
   *         UnMHT の展開したヘッダを継承するか
   */
  addDocument : function (targetDocument, useCurrentState,
                          isScrapBook, scrapBookData,
                          targetSelection,
                          location, isHTML, useUnMHTHeader) {
    if (this.isCanceled) {
      this.isStopped = true;
      if (UnMHTSave.enableNolimit) {
        UnMHTSave.config.restoreTime ();
      }
      this.listener.onCancel ();
      return;
    }
    
    if (UnMHTSave.enableNolimit) {
      UnMHTSave.config.setTime (UnMHTSave.nolimitTime);
    }
    
    var istream;
    var content;
        
    try {
      if (!isScrapBook
          && !useCurrentState
          && UnMHTSave.protocolHandler.isUnMHTURI (location)) {
        this.addUnMHTDocument (location, useUnMHTHeader);
      }
      else {
        if (isScrapBook) {
          this.addScrapBook (scrapBookData);
        }
        else if (isHTML) {
          this.addNormalDocument
            (targetDocument, useCurrentState,
             targetSelection,
             location, useUnMHTHeader,
             0, false);
        }
        else {
          this.addFile (targetDocument, location);
        }
      }
    }
    catch (e) {
      this.listener.onError (this, "save", e);
      this.isStopped = true;
      if (UnMHTSave.enableNolimit) {
        UnMHTSave.config.restoreTime ();
      }
    }
  },
  
  /**
   * データドキュメントを追加する
   *
   * @param  String content
   *         内容
   * @param  String location
   *         Content-Location の値
   * @param  String charset
   *         Content-Type のうち、文字コード
   * @param  String filetype
   *         Content-Type のうち、MIME タイプ
   */
  addDataDocument : function (content, location, charset, filetype) {
    var file = new UnMHTSaveFile (location);
    file.context = this;
    file.status = UnMHTSave.STATUS_OK;
    file.depth = 0;
    file.charset = charset;
        
    file.filetype = filetype;
    if (file.filetype == "text/html") {
      file.isHTML = true;
      file.isText = true;
    }
    else if (file.filetype == "text/css") {
      file.isText = true;
    }
    else if (file.filetype == "text/javascript") {
      file.isText = true;
    }
        
    file.originalContent = content;
    if (file.isText) {
      file.content
        = UnMHTSave.conv.encode_qp (file.originalContent);
    }
    else {
      file.content = btoa (file.originalContent);
    }
                    
    this.files.unshift (file);
  },
    
  /**
   * unmht:// なドキュメントのデータを収集する
   *
   * @param  HTMLDocument targetDocument
   *         対象のドキュメント
   * @param  Boolean useUnMHTHeader
   *         UnMHT の展開したヘッダを継承するか
   */
  addUnMHTDocument : function (location,
                               useUnMHTHeader) {
    var param = UnMHTSave.protocolHandler.getUnMHTURIParam (location);
    
    var original = param.original;
    var refName = param.refName;
        
    /* 展開したウィンドウを探す */
    var entries
    = Components.classes ["@mozilla.org/appshell/window-mediator;1"]
    .getService (Components.interfaces.nsIWindowMediator)
    .getEnumerator ("navigator:browser");
    while (entries.hasMoreElements ()) {
      var w = entries.getNext ();
      
      var extractor;
      if ((extractor = UnMHT.getExtractor (original))) {
        /* 発見 */
        var srcFile = null;
        var rootFile;
                
        if (refName
            && refName in extractor.refFiles) {
          rootFile = extractor.refFiles [refName];
        }
        else {
          rootFile = extractor.rootFile;
          this.addUnMHTHeader (extractor.firstFile.headerList);
        }
                
        for (var i = -1; i < extractor.files.length; i ++) {
          var srcFile;
                    
          /* 指定されたファイルを最初にする */
          if (i == -1) {
            srcFile = rootFile;
          }
          else {
            srcFile = extractor.files [i];
            if (srcFile == rootFile) {
              continue;
            }
          }
          
          var location
            = UnMHTSave.protocolHandler.deUnMHTURI (srcFile.location);
          var file = new UnMHTSaveFile (location);
          file.context = this;
          file.headerList = srcFile.headerList;
          file.status = UnMHTSave.STATUS_OK;
          file.depth = 0;
          file.charset = srcFile.charset;
          
          file.filetype = srcFile.filetype;
          if (file.filetype == "text/html") {
            file.isHTML = true;
            file.isText = true;
          }
          else if (file.filetype == "text/css") {
            file.isText = true;
          }
          else if (file.filetype == "text/javascript") {
            file.isText = true;
          }
          var loc = baseURI;
          if (srcFile != rootFile) {
            loc += srcFile.refName;
          }
          
          file.originalContent = srcFile.originalContent;
          if (file.isHTML
              && !file.isIFrame) {
              
            if (this.enableInsertURL) {
              file.originalContent
                = UnMHTSave.insertComment
                (file.originalContent,
                 loc);
            }
            if (this.enableInsertLink) {
              var ok = true;
              if (UnMHTSave.enableInsertLinkNoLocal) {
                if (loc.match (/^file:\/\//)) {
                  ok = false;
                }
              }
              if (ok) {
                file.originalContent
                  = UnMHTSave.insertLink
                  (file.originalContent,
                   loc,
                   this.insertLinkPos,
                   this.enableInsertLinkDate,
                   this.enableInsertLinkVersion);
              }
            }
          }
          if (file.isText) {
            file.content
              = UnMHTSave.conv.encode_qp
              (file.originalContent);
          }
          else {
            file.content = btoa (file.originalContent);
          }
          file.referrer = "";
          this.files.push (file);
        }
        break;
      }
    }
  },
    
  /**
   * UnMHT が展開したファイルのヘッダを継承する
   *
   * @param  Array headerList
   *         ヘッダ
   *           [[String フィールド名,
   *             String フィールド値], ...]
   */
  addUnMHTHeader : function (headerList) {
    for (var j = 0; j < headerList.length; j ++) {
      var name = headerList [j][0];
      var value = headerList [j][1];
      if (name == "From"
          || name == "Subject"
          || name == "Date"
          || name == "MIME-Version"
          || name == "Content-Type") {
        continue;
      }
            
      this.addExtraHeader (name, value);
    }
  },
  
  /**
   * ScrapBook のデータを収集する
   *
   * @param  Object scrapBookData
   *         ScrapBook のデータ
   */
  addScrapBook : function (scrapBookData) {
    var dir = scrapBookData ["dir"].directoryEntries;
    while (dir.hasMoreElements ()) {
      var targetFile
        = dir.getNext ().QueryInterface (Components.interfaces.nsIFile);
            
      if (targetFile.leafName == "index.dat") {
        continue;
      }
            
      var isIndex = false;
      if (targetFile.leafName == "index.html") {
        isIndex = true;
      }
            
      var file = new UnMHTSaveFile (targetFile.leafName);
      file.context = this;
      file.depth = 0;
      file.status = UnMHTSave.STATUS_OK;
            
      if (isIndex) {
        file.location = scrapBookData ["source"];
        file.filetype = "text/html";
        file.charset = scrapBookData ["charset"];
      }
      else {
        file.location
          = UnMHTFileManager.jointPath
          (scrapBookData ["source"],
           file.location);
      }
            
      var fstream
        = Components
        .classes ["@mozilla.org/network/file-input-stream;1"]
        .createInstance (Components.interfaces.nsIFileInputStream);
      var bstream
        = Components.classes ["@mozilla.org/binaryinputstream;1"]
        .createInstance (Components.interfaces.nsIBinaryInputStream);
      fstream.init (targetFile, 0x01, 0444, 0);
      bstream.setInputStream (fstream);
      file.originalContent = bstream.readBytes (targetFile.fileSize);
      bstream.close ();
      fstream.close ();
            
      this.encodedIndex = 0;
            
      file.onStopRequest (null, null,
                          Components.results.NS_OK);
            
      if (!file.isIFrame) {
        if (this.enableInsertURL) {
          file.originalContent
            = UnMHTSave.insertComment (file.originalContent,
                                       file.location);
        }
        if (this.enableInsertLink) {
          file.originalContent
            = UnMHTSave.insertLink
            (file.originalContent,
             file.location,
             this.insertLinkPos,
             this.enableInsertLinkDate,
             this.enableInsertLinkVersion);
        }
      }
      file.referrer = "";
            
      if (isIndex) {
        this.files.unshift (file);
      }
      else {
        this.files.push (file);
      }
    }
  },
    
  /**
   * mht で保存 - ファイル収集のチェック
   */
  checkFiles : function () {
    if (this.isCanceled) {
      this.isStopped = true;
      if (UnMHTSave.enableNolimit) {
        UnMHTSave.config.restoreTime ();
      }
      this.listener.onCancel ();
      return;
    }
    
    var i;
        
    var ok, na, ng, conns, connsPerServer;
    ok = 0;
    na = 0;
    ng = 0;
    
    conns = 0;
    connsPerServer = new Object ();
    
    var currentFile = "";
    var currentSize = "";
        
    var channel;
    var httpChannel;
        
    var now = (new Date ()).getTime ();
        
    var ios
    = Components.classes ["@mozilla.org/network/io-service;1"]
    .getService (Components.interfaces.nsIIOService);
        
    var cacheService
    = Components.classes ["@mozilla.org/network/cache-service;1"]
    .getService (Components.interfaces.nsICacheService);
    
    var httpCacheSession = null;
    for (var i = 0; i < this.files.length; i ++) {
      if (currentFile == ""
          && this.files [i].status == UnMHTSave.STATUS_NET
          && this.files [i].fileSize) {
        currentFile = this.files [i].location;
        currentSize
          = this.files [i].recvSize + " bytes / "
          + this.files [i].fileSize;
      }
      
      if (currentFile == ""
          && this.files [i].status != UnMHTSave.STATUS_OK
          && this.files [i].status != UnMHTSave.STATUS_NG) {
        currentFile = this.files [i].location;
        var size = this.files [i].fileSize;
        if (size == "") {
          size = "unknown";
        }
        currentSize
          = this.files [i].recvSize + " bytes / "
          + size;
      }
      
      if ((this.files [i].status == UnMHTSave.STATUS_OK
           || this.files [i].status == UnMHTSave.STATUS_NG)
          && this.files [i].importFiles
          && this.files [i].importFiles.length > 0) {
        var nextStatus = UnMHTSave.STATUS_NA;
        
        if (this.files [i].depth + 1 >= 10) {
          /* 階層が深い */
          
          if (this.skipAlertValue == "") {
            /* デフォルト動作が決まっていない場合は
             * ダイアログを表示 */
            this.skipAlertValue = UnMHTSave.showDeepCSSAlert ();
          }
                    
          if (this.skipAlertValue == "ignore") {
            nextStatus = UnMHTSave.STATUS_NG;
          }
        }
                
        for (var j = 0;
             j < this.files [i].importFiles.length; j ++) {
          var url = this.files [i].importFiles [j];
          if (!(url in this.urls)) {
            this.urls [url] = 1;
                            
            na ++;
            var file = new UnMHTSaveFile (url);
            file.context = this;
            file.depth = this.files [i].depth + 1;
            file.status = nextStatus;
            file.isText = true;
            file.isCSS = true;
            file.filetype = "text/css";
            file.referrer = "";
            this.files.push (file);
          }
        }
        this.files [i].importFiles = null;
      }
      
      if (this.files [i].status == UnMHTSave.STATUS_NA) {
        na ++;
        
        this.files [i].startTime = (new Date ()).getTime ();
        this.files [i].status = UnMHTSave.STATUS_CACHE;
        try {
          if (httpCacheSession == null) {
            httpCacheSession
              = cacheService.createSession
              ("HTTP",
               Components.interfaces.nsICache.STORE_ANYWHERE,
               true);
          }
          httpCacheSession.doomEntriesIfExpired = false;
          
          httpCacheSession
            .asyncOpenCacheEntry
            (this.files [i].location,
             Components.interfaces.nsICache.ACCESS_READ,
             this.files [i]);
        }
        catch (e) {
          this.files [i].status = UnMHTSave.STATUS_CACHE_NG;
        }
      }
      else if (this.files [i].status == UnMHTSave.STATUS_CACHE) {
        na ++;
        
        if (now > this.files [i].startTime + 10 * 1000) {
          this.files [i].status = UnMHTSave.STATUS_CACHE_NG;
        }
      }
      else if (this.files [i].status == UnMHTSave.STATUS_NET) {
        na ++;
        
        conns ++;
        if (this.files [i].host in connsPerServer) {
          connsPerServer [this.files [i].host] ++;
        }
        else {
          connsPerServer [this.files [i].host] = 1;
        }
        
        if (now > this.files [i].startTime + 15 * 1000
            && this.files [i].recvSize == 0) {
          var NS_BINDING_ABORTED = 0x804b0002;
          try {
            this.files [i].channel.cancel (NS_BINDING_ABORTED);
          }
          catch (e) {
          }
                    
          this.files [i].status = UnMHTSave.STATUS_NG;
        }
      }
      else if (this.files [i].status == UnMHTSave.STATUS_OK) {
        ok ++;
      }
      else if (this.files [i].status == UnMHTSave.STATUS_NG) {
        ng ++;
      }
    }
    
    for (var i = 0; i < this.files.length; i ++) {
      if (this.files [i].status == UnMHTSave.STATUS_CACHE_NG) {
        na ++;
        
        if (!this.files [i].host) {
          var uri
          = Components.classes
          ["@mozilla.org/network/standard-url;1"]
          .createInstance (Components.interfaces.nsIURI);
          uri.spec = this.files [i].location;
          
          this.files [i].host = uri.host;
          if (!this.files [i].host) {
            this.files [i].host = "unknown";
          }
        }
        
        if (conns >= this.maxConns) {
          continue;
        }
        if (this.files [i].host in connsPerServer
            && connsPerServer [this.files [i].host] >= this.maxConnsPerServer) {
          continue;
        }
        
        if (this.files [i].host in connsPerServer) {
          connsPerServer [this.files [i].host] ++;
        }
        else {
          connsPerServer [this.files [i].host] = 1;
        }
        conns ++;
        
        this.files [i].startTime = (new Date ()).getTime ();
        this.files [i].status = UnMHTSave.STATUS_NET;
        try {
          channel 
            = ios.newChannel
            (this.files [i].location, null, null);
                    
          if (this.files [i].referrer) {
            try {
              var httpChannel
                = channel.QueryInterface
                (Components.interfaces.nsIHttpChannel);
                            
              var r
                = Components.classes
                ["@mozilla.org/network/standard-url;1"]
                .createInstance (Components.interfaces.nsIURI);
              r.spec = this.files [i].referrer;
                            
              httpChannel.referrer = r;
            }
            catch (e) {
            }
          }
                    
          this.files [i].channel = channel;
          this.files [i].channel.asyncOpen
            (this.files [i], null);
        }
        catch (e) {
          this.files [i].status = UnMHTSave.STATUS_NG;
        }
      }
    }
    
    this.listener.onProgress (this, currentFile, currentSize, ok, na, ng);
    
    if (na > 0) {
      /* まだ未取得の物が残っている */
      setTimeout
        ((function (self) {
            return function () {
              self.checkFiles ();
            };
          })(this), 100);
    }
    else {
      /* 取得完了 */
      setTimeout
      ((function (self) {
          return function () {
            self.storeToFile ();
          };
        })(this), 100);
    }
  },
    
  /**
   * mht で保存 - ファイルの保存
   */
  storeToFile : function () {
    if (this.isCanceled) {
      this.isStopped = true;
      if (UnMHTSave.enableNolimit) {
        UnMHTSave.config.restoreTime ();
      }
      this.listener.onCancel ();
      return;
    }
    
    this.decodeUnMHTLocation ();
    
    var i;
    
    var fstream;
    
    var fstream
    = Components
    .classes ["@mozilla.org/network/file-output-stream;1"]
    .createInstance (Components.interfaces.nsIFileOutputStream);
    try {
      fstream.init (this.targetFile, 0x02 | 0x08 | 0x20, 0664, 0);
    }
    catch (e) {
      this.listener.onError (this, "open", e);
      this.isStopped = true;
      if (UnMHTSave.enableNolimit) {
        UnMHTSave.config.restoreTime ();
      }
      return;
    }
        
    /* ヘッダに必要な情報を設定する */
    var subject
    = "=?iso-2022-jp?B?"
    + btoa (UnMHTSave.conv.convertToISO2022JP (this.subject))
    + "?=";
    var boundary = "----=_NextPart_000_0000_";
    var hex = [
      "0", "1", "2", "3", "4", "5", "6", "7",
      "8", "9", "A", "B", "C", "D", "E", "F"
      ];
    for (i = 0; i < 8; i ++) {
      boundary += hex [parseInt (Math.random () * 15)];
    }
    boundary += ".";
    for (i = 0; i < 8; i ++) {
      boundary += hex [parseInt (Math.random () * 15)];
    }
    var date
    = (new Date ()).toString ()
    .replace (/^([A-Za-z]{3}) /, "$1, ")
    .replace (/ \(.*\)$/, "");
        
    var data = "";
        
    /* 全体のヘッダ */
    data
    = "From: <Saved by UnMHT>\r\n"
    + "Subject: " + subject + "\r\n"
    + "Date: " + date + "\r\n"
    + "MIME-Version: 1.0\r\n"
    + "Content-Type: multipart/related;\r\n"
    + "\tboundary=\"" + boundary + "\";\r\n"
    + "\ttype=\"text/html\"\r\n";
    for (var i = 0; i < this.extraHeaders.length; i ++) {
      data += this.extraHeaders [i][0]
        + ": " + this.extraHeaders [i][1] + "\r\n";
    }
    data
    += "\r\n";

    fstream.write (data, data.length);
        
    for (i = 0; i < this.files.length; i ++) {
      var file = this.files [i];
      
      if (i != 0
          && file.location.match (/^data:/)) {
        /* ルートドキュメント以外の data scheme は保存しない */
        continue;
      }
      
      if (file.status == UnMHTSave.STATUS_NG) {
        continue;
      }
            
      data = "--" + boundary + "\r\n";
      fstream.write (data, data.length);
            
      data = "";
      if (file.charset) {
        data += "Content-Type: " + file.filetype
          + "; charset=\"" + file.charset + "\"\r\n";
      }
      else {
        data += "Content-Type: " + file.filetype + "\r\n";
      }
      if (file.isText) {
        data += "Content-Transfer-Encoding: quoted-printable\r\n";
      }
      else {
        data += "Content-Transfer-Encoding: base64\r\n";
      }
            
      var location = file.location;
      if (location.search
          (new RegExp ("[\u0100-\uffff]")) != -1) {
        location
          = "=?iso-2022-jp?B?"
          + btoa (UnMHTSave.conv.convertToISO2022JP
                  (file.location)) + "?=";
      }
      if (location.match
          (/^(imap|mailbox|news|nntp|pop|snews):\/\//)) {
        location
          = location
          .replace (/^(imap|mailbox|news|nntp|pop|snews):\/\/[^\/]*\//, "")
          .replace (this.location
                    .replace (/^(imap|mailbox|news|nntp|pop|snews):\/\/[^\/]*\//, ""),
                    "file:///mail/attachment/");
        if (location == "file:///mail/attachment/") {
          location = "file:///mail/index.html";
        }
      }
      data += "Content-Location: " + location + "\r\n";
      if (file.headerList) {
        for (var j = 0; j < file.headerList.length; j ++) {
          var name = file.headerList [j][0];
          var value = file.headerList [j][1];
          if (name == "Content-Location"
              || name == "Content-Transfer-Encoding"
              || name == "Content-Type") {
            continue;
          }
          data += name + ": " + value + "\r\n";
        }
      }
            
      data += "\r\n";
      fstream.write (data, data.length);
            
      if (file.isText) {
        fstream.write (file.content,
                       file.content.length);
      }
      else {
        data = file.content.replace(/.{76}/g, "$&\r\n");
        fstream.write (data,
                       data.length);
      }
            
      data = "\r\n";
      fstream.write (data, data.length);
    }
        
    data = "--" + boundary + "--\r\n";
    fstream.write (data, data.length);
        
    fstream.close ();
        
    var ngList = new Array ();
    for (var i = 0; i < this.files.length; i ++) {
      if (this.files [i].status == UnMHTSave.STATUS_NG) {
        ngList.push (this.files [i].location);
      }
    }
        
    if (!UnMHTSave.isThunderbird
        && UnMHTSave.enableRegisterHistory) {
      try {
        var dlManager
        = Components.classes ["@mozilla.org/download-manager;1"]
        .getService (Components.interfaces.nsIDownloadManager);
        var srcURI
        = Components.classes ["@mozilla.org/network/standard-url;1"]
        .createInstance (Components.interfaces.nsIURI);
        srcURI.spec = this.location;
        var ios = Components.classes ["@mozilla.org/network/io-service;1"].
        getService (Components.interfaces.nsIIOService);
        var targetURI
        = ios.newFileURI (this.targetFile);
            
        var mime = Components.classes ["@mozilla.org/mime;1"]
        .getService (Components.interfaces.nsIMIMEService);
        var mimeInfo
        = mime.getFromTypeAndExtension ("message/rfc822", "mht");
            
        var cancelable = {
          QueryInterface : function (iid) {
            if (iid.equals (Components.interfaces.nsISupports)
                || iid.equals (Components.interfaces
                               .nsISupportsWeakReference)
                || iid.equals (Components.interfaces.nsIStreamListener)
                || iid.equals (Components.interfaces
                               .nsIWebBrowserPersist)
                || iid.equals (Components.interfaces.nsICancelable)) {
              return this;
            }
                    
            throw Components.results.NS_NOINTERFACE;
          },
                
          cancel : function (reason) {
          }
        };
            
        var dl;
        var listener = new UnMHTSaveDownloadListener ();
        listener.fileSize = this.targetFile.fileSize;
        if (dlManager.addDownload.length == 7) {
          dl
            = dlManager.addDownload
            (srcURI, targetURI,
             this.targetFile.leafName,
             null, null, null,
             listener);
        }
        else if (dlManager.addDownload.length == 8) {
          dl
          = dlManager.addDownload
          (0, srcURI, targetURI,
           this.targetFile.leafName,
           null, null, null,
           listener);
        }
        else if (dlManager.addDownload.length == 9) {
          dl
          = dlManager.addDownload
          (0, srcURI, targetURI,
           this.targetFile.leafName,
           null, null, null, null,
           listener);
        }
        else {
          dl
          = dlManager.addDownload
          (srcURI, targetURI,
           this.subject,
           null, null,
           listener);
        }
        listener.dl = dl;
        
        var dummyURI
        = Components.classes ["@mozilla.org/network/standard-url;1"]
        .createInstance (Components.interfaces.nsIURI);
        dummyURI.spec = "unmht://dummy/hoge/fuga/";
        
        var channel = ios.newChannelFromURI (dummyURI);
        channel.asyncOpen (listener, null);
      }
      catch (e) {
      }
    }
    
    this.listener.onSave (this, ngList, false);
    
    this.isStopped = true;
    if (UnMHTSave.enableNolimit) {
      UnMHTSave.config.restoreTime ();
    }
  },
  
  /**
   * UnMHT のアドレスを消す
   */
  decodeUnMHTLocation : function () {
    for (var i = 0; i < this.files.length; i ++) {
      var file = this.files [i]
      if (file.srcFile) {
        var srcFile = file.srcFile;
        var extractor = file.extractor;
        file.location = UnMHTSave.protocolHandler.deUnMHTURI (srcFile.location);
        
        file.originalContent
        = extractor.unModifyAttribute
        (srcFile, file.filetype, file.originalContent);
        
        if (file.isText) {
          file.content
            = UnMHTSave.conv.encode_qp (file.originalContent);
        }
        else {
          file.content = btoa (file.originalContent);
        }
      }
    }
  },
  
  /**
   * 保存をキャンセル
   */
  cancel : function () {
    if (this.isStopped) {
      return true;
    }
    
    this.isCanceled = true;
    
    try {
      for (var i = 0; i < this.files.length; i ++) {
        var NS_BINDING_ABORTED = 0x804b0002;
        try {
          this.files [i].channel.cancel (NS_BINDING_ABORTED);
        }
        catch (e) {
        }
      }
    }
    catch (e) {
    }
        
    return false;
  }
};

/**
 * 保存器
 */
var UnMHTSave = {
  protocolHandler : null,        /* arUnMHTMhtmlProtocolHandler
                                  *   unmht プロトコルハンドラ */
  
  initialized : false,           /* Boolean  初期化フラグ */
  
  isThunderbird : false, /* Boolean  Thunderbird か */
    
  config : null,         /* UnMHTConfigManager 設定管理 */
  conv : null,           /* UnMHTConverter 文字列変換器 */
    
  /* ファイルの取得状況 */
  STATUS_NA : 0,       /* Number  未取得 */
  STATUS_CACHE : 1,    /* Number  キャッシュ開始 */
  STATUS_CACHE_NG : 2, /* Number  キャッシュ失敗 */
  STATUS_NET : 3,      /* Number  ネットワーク開始 */
  STATUS_OK : 4,       /* Number  取得完了 */
  STATUS_NG : 5,       /* Number  取得失敗 */
    
  messages : null,     /* nsIStringBundle  メッセージ */
  
  enableNolimit : false,        /* Boolean  保存中は n 秒経過するまで
                                 *   警告を表示しない*/
  nolimitTime : 0,              /* Number  秒数 */
  
  enableInsertURL : false,         /* Boolean  URL をコメントとして挿入 */
  enableInsertLink : false,        /* Boolean  URL をリンクとして挿入 */
  insertLinkPos : "bottom",        /* String  ページ先頭/ページ末尾 */
  enableInsertLinkDate : false,    /* Boolean  保存日時 */
  enableInsertLinkVersion : false, /* Boolean  バージョン情報 */
  
  enableInsertLinkNoLocal : false, /* Boolean  ローカルファイルには適用しない */
  enableSkipAlert : false,    /* Boolean  CSS の階層が深い場合に
                               *   警告を表示しない */
  enableRegisterHistory : false, /* Boolean  ダウンロード履歴に登録する */
  
  skipAlertDefault : "save",  /* String  警告を表示しない場合の動作 */
  skipAlertValue : "",        /* String  現在の値
                               *   "": 未設定
                               *   "save": 保存する
                               *   "ignore": 無視する */
    
  /**
   * 初期化処理
   */
  init : function () {
    if (UnMHTSave.initialized) {
      return;
    }
    UnMHTSave.initialized = true;
    
    var value;
        
    try {
      var id
        = Components.classes ["@mozilla.org/xre/app-info;1"]
        .getService (Components.interfaces.nsIXULAppInfo).ID;
      if (id == "{3550f703-e582-4d05-9a08-453d09bdfdc6}") {
        /* Thunderbird */
        UnMHTSave.isThunderbird = true;
      }
    }
    catch (e) {
    }
        
    if (UnMHTSave.isThunderbird) {
      /* Thunderbird */
      UnMHTSave.protocolHandler = {
        enUnMHTURI : function (uri) {
          return uri;
        },
        isUnMHTURI : function (uri) {
          return false;
        },
        deUnMHTURI : function (uri) {
          return uri;
        },
        getUnMHTURIParam : function (uri) {
          return null;
        }
      };
    }
    else {
      UnMHTSave.protocolHandler
      = Components.classes ["@mozilla.org/network/protocol;1?name=unmht"]
      .getService (Components.interfaces.arIUnMHTProtocolHandler);
    }
        
    UnMHTSave.messages
    = Components.classes ["@mozilla.org/intl/stringbundle;1"]
    .getService (Components.interfaces.nsIStringBundleService)
    .createBundle ("chrome://unmht/locale/unmht.properties");
        
    UnMHTSave.conv = new UnMHTConverter ();
        
    UnMHTSave.config = new UnMHTConfigManager ();
        
    UnMHTSave.enableNolimit
    = UnMHTSave.config
    .initPref ("bool", "unmht.save.nolimit", false);
    UnMHTSave.nolimitTime
    = UnMHTSave.config
    .initPref ("int",  "unmht.save.nolimit.time", 0);
    
    UnMHTSave.enableInsertURL
    = UnMHTSave.config
    .initPref ("bool", "unmht.save.inserturl", false);
    
    UnMHTSave.enableInsertLink
    = UnMHTSave.config
    .initPref ("bool", "unmht.save.insertlink", false);
    UnMHTSave.insertLinkPos
    = UnMHTSave.config
    .initPref ("char", "unmht.save.insertlink.pos", "bottom");
    UnMHTSave.enableInsertLinkDate
    = UnMHTSave.config
    .initPref ("bool", "unmht.save.insertlink.date", true);
    UnMHTSave.enableInsertLinkVersion
    = UnMHTSave.config
    .initPref ("bool", "unmht.save.insertlink.version", true);
    UnMHTSave.enableInsertLinkNoLocal
    = UnMHTSave.config
    .initPref ("bool", "unmht.save.insertlink.nolocal", false);
        
    UnMHTSave.enableSkipAlert
    = UnMHTSave.config
    .initPref ("bool", "unmht.save.skip_alert_on_deep", false);
    value
    = UnMHTSave.config
    .initPref ("char", "unmht.save.skip_alert_on_deep.default", "save");
    UnMHTSave.skipAlertDefault = unescape (value);
    if (UnMHTSave.enableSkipAlert) {
      UnMHTSave.skipAlertValue = UnMHTSave.skipAlertDefault;
    }
        
    UnMHTSave.enableRegisterHistory
    = UnMHTSave.config
    .initPref ("bool", "unmht.save.register_download", true);
  },
  
  /**
   * HTML ツリーを文字列に変換する
   *
   * @param  UnMHTSaveContext context
   *         保存コンテキスト
   * @param  HTMLElement element
   *         対象の要素
   * @param  nsISelection targetSelection
   *         対象の選択範囲もしくは null
   * @return String
   *         要素全体の HTML
   */
  convertToPlainText : function (context, element, targetSelection) {
    var text = "";
    var attributes = "";
    var name, value;
    var node = null;
        
    if (targetSelection) {
      if (element.nodeName.toLowerCase () == "head") {
        targetSelection = null;
      }
    }
    
    if (targetSelection) {
      var ok = false;
      for (var i = 0; i < targetSelection.rangeCount; i ++) {
        var range = targetSelection.getRangeAt (i);
        var sourceRange = document.createRange ();
        sourceRange.selectNode (element);
        var compareSS
          = range.compareBoundaryPoints
          (Range.START_TO_START, sourceRange);
        var compareSE
          = range.compareBoundaryPoints
          (Range.START_TO_END, sourceRange);
        var compareES
          = range.compareBoundaryPoints
          (Range.END_TO_START, sourceRange);
        var compareEE
          = range.compareBoundaryPoints
          (Range.END_TO_END, sourceRange);
                
        if (!((compareSS == -1
               && compareSE == -1
               && compareES == -1
               && compareEE == -1)
              || (compareSS == 1
                  && compareSE == 1
                  && compareES == 1
                  && compareEE == 1))) {
          ok = true;
        }
      }
      if (!ok) {
        return "";
      }
    }
        
    if (element
        && element.nodeName) {
      var nodeName = element.nodeName.toLowerCase ();
      
      if (nodeName == "#text") {
        text = UnMHTSave.conv.escapeEntity (element.nodeValue);
        if (targetSelection) {
          var textList = [];
          for (var i = 0; i < targetSelection.rangeCount; i ++) {
            var t = text;
            var modified = false;
            var range = targetSelection.getRangeAt (i);
            if (range.endContainer == element) {
              try {
                t = t.substr (0, range.endOffset);
                modified = true;
              }
              catch (e) {
              }
            }
            
            if (range.startContainer == element) {
              try {
                t = t.substr (range.startOffset);
                modified = true;
              }
              catch (e) {
              }
            }
            
            if (modified) {
              textList.push (t);
            }
          }
          if (textList.length > 0) {
            text = textList.join ("");
          }
        }
      }
      else if (nodeName == "#comment") {
        /* コメントをコメントとして展開する */
        text
        = "<!--"
        + element.nodeValue
        + "-->";
      }
      else if (nodeName == "base"
               && UnMHTSave.isThunderbird) {
      }
      else {
        if (element.attributes) {
          for (var i = 0; i < element.attributes.length; i ++) {
            name
            = element.attributes.item (i).nodeName
            .toLowerCase ();
            value = element.attributes.item (i).nodeValue;
            
            if (name.match (/^__unmht_/)) {
              continue;
            }
                        
            if (nodeName == "option") {
              if (name == "selected") {
                continue;
              }
            }
            
            if (nodeName == "input") {
              if (element.type == "checkbox"
                  || element.type == "radio") {
                if (name == "checked") {
                  continue;
                }
              }
              else {
                if (name == "value") {
                  continue;
                }
              }
            }
            
            if ((nodeName == "frame"
                 || nodeName == "iframe")
                && name == "src") {
              value = element.contentDocument.location.href;
            }
            
            if ((name == "src" || name == "href")
                && value.match
                (/^(imap|mailbox|news|nntp|pop|snews):\/\//)) {
              value
              = value
              .replace (/^(imap|mailbox|news|nntp|pop|snews):\/\/[^\/]*\//, "")
              .replace (context.location
                        .replace (/^(imap|mailbox|news|nntp|pop|snews):\/\/[^\/]*\//, ""),
                        "file:///mail/attachment/");
              if (value == "file:///mail/attachment/") {
                value = "file:///mail/index.html";
              }
            }
                        
            value = UnMHTSave.conv.escapeEntity (value);
                        
            attributes += " " + name +  "=" + "\"" + value + "\"";
          }
        }
                
        if (nodeName == "option") {
          if (element.selected) {
            name = "selected";
            value = "selected";
            attributes += " " + name +  "=" + "\"" + value + "\"";
          }
        }
        
        if (nodeName == "input") {
          if (element.type == "checkbox"
              || element.type == "radio") {
            if (element.checked) {
              name = "checked";
              value = "checked";
              attributes += " " + name +  "=" + "\"" + value + "\"";
            }
          }
          else if (element.type == "file") {
          }
          else {
            name = "value";
            value = UnMHTSave.conv.escapeEntity (element.value);
            attributes += " " + name +  "=" + "\"" + value + "\"";
          }
        }
        
        text += "<" + nodeName + attributes + ">";
                
        if (nodeName == "noscript"
            && element.firstChild
            && element.firstChild.nodeName.toLowerCase () == "#text"
            && element.firstChild.nextSibling == null) {
          /* NOSCRIPT 要素が展開されていない場合 */
          text += element.firstChild.nodeValue;
        }
        else if (nodeName == "noembed"
                 && element.firstChild
                 && element.firstChild.nodeName.toLowerCase () == "#text"
                 && element.firstChild.nextSibling == null) {
          /* NOEMBED 要素が展開されていない場合 */
          text += element.firstChild.nodeValue;
        }
        else if (nodeName == "style"
                 || nodeName == "script") {
          node = element.firstChild;
          while (node) {
            text
              += UnMHTSave.conv.unescapeEntity
              (UnMHTSave.convertToPlainText (context, node, targetSelection));
                        
            node = node.nextSibling;
          }
        }
        else if (nodeName == "textarea") {
          text += UnMHTSave.conv.escapeEntity (element.value);
        }
        else {
          node = element.firstChild;
          while (node) {
            text
              += UnMHTSave.convertToPlainText
              (context, node, targetSelection);
                        
            node = node.nextSibling;
          }
        }
                
        if (nodeName != "br"
            && nodeName != "img"
            && nodeName != "imput"
            && nodeName != "hr"
            && nodeName != "meta") {
          text += "</" + nodeName + ">";
        }
      }
    }
        
    return text;
  },
    
  /**
   * CSS の import をチェックする
   *
   * @param  String content
   *         ファイルの内容
   * @param  String base
   *         CSS の記述されたファイル
   * @param  Number depth
   *         現在の深さ
   * @param  String referrer
   *         リファラ
   * @return Array
   *         収集した URL
   */
  checkCSS : function (content, base, depth, referrer) {
    var newURLs = new Array ();
    
    content.replace
    (/(@import(?:[ \t\r\n]+url)?[ \t\r\n]*\(?[ \t\r\n]*)(\"(?:\\\"|[^\"])+\"|\'(?:\\\'|[^\'])+\'|[^\"\'\\;\)]+)/ig,
     function (matched, prev, path) {
      var next = "";
      if (path.match (/^\"(.*)\"$/)) {
        path = RegExp.$1;
        prev += "\"";
        next = "\"";
      }
      else if (path.match (/^\'(.*)\'$/)) {
        path = RegExp.$1;
        prev += "\'";
        next = "\'";
      }
      path = path.replace (/^ +/, "").replace (/ +$/, "");
      
      if (path) {
        if (base.match (/^(.+\/)([^\/]+)$/)) {
          base = RegExp.$1;
        }
        path = UnMHTFileManager.jointPath (base, path);
        
        newURLs.push (path);
      }
    });
        
    return newURLs;
  },
    
  /**
   * URL をコメントとして挿入する
   *
   * @param  String content
   *         ファイルの内容
   * @param  String location
   *         URL
   * @return String
   *         ファイルの内容
   */
  insertComment : function (content, location) {
    if (UnMHTSave.protocolHandler.isUnMHTURI (location)) {
      location = UnMHTSave.protocolHandler.deUnMHTURI (location);
    }
    
    var comment;
    comment = "<!-- Saved from " + location + " -->";
    if (content.match (/<html[^>]*>/i)) {
      content
        = content.replace
        (/<html[^>]*>/i,
         function (matched) {
          return matched + comment;
        });
    }
    else if (content.match (/<head[^>]*>/i)) {
      content
      = content.replace
      (/<head[^>]*>/i,
       function (matched) {
        return matched + comment;
      });
    }
    else if (content.match (/<body[^>]*>/i)) {
      content
      = content.replace
      (/<body[^>]*>/i,
       function (matched) {
        return matched + comment;
      });
    }
    else {
      content = comment + content;
    }
        
    return content;
  },

  /**
   * URL をリンクとして挿入する
   *
   * @param  String content
   *         ファイルの内容
   * @param  String location
   *         URL
   * @param  String pos
   *         挿入する場所
   * @param  Boolean date
   *         日付を挿入するか
   * @param  Boolean version
   *         バージョンを挿入するか
   * @return String
   *         ファイルの内容
   */
  insertLink : function (content, location, pos, date, version) {
    if (UnMHTSave.protocolHandler.isUnMHTURI (location)) {
      location = UnMHTSave.protocolHandler.deUnMHTURI (location);
    }
    
    var link;
    var style = "";
    if (pos == "bottom") {
      style = "clear: both; margin-top: 2em; ";
    }
        
    link
    = "<div style=\"font-size: 0.8em; " + style + "border: 1px solid black; background-color: white; color: black; padding: 0.5em; text-align: center;\">"
    + "Saved from <a class=\"unmht_link_to_original\" style=\"color: blue; background-color: inherit; text-decoration: underline;\" href=\"" + UnMHTSave.conv.escapeEntity (location) + "\">"
    + UnMHTSave.conv.escapeEntity (location)
    + "</a>";
        
    if (date) {
      link 
        += "<br />"
        + (new Date ());
    }
    if (version) {
      link
      += "<br />"
      + "Saved by UnMHT Ver." + UnMHTVersion;
    }
        
    link 
    += "</div>";
        
    if (pos == "bottom") {
      if (content.match (/<\/body>/i)) {
        content
          = content.replace
          (/<\/body>/i,
           function (matched) {
            return link + matched;
          });
      }
    }
    else {
      if (content.match (/<body[^>]*/i)) {
        content
        = content.replace
        (/<body[^>]*>/i,
         function (matched) {
          return matched + link + "<div style=\"position: relative\">";
        });
                
        content
        = content.replace
        (/<\/body>/i,
         function (matched) {
          return "</div>" + matched;
        });
      }
    }
        
    return content;
  },
    
  /**
   * CSS の階層が深い場合の警告を表示する
   *
   * @return String
   *         今回の動作
   */
  showDeepCSSAlert : function () {
    var confirmURL = "chrome://unmht/content/confirm.xul";
    var features = "chrome,dialog,modal,centerscreen";
    var result = {
      result : 0,
      applyToAll: false,
      rename : false,
      filename : ""
    };
        
    var m = UnMHTSave.messages;
    var param = {
      result: result,
      title: m.GetStringFromName ("deep_css_title"),
      message1: m.GetStringFromName ("deep_css_message"),
      message2: m.GetStringFromName ("deep_css_message2"),
      message3: m.GetStringFromName ("deep_css_message3"),
      accept: m.GetStringFromName ("deep_css_save"),
      cancel: m.GetStringFromName ("deep_css_ignore"),
      toall: m.GetStringFromName ("deep_css_do_not_ask_again")
    };
    openDialog (confirmURL, "", features, param);
        
    var value;
        
    if (result.result == "yes") {
      value = "save";
    }
    else if (result.result == "no") {
      value = "ignore";
    }
        
    if (result.applyToAll) {
      try {
        var prefBranch
        = Components.classes ["@mozilla.org/preferences-service;1"]
        .getService (Components.interfaces.nsIPrefBranch);
                
        prefBranch.setBoolPref
        ("unmht.save.skip_alert_on_deep", true);
                
        UnMHTSave.skipAlertDefault = value;
        prefBranch.setCharPref
        ("unmht.save.skip_alert_on_deep.default",
         escape (UnMHTSave.skipAlertDefault));
      }
      catch (e) {
      }
    }
        
    return value;
  }
};
