/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/**
 * MAF 組み込みオブジェクト
 */
function UnMHTSavePersist (useCurrentState, useAdditionalFiles) {
  this._useCurrentState = useCurrentState;
  this._useAdditionalFiles = useAdditionalFiles;
  this.currentState = 
    Components.interfaces.nsIWebBrowserPersist.PERSIST_STATE_READY;
}
UnMHTSavePersist.prototype = {
  _useCurrentState : false,    /* Boolean  現在の状態を使用するか */
  _useAdditionalFiles : false, /* Boolean  追加ファイルを使用するか */
    
  /**
   * インターフェースの要求
   *   nsISupports.QueryInterface
   *
   * @param  nsIIDRef iid
   *         インターフェース ID
   * @throws Components.results.NS_NOINTERFACE
   * @return nsIWebBrowserPersist
   *         this
   */
  QueryInterface : function (iid) {
    if (iid.equals (Components.interfaces.nsISupports)
        || iid.equals (Components.interfaces.nsIWebBrowserPersist)) {
      return this;
    }
        
    throw Components.results.NS_NOINTERFACE;
  },
                    
  /**
   * 保存キャンセル
   *   nsICancelable.cancel
   *
   * @param  nsresult reason
   *         理由
   */
  cancel : function (reason) {
    this.result = reason;
  },
    
  persistFlags : 0,                  /* Number  データ取得方法フラグ */
  currentState : 0,                  /* Number  状態 */
  result : Components.results.NS_OK, /* nsresult  理由 */
  progressListener : null,           /* nsIWebProgressListener
                                      *   進行状況リスナ */
    
  /**
   * URI を保存する
   *   nsIWebBrowserPersist.saveURI
   *
   * @param  nsIURI URI
   *         対象の URI
   * @param  nsISupports cacheKey
   *         キャッシュオブジェクト
   * @param  nsIURI referrer
   *         リファラ
   * @param  nsIInputStream postData
   *         POST データ
   * @param  String extraHeaders
   *         拡張ヘッダ
   * @param  nsIFile file
   *         対象のファイル
   */
  saveURI : function (URI, cacheKey, referrer, postData, extraHeaders, file) {
    throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
  },
  
  /**
   * チャネルを保存する
   *   nsIWebBrowserPersist.saveChannel
   *
   * @param  nsIChannel channel
   *         対象のチャネル
   * @param  nsIFile file
   *         対象のファイル
   */
  saveChannel: function (channel, file) {
    throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
  },
  
  /**
   * 保存開始イベント
   */
  saveDocument: function (targetDocument, targetFile,
                          targetDataDir, outputContentType,
                          encodingFlags, wrapColumn) {
    try {
      this.currentState
      = Components.interfaces.nsIWebBrowserPersist
      .PERSIST_STATE_SAVING;
      if (this.progressListener) {
        this.progressListener.onStateChange
          (null, null,
           Components.interfaces.nsIWebProgressListener
           .STATE_START
           | Components.interfaces.nsIWebProgressListener
           .STATE_IS_NETWORK,
           Components.results.NS_OK);
      }
            
      var targetFile
      = targetFile.QueryInterface
      (Components.interfaces.nsIFileURL).file;
            
      if (!targetFile.leafName.match (/\.mht(ml)?$/)) {
        var path = targetFile.path + ".mht";
        file
          = Components.classes ["@mozilla.org/file/local;1"]
          .createInstance (Components.interfaces.nsILocalFile);
        targetFile.initWithPath (path);
      }
            
      var useCurrentState = this._useCurrentState;
            
      var f
      = (function (self) {
          return function () {
            self.result = Components.results.NS_OK;
            self._onComplete ();
          };
        })(this);
            
      UnMHTUIManager.startSave
      (targetFile, targetDocument, useCurrentState,
       false, null,
       null, this._useAdditionalFiles,
       f);
    }
    catch (e) {
      if (e instanceof Components.interfaces.nsIXPCException) {
        this.result = e.result;
      } else {
        this.result = Components.results.NS_ERROR_FAILURE;
      }
            
      this._onComplete ();
    }
  },
    
  /**
   * 保存キャンセルイベント
   */
  cancelSave : function () {
    this.cancel (Components.results.NS_BINDING_ABORTED);
  },
  
  /**
   * 保存完了イベント
   */
  _onComplete : function () {
    if (this.currentState
        == Components.interfaces.nsIWebBrowserPersist
        .PERSIST_STATE_FINISHED) {
      return;
    }
        
    this.currentState
    = Components.interfaces.nsIWebBrowserPersist.PERSIST_STATE_FINISHED;
        
    if (this.progressListener) {
      this.progressListener.onStateChange
        (null, null,
         Components.interfaces.nsIWebProgressListener.STATE_STOP |
         Components.interfaces.nsIWebProgressListener.STATE_IS_NETWORK,
         this.result);
    }
  }
};
