/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/**
 * 保存イベントリスナ
 */
function UnMHTUIManagerSaveListener () {
}
UnMHTUIManagerSaveListener.prototype = {
  callback : null,  /* Function  完了時コールバック */
  
  progress : 0, /* Number  進行状況
                 *   0-1: 保存中 
                 *   2:   保存完了 */
  
  /**
   * 保存失敗イベント
   *
   * @param  UnMHTSaveContext context
   *         対象の保存コンテキスト
   * @param  String type
   *         エラーの種類
   * @param  String desc
   *         エラーの詳細
   */
  onError : function (context, type, desc) {
    var result = {
      result : 0,
      applyToAll: false,
      rename : false,
      filename : ""
    };
        
    var mes;
    if (type == "save") {
      mes
        = UnMHT.messages.GetStringFromName ("error_save");
    }
    else if (type == "open") {
      mes
      = UnMHT.messages.GetStringFromName ("error_open");
    }
    
    var confirmURL = "chrome://unmht/content/confirm.xul";
    var features = "chrome,dialog,modal,centerscreen";
    var m = UnMHT.messages;
    var param = {
      result: result,
      title: m.GetStringFromName ("error_title"),
      message1: mes,
      message2: desc,
      accept: m.GetStringFromName ("ok"),
    };
    openDialog (confirmURL, "", features, param);
    
    UnMHTUIManager.removeContext (context);
  },
    
  /**
   * 進行状況変化イベント
   *
   * @param  UnMHTSaveContext context
   *         対象の保存コンテキスト
   * @param  String currentFile
   *         現在取得のファイル名
   * @param  Number currentSize
   *         取得したバイト数
   * @param  Number ok
   *         取得完了した数
   * @param  Number na
   *         取得中の数
   * @param  Number ng
   *         取得失敗した数
   */
  onProgress : function (context, currentFile, currentSize, ok, na, ng) {
    if (na > 0) {
      this.progress = ok / context.files.length;
    }
    else {
      this.progress = 1;
    }
    
    UnMHTUIManager.updateProgress ();
  },
    
  /**
   * 保存完了
   *
   * @param  UnMHTSaveContext context
   *         対象の保存コンテキスト
   * @param  Array ngList
   *         失敗したファイル
   *           [String ファイル名, ...]
   */
  onSave : function (context, ngList) {
    if (UnMHT.enableShowErrors && ngList.length > 0) {
      var nglistURL = "chrome://unmht/content/nglist.xul";
      var features = "chrome,titlebar,toolbar,centerscreen,resizable";
      var param = { ngList : ngList };
      openDialog (nglistURL, "", features, param);
    }
    
    if (this.callback) {
      this.callback (false);
    }
    
    this.progress = 2;
    UnMHTUIManager.updateProgress ();
    UnMHTUIManager.removeContext (context);
  },
  
  /**
   * キャンセルが完了したイベント
   */
  onCancel : function () {
  }
};

/**
 * UI 管理
 */
var UnMHTUIManager = {
  prefDialog : null, /* ChromeWindow  設定ダイアログ */
    
  scrapBookData : new Object (), /* Object  ScrapBook のデータ */
  
  contexts : null,  /* Array 保存コンテキスト
                     *   [UnMHTSaveContext, ...] */
    
  prog : null,      /* XULElement  進行状況全体のボックス */
  progbar : null,   /* XULElement  進行状況のプログレスバー */
  proglabel : null, /* XULElement  進行状況のラベル */
  
  menuInserted : false, /* Boolean  フレームのメニューを挿入したか */
  
  /**
   * 初期化処理
   */
  init : function () {
    UnMHTUIManager.contexts = new Array ();
  },
    
  /**
   * 設定ダイアログを開く
   *
   * @param  Event event
   *         対象のイベント
   * @param  Boolean fromToolbar
   *         ツールバーからか
   */
  showPreferences : function (event, fromToolbar) {
    if (("button" in event && event.button != 0)
        || event.ctrlKey || event.shiftKey
        || event.altKey || event.metaKey) {
      return;
    }
        
    if (fromToolbar
        && event.target
        && event.target.nodeName
        && event.target.nodeName.match (/menuitem/i)) {
      return;
    }
        
    try {
      if (UnMHTUIManager.prefDialog != null) {
        if (!UnMHTUIManager.prefDialog.closed) {
          UnMHTUIManager.prefDialog.focus ();
          return;
        }
      }
    }
    catch (e) {
      UnMHTUIManager.prefDialog = null;
    }
    var optionsURL = "chrome://unmht/content/options.xul";
    var features = "chrome,titlebar,toolbar,centerscreen,resizable";
    UnMHTUIManager.prefDialog = openDialog (optionsURL, "", features);
        
    if (Components.interfaces.nsIPrefBranch2 == undefined) {
      UnMHTUIManager.prefDialog.addEventListener
        ("unload",
         function () {
          UnMHTUIManager.setPanelStatus ();
        }, false);
    }
  },
    
  /**
   * ツールバーーのメニューを設定する
   */
  setToolbarPopup : function () {
    var menuitem;
    
    var targetDocument;
    var savable = true;
        
    if (!UnMHT.isThunderbird) {
      var tabbrowser = document.getElementById ("content");
      if (!tabbrowser) {
        tabbrowser = document.getElementById ("browser_content");
      }
      if (!tabbrowser) {
        return;
      }
      targetDocument = tabbrowser.contentDocument;
    }
    else {
      var messagepane = document.getElementById ("messagepane");
      targetDocument = messagepane.contentDocument;
            
      var messagepanebox = document.getElementById ("messagepanebox");
      if (messagepanebox.collapsed) {
        savable = false;
      }
    }
        
    if (savable && UnMHT.isSavable (targetDocument)) {
      menuitem = document.getElementById ("unmht-toolbar-menuitem-save");
      if (menuitem) {
        menuitem.setAttribute ("disabled", "false");
      }
      menuitem = document.getElementById ("unmht-toolbar-menuitem-sendmail");
      if (menuitem) {
        menuitem.setAttribute ("disabled", "false");
      }
    }
    else {
      menuitem = document.getElementById ("unmht-toolbar-menuitem-save");
      if (menuitem) {
        menuitem.setAttribute ("disabled", "true");
      }
      menuitem = document.getElementById ("unmht-toolbar-menuitem-sendmail");
      if (menuitem) {
        menuitem.setAttribute ("disabled", "true");
      }
    }
  },
  
  /**
   * メニューが開かれるイベント
   * メニューの項目の表示／非表示を設定する
   *
   * @param  Event event
   *         対象のイベント
   */
  setContextMenu : function (event) {
    var menuitem;
    
    if (!UnMHTUIManager.menuInserted) {
      UnMHTUIManager.menuInserted = true;
      var frameMenu = document.getElementById ("frame");
      var unmhtMenu = document.getElementById ("unmht-frame-context-menu");
      if (frameMenu && frameMenu.firstChild) {
        var framePopup = frameMenu.firstChild;
        while (unmhtMenu.firstChild) {
          framePopup.appendChild (unmhtMenu.firstChild);
        }
      }
    }
    
    var tabbrowser = document.getElementById ("content");
    if (!tabbrowser) {
      tabbrowser = document.getElementById ("browser_content");
    }
    if (!tabbrowser) {
      return;
    }
    var targetDocument = tabbrowser.contentDocument;
    
    var savable = UnMHT.isSavable (targetDocument);
    var shown = false;
    
    menuitem
    = document.getElementById ("unmht-context-menuitem-save");
    if (menuitem) {
      menuitem.hidden = !(savable
                          && UnMHT.enableSaveContextMenu);
      shown = (shown || !menuitem.hidden);
    }
    menuitem
    = document.getElementById ("unmht-context-menuitem-save-selection");
    if (menuitem) {
      if ("gContextMenu" in window) {
        menuitem.hidden = !(savable
                            && UnMHT.enableSaveContextMenu
                            && gContextMenu.isTextSelected);
      }
      else {
        targetDocument = document.popupNode.ownerDocument;
        var isTextSelected = false;
        var targetSelection = targetDocument.defaultView.getSelection ();
        for (var i = 0; i < targetSelection.rangeCount; i ++) {
          range = targetSelection.getRangeAt (i);
          if (range.startContainer != range.endContainer
              || range.startOffset != range.endOffset) {
            isTextSelected = true;
          }
        }
          
        menuitem.hidden = !(savable
                            && UnMHT.enableSaveContextMenu
                            && isTextSelected);
      }
      shown = (shown || !menuitem.hidden);
    }
    menuitem
    = document.getElementById ("unmht-context-menuitem-save-link");
    if (menuitem) {
      if ("gContextMenu" in window) {
        menuitem.hidden = !(savable
                            && UnMHT.enableSaveContextMenu
                            && gContextMenu.onLink);
      }
      else {
        var target = document.popupNode;
        var node = target;
        var onLink = false;
        while (node) {
          if (node instanceof HTMLAnchorElement && node.href) {
            onLink = true;
            break;
          }
          node = node.parentNode;
        }
        menuitem.hidden = !(savable
                            && UnMHT.enableSaveContextMenu
                            && onLink);
      }
      shown = (shown || !menuitem.hidden);
    }
    menuitem
    = document.getElementById ("unmht-context-menuitem-quicksave");
    if (menuitem) {
      menuitem.hidden = !(savable
                          && UnMHT.enableQuicksaveMenu);
      shown = (shown || !menuitem.hidden);
    }
    menuitem
    = document.getElementById ("unmht-context-menuitem-sendmail");
    if (menuitem) {
      menuitem.hidden = !(savable
                          && UnMHT.enableSendmailMenu);
      shown = (shown || !menuitem.hidden);
    }
    menuitem
    = document.getElementById ("unmht-context-menuitem-separator");
    if (menuitem) {
      menuitem.hidden = !shown;
    }
    
    targetDocument = document.popupNode.ownerDocument;

    savable = UnMHT.isSavable (targetDocument);
    shown = false;
    
    menuitem
    = document.getElementById ("unmht-frame-context-menuitem-save");
    if (menuitem) {
      menuitem.hidden = !(savable
                          && UnMHT.enableSaveContextMenu);
      shown = (shown || !menuitem.hidden);
    }
    menuitem
    = document.getElementById ("unmht-frame-context-menuitem-sendmail");
    if (menuitem) {
      menuitem.hidden = !(savable
                          && UnMHT.enableSendmailMenu);
      shown = (shown || !menuitem.hidden);
    }
    menuitem
    = document.getElementById ("unmht-frame-context-menuitem-separator");
    if (menuitem) {
      menuitem.hidden = !shown;
    }
  },
    
  /**
   * ステータスバーのメニューを設定する
   */
  setStatusbarPopup : function () {
    var menuitem;
    
    menuitem = document.getElementById ("unmht-statusbar-popup-all");
    if (menuitem) {
      if (UnMHT.enableAll) {
        menuitem.label
          = UnMHT.messages.GetStringFromName ("turnoff");
      }
      else {
        menuitem.label
          = UnMHT.messages.GetStringFromName ("turnon");
      }
    }
    
    var targetDocument;
    var savable = true;
    if (!UnMHT.isThunderbird) {
      var tabbrowser = document.getElementById ("content");
      if (!tabbrowser) {
        tabbrowser = document.getElementById ("browser_content");
      }
      targetDocument = tabbrowser.contentDocument;
    }
    else {
      var messagepane = document.getElementById ("messagepane");
      targetDocument = messagepane.contentDocument;
      
      var messagepanebox = document.getElementById ("messagepanebox");
      if (messagepanebox.collapsed) {
        savable = false;
      }
    }
    
    savable = (savable && UnMHT.isSavable (targetDocument));
    
    menuitem
    = document.getElementById ("unmht-statusbar-menuitem-save");
    if (menuitem) {
      menuitem.disabled = !savable;
    }
    
    menuitem
    = document.getElementById ("unmht-statusbar-menuitem-sendmail");
    if (menuitem) {
      menuitem.hidden = !UnMHT.enableSendmailMenu || !savable;
      menuitem.disabled = !UnMHT.enableSendmailMenu || !savable;
    }
  },
    
  /**
   * ファイルメニューを設定する
   *
   * @param  Event event
   *         対象のイベント
   */
  setFileMenuPopup : function (event) {
    var tabbrowser = document.getElementById ("content");
    if (!tabbrowser) {
      tabbrowser = document.getElementById ("browser_content");
    }
    var targetDocument = tabbrowser.contentDocument;
    
    var savable = UnMHT.isSavable (targetDocument);
    var value = "true";
    if (savable) {
      value = "false";
    }
    var command;
    
    command = document.getElementById ("unmht-command-save");
    if (command) {
      command.setAttribute ("disabled", value);
    }
    command = document.getElementById ("unmht-command-sendmail");
    if (command) {
      command.setAttribute ("disabled", value);
    }
  },
    
  /**
   * 全機能の ON／OFF を切り替える
   */
  switchDisabled : function () {
    UnMHT.enableAll
    = !UnMHT.config.initPref ("bool", "unmht.all", true);
    UnMHT.config.prefBranch.setBoolPref ("unmht.all",
                                         UnMHT.enableAll);
        
    UnMHTUIManager.setPanelStatus ();
  },
    
  /**
   * キャッシュを開く
   */
  openCache : function () {
    var tabbrowser = document.getElementById ("content");
    if (!tabbrowser) {
      tabbrowser = document.getElementById ("browser_content");
    }
    var uri = UnMHT.protocolHandler.enUnMHTURI ("cache:");
    if ("addTab" in tabbrowser) {
      var newTab = tabbrowser.addTab (uri);
      tabbrowser.selectedTab = newTab;
    }
    else {
      tabbrowser.contentDocument.location.href = uri;
    }
  },
    
  /**
   * キャッシュをクリア
   */
  confirmClearCache : function () {
    if (confirm (UnMHT.messages.GetStringFromName
                 ("confirm_clear_cache"))) {
      UnMHT.clearCache (0, true);
    }
  },
    
  /**
   * ScrapBook のポップアップを設定する
   *
   * @param  Event event
   *         対象のイベント
   */
  setScrapBookPopup : function (event) {
    var children = document.popupNode;
    var tree = children;
    while (tree) {
      if (tree.nodeName.toLowerCase () == "tree") {
        break;
      }
      tree = tree.parentNode;
    }
    if (!tree) {
      return;
    }
        
    var resource = tree.builderView.getResourceAtIndex (tree.currentIndex);
        
    UnMHTUIManager.scrapBookData ["source"]
    = sbDataSource.getProperty (resource, "source");
    UnMHTUIManager.scrapBookData ["title"]
    = sbDataSource.getProperty (resource, "title");
    UnMHTUIManager.scrapBookData ["id"]
    = sbDataSource.getProperty (resource, "id");
    UnMHTUIManager.scrapBookData ["dir"]
    = sbCommonUtils.getContentDir (UnMHTUIManager.scrapBookData ["id"]);
    UnMHTUIManager.scrapBookData ["charset"]
    = sbDataSource.getProperty (resource, "chars");
        
    var menuitem
    = tree.ownerDocument.getElementById ("unmht-scrapbook-save");
    if (UnMHTUIManager.scrapBookData ["source"]) {
      menuitem.hidden = false;
    }
    else {
      menuitem.hidden = true;
    }
  },
    
  /**
   * 保存状況をステータスバーに表示するリスナを作成する
   */
  createListener : function () {
    var pref, prog, progbar, proglabel;
    pref
    = document.getElementById
    ("unmht-statusbarpanel-preferences");
    prog
    = document.getElementById
    ("unmht-statusbarpanel-progress");
    progbar
    = document.getElementById
    ("unmht-statusbarpanel-progressbar");
    proglabel
    = document.getElementById
    ("unmht-statusbarpanel-progress-label");
    if (pref && prog && pref.parentNode) {
      if (pref.nextSibling) {
        pref.parentNode.insertBefore (prog, pref.nextSibling);
      }
      else {
        pref.parentNode.appendChild (prog);
      }
    }
                
    var listener = new UnMHTUIManagerSaveListener ();
        
    if (prog && progbar && proglabel) {
      progbar.setAttribute ("value", "0%");
      prog.hidden = false;
      progbar.hidden = false;
      proglabel.hidden = true;
      
      UnMHTUIManager.prog = prog;
      UnMHTUIManager.progbar = progbar;
      UnMHTUIManager.proglabel = proglabel;
    }
        
    return listener;
  },
    
  /**
   * ファイルを保存する
   *
   * @param  UnMHTSaveContext saveContext
   *         保存コンテキスト
   * @param  nsIFile targetFile
   *         対象のファイル
   * @param  HTMLDocument targetDocument
   *         対象のドキュメント
   * @param  Boolean useCurrentState
   *         現在の状態を使用するか
   * @param  Boolean isScrapBook
   *         ScrapBook か
   * @param  Object scrapBookData
   *         ScrapBook のデータ
   * @param  nsISelection targetSelection
   *         対象の選択範囲もしくは null
   * @param  Boolean useAdditionalFiles
   *         追加ファイルダイアログを使用するか
   * @param  Array additionalFiles
   *         追加ファイル
   * @param  Function callback
   *         終了時コールバック
   */
  startSave : function (saveContext,
                        targetFile, targetDocument, useCurrentState,
                        isScrapBook, scrapBookData,
                        targetSelection, useAdditionalFiles, additionalFiles,
                        callback) {
    var callback2
    = function (additionalFiles) {
      if (UnMHT.enableSaveStatusbar) {
        var listener = UnMHTUIManager.createListener ();
        listener.callback = callback;
        
        saveContext.init
        (targetFile,
         targetDocument.location.href, targetDocument.title,
         listener);
        
        saveContext.addDocument
        (targetDocument, useCurrentState,
         isScrapBook, scrapBookData,
         targetSelection,
         targetDocument.location.href, true, true);
        if (saveContext.isStopped) {
          return;
        }
        
        if (additionalFiles) {
          for (var i = 0; i < additionalFiles.length; i ++) {
            saveContext.addFile (null, additionalFiles [i]);
          }
        }
        
        saveContext.start ();
        
        UnMHTUIManager.contexts.push (saveContext);
      }
      else {
        var pageURL = "chrome://unmht/content/save.xul";
        var features = "chrome,titlebar,centerscreen,resizable";
                    
        var param = {
          targetFile : targetFile,
          targetDocument : targetDocument,
          useCurrentState : useCurrentState,
          isScrapBook : isScrapBook,
          scrapBookData : scrapBookData,
          targetSelection : targetSelection,
          additionalFiles : additionalFiles,
          callback : callback
        };
            
        openDialog (pageURL, "", features, param);
      }
    };
    
    if (useAdditionalFiles) {
      var pageURL = "chrome://unmht/content/additional.xul";
      var features = "chrome,titlebar,centerscreen,resizable";
            
      var param = {
        callback : callback2
      };
            
      openDialog (pageURL, "", features, param);
    }
    else {
      callback2 (additionalFiles);
    }
  },
  
  /**
   * 保存をキャンセルする
   */
  cancelSave : function () {
    for (var i = 0; i < UnMHTUIManager.contexts.length; i ++) {
      UnMHTUIManager.contexts [i].cancel ();
    }
    UnMHTUIManager.contexts = new Array ();
    
    UnMHTUIManager.updateProgress ();
  },

  /**
   * 保存コンテキストを監視対象から削除
   *
   * @param  UnMHTSaveContext context
   *         対象の保存コンテキスト
   */
  removeContext : function (context) {
    for (var i = 0; i < UnMHTUIManager.contexts.length; i ++) {
      if (UnMHTUIManager.contexts [i] == context) {
        UnMHTUIManager.contexts.splice (i, 1);
        break;
      }
    }
    
    UnMHTUIManager.updateProgress ();
  },
  
  /**
   * ステータスバーの進行状況を更新する
   */
  updateProgress : function () {
    var na = 0;
    var ok = 0;
    var progress = 0;
    
    for (var i = 0; i < UnMHTUIManager.contexts.length; i ++) {
      var listener = UnMHTUIManager.contexts [i].listener;
      
      if (listener.progress == 2) {
        progress += 1;
        ok ++;
      }
      else {
        progress += listener.progress;
        na ++;
      }
    }
    
    if (na == 0) {
      if (ok > 0) {
        /* 保存完了 */
        if (UnMHTUIManager.prog) {
          UnMHTUIManager.prog.hidden = false;
        }
        if (UnMHTUIManager.progbar) {
          UnMHTUIManager.progbar.hidden = true;
        }
        if (UnMHTUIManager.proglabel) {
          UnMHTUIManager.proglabel.hidden = false;
          UnMHTUIManager.proglabel.value
          = UnMHT.messages.GetStringFromName ("save_done")
        }
        if (UnMHTUIManager.prog) {
          setTimeout
          ((function (self) {
              return function () {
                UnMHTUIManager.updateProgress ();
              };
            })(this), 3000);
        }
      }
      else {
        if (UnMHTUIManager.prog) {
          UnMHTUIManager.prog.hidden = true;
        }
      }
    }
    else {
      /* 保存中 */
      if (UnMHTUIManager.prog) {
        UnMHTUIManager.prog.hidden = false;
      }
      if (UnMHTUIManager.progbar) {
        progress /= UnMHTUIManager.contexts.length;
        
        UnMHTUIManager.progbar.hidden = false;
        UnMHTUIManager.progbar.setAttribute
        ("value", parseInt (progress * 100) + "%");
      }
      if (UnMHTUIManager.proglabel) {
        UnMHTUIManager.proglabel.hidden = true;
      }
    }
  },
  
  /**
   * 保存
   *
   * @param  Boolean isScrapBook
   *         ScrapBook 使用か
   * @param  HTMLDocument targetDocument
   *         対象のドキュメント
   *         空なら自動取得
   * @param  function callback
   *         保存終了時のコールバック
   * @param  Selection targetSelection
   *         対象の選択範囲
   * @param  Boolean isMail
   *         メールで送信するか
   */
  save : function (isScrapBook, targetDocument, callback, targetSelection,
                   isMail) {
    if (!targetDocument) {
      if (!UnMHT.isThunderbird) {
        var tabbrowser = document.getElementById ("content");
        if (!tabbrowser) {
          tabbrowser = document.getElementById ("browser_content");
        }
        targetDocument = tabbrowser.contentDocument;
      }
      else {
        var messagepane = document.getElementById ("messagepane");
        targetDocument = messagepane.contentDocument;
      }
    }
    
    var targetFilename
    = UnMHTUIManager.getFilename (targetDocument, isScrapBook);
    targetFilename += ".mht";
    
    if (!isMail && UnMHT.enableShowOptionsDialog) {
      var callback2
        = (function (filename) {
            return function (param) {
              UnMHTUIManager.save2
              (isScrapBook, targetDocument, callback, targetSelection,
               filename, isMail, param);
            };
          })(targetFilename);
      
      var pageURL = "chrome://unmht/content/saveoptions.xul";
      var features = "chrome,titlebar,centerscreen,resizable";
      
      var current, original;
      
      current = true;
      original = true;
      
      if (isScrapBook) {
        current = false;
      }
      if (UnMHT.isThunderbird) {
        original = false;
      }
      if (targetSelection) {
        original = false;
      }
      
      var param = {
        callback : callback2,
        filename : targetFilename,
        showFilename : isMail,
        enableCurrent : current,
        enableOriginal : original
      };
      
      openDialog (pageURL, "", features, param);
    }
    else {
      UnMHTUIManager.save2
      (isScrapBook, targetDocument, callback, targetSelection,
       targetFilename, isMail, null);
    }
  },
  
  getLastSaveDir : function () {
    if (UnMHT.enableUseGlobalLastDir) {
      if (UnMHT.config.prefBranch.prefHasUserValue ("browser.download.lastDir")) {
        lastDir
        = UnMHT.config.prefBranch.getComplexValue
        ("browser.download.lastDir", Components.interfaces.nsILocalFile);
        if (lastDir) {
          lastDir = lastDir.path;
        }
        else {
          lastDir = "";
        }
      }
    }
    else {
      lastDir
      = UnMHT.config
      .initPref (null, "char", "unmht.save.lastdir", "");
      lastDir = unescape (lastDir);
    }
    
    return lastDir;
  },
  
  setLastSaveDir : function (lastDir) {
    if (UnMHT.enableUseGlobalLastDir) {
      var dir
        = Components.classes ["@mozilla.org/file/local;1"]
        .createInstance (Components.interfaces.nsILocalFile);
      dir.initWithPath (lastDir);
      
      UnMHT.config.prefBranch.setComplexValue
      ("browser.download.lastDir", Components.interfaces.nsILocalFile, dir);
    }
    else {
      UnMHT.config.prefBranch.setCharPref
      ("unmht.save.lastdir", escape (lastDir));
    }
  },
  
  /**
   * 保存
   * オプションダイアログ後
   *
   * @param  Boolean isScrapBook
   *         ScrapBook 使用か
   * @param  HTMLDocument targetDocument
   *         対象のドキュメント
   *         空なら自動取得
   * @param  function callback
   *         保存終了時のコールバック
   * @param  Selection targetSelection
   *         対象の選択範囲
   * @param  String targetFilename
   *         ファイル名
   * @param  Boolean isMail
   *         メールで送信するか
   * @param  Object saveParam
   *         追加パラメータ
   */
  save2 : function (isScrapBook, targetDocument, callback, targetSelection,
                    targetFilename, isMail, saveParam) {
    UnMHTSave.init ();
    var saveContext = new UnMHTSaveContext ();
    var targetFile = null;
    var useCurrentState = false;
    var useAdditionalFiles = false;
    var additionalFiles = null;
    
    if (isMail) {
      useCurrentState = UnMHT.sendmailSaveType;
      useAdditionalFiles = false;
      additionalFiles = new Array ();
      
      var fpHandler
        = Components.classes ["@mozilla.org/network/io-service;1"]
        .getService (Components.interfaces.nsIIOService)
        .getProtocolHandler ("file")
        .QueryInterface (Components.interfaces.nsIFileProtocolHandler);
      
      var tmpDir
        = Components.classes ["@mozilla.org/file/directory_service;1"]
        .getService (Components.interfaces.nsIProperties)
        .get ("TmpD", Components.interfaces.nsIFile);
      
      var tmpURI
        = Components
        .classes ["@mozilla.org/network/standard-url;1"]
        .createInstance (Components.interfaces.nsIURI);
      tmpURI.spec = fpHandler.getURLSpecFromFile (tmpDir);
      
      var filename = UnMHT.conv.convertFromUnicode (targetFilename, "utf-8");
      var targetURI = tmpURI.resolve (filename);
      
      targetFile = fpHandler.getFileFromURLSpec (targetURI);
      
      var title = UnMHT.conv.convertFromUnicode (targetDocument.title, "utf-8");
      
      var mailCallback
        = (function (targetPath, targetURI) {
            return function (isStopped) {
              if (isStopped) {
                return;
              }
              UnMHTMail.launch
              (UnMHT.sendmailApp,
               UnMHT.sendmailArg,
               targetPath,
               targetURI,
               title)
            };
          })(targetFile.path, targetURI);
      
      if (callback) {
        callback
          = (function (mc, c) {
              c ();
              mc ();
            })(mailCallback, callback);
      }
      else {
        callback = mailCallback;
      }
    }
    else {
      var current, original;
      var index;
      var currentIndex, originalIndex;
      var currentAddIndex, originalAddIndex;
    
      current = true;
      original = true;
        
      var lastType
      = UnMHT.config
      .initPref ("char", "unmht.save.type", "current");
      
      if (isScrapBook) {
        current = false;
      }
      if (UnMHT.isThunderbird) {
        original = false;
      }
      if (targetSelection) {
        original = false;
      }
      
      var filePicker
      = Components.classes ["@mozilla.org/filepicker;1"]
      .createInstance (Components.interfaces.nsIFilePicker);
      var mode;
      filePicker.init (window,
                       UnMHT.messages.GetStringFromName ("save_title"),
                       Components.interfaces.nsIFilePicker.modeSave);
    
      var lastDir = UnMHTUIManager.getLastSaveDir ();
      if (lastDir) {
        try {
          var dir
            = Components.classes ["@mozilla.org/file/local;1"]
            .createInstance (Components.interfaces.nsILocalFile);
          dir.initWithPath (lastDir);
          filePicker.displayDirectory = dir;
        }
        catch (e) {
        }
      }
    
      if (UnMHT.enableLocalToSame) {
        var original = targetDocument.location.href;
        if (UnMHT.protocolHandler.isUnMHTURI (original)) {
          var param = UnMHT.protocolHandler.getUnMHTURIParam (original);
          original = param.original;
        }
      
        if (original.match (/^file:\/\/\//)) {
          /* ローカルファイルである */
          var file = UnMHTFileManager.getFileFromURLSpec (original);
          filePicker.displayDirectory = file.parent;
        }
      }
    
      if (saveParam) {
        if (saveParam.useCurrentState) {
          if (saveParam.additionalFiles.length > 0) {
            filePicker.appendFilter
            (UnMHT.messages.GetStringFromName ("filetype")
             + " " + UnMHT.messages.GetStringFromName ("filetype_current_add"),
             "*.mht");
          }
          else {
            filePicker.appendFilter
            (UnMHT.messages.GetStringFromName ("filetype")
             + " " + UnMHT.messages.GetStringFromName ("filetype_current"),
             "*.mht");
          }
        }
        else {
          if (saveParam.additionalFiles.length > 0) {
            filePicker.appendFilter
            (UnMHT.messages.GetStringFromName ("filetype")
             + " " + UnMHT.messages.GetStringFromName ("filetype_original"),
             "*.mht");
          }
          else {
            filePicker.appendFilter
            (UnMHT.messages.GetStringFromName ("filetype")
             + " " + UnMHT.messages.GetStringFromName ("filetype_original_add"),
             "*.mht");
          }
        }
      }
      else {
        index = 0;
        if (current) {
          filePicker.appendFilter
            (UnMHT.messages.GetStringFromName ("filetype")
             + " " + UnMHT.messages.GetStringFromName ("filetype_current"),
             "*.mht");
          currentIndex = index;
          index ++;
        }
        else {
          currentIndex = -1;
        }
        if (original) {
          filePicker.appendFilter
          (UnMHT.messages.GetStringFromName ("filetype")
           + " " + UnMHT.messages.GetStringFromName ("filetype_original"),
           "*.mht");
          originalIndex = index;
          index ++;
        }
        else {
          originalIndex = -2;
        }
        if (current) {
          filePicker.appendFilter
          (UnMHT.messages.GetStringFromName ("filetype")
           + " "
           + UnMHT.messages.GetStringFromName ("filetype_current_add"),
           "*.mht");
          currentAddIndex = index;
          index ++;
        }
        else {
          currentAddIndex = -1;
        }
        if (original) {
          filePicker.appendFilter
          (UnMHT.messages.GetStringFromName ("filetype")
           + " "
           + UnMHT.messages.GetStringFromName ("filetype_original_add"),
           "*.mht");
          originalAddIndex = index;
          index ++;
        }
        else {
          originalAddIndex = -2;
        }
        
        if (lastType == "current") {
          filePicker.filterIndex = currentIndex;
        }
        else if (lastType == "original") {
          filePicker.filterIndex = originalIndex;
        }
        else if (lastType == "current_add") {
          filePicker.filterIndex = currentAddIndex;
        }
        else if (lastType == "original_add") {
          filePicker.filterIndex = originalAddIndex;
        }
      }
      filePicker.defaultString = targetFilename;
    
      var ret = filePicker.show ();
      if (ret == Components.interfaces.nsIFilePicker.returnOK ||
          ret == Components.interfaces.nsIFilePicker.returnReplace) {
        if (saveParam) {
          useCurrentState = saveParam.useCurrentState;
          useAdditionalFiles = false;
          additionalFiles = saveParam.additionalFiles;
        
          saveContext.enableInsertLink = saveParam.enableInsertLink;
        }
        else {
          if (filePicker.filterIndex == currentIndex) {
            useCurrentState = true;
            UnMHT.config
              .setPref ("char", "unmht.save.type", "current");
          }
          else if (filePicker.filterIndex == originalIndex) {
            UnMHT.config
              .setPref ("char", "unmht.save.type", "original");
          }
          else if (filePicker.filterIndex == currentAddIndex) {
            useCurrentState = true;
            useAdditionalFiles = true;
            UnMHT.config
              .setPref ("char", "unmht.save.type", "current_add");
          }
          else if (filePicker.filterIndex == originalAddIndex) {
            useAdditionalFiles = true;
            UnMHT.config
              .setPref ("char", "unmht.save.type", "original_add");
          }
        }
      
        targetFile
          = filePicker.file
          .QueryInterface (Components.interfaces.nsILocalFile);
            
        UnMHTUIManager.setLastSaveDir (targetFile.parent.path);
      
        targetFile.leafName
          = UnMHTFormatter.validateFileName (targetFile.leafName);
            
        if (!targetFile.leafName.match (/\.mht(ml)?$/)) {
          var filenameBase = targetFile.leafName;
          var path = targetFile.path + ".mht";
          targetFile
            = Components.classes ["@mozilla.org/file/local;1"]
            .createInstance (Components.interfaces.nsILocalFile);
          targetFile.initWithPath (path);
          
          if (targetFile.exists ()) {
            /* .mht を追加した事でファイル名が被った */
            var base = targetFile.parent.path;
            var filename = targetFile.leafName;
            
            var file
              = Components.classes ["@mozilla.org/file/local;1"]
              .createInstance (Components.interfaces.nsILocalFile);
            file.initWithPath (base);
            file.appendRelativePath (filename);
            if (file.exists ()) {
              var filename2 = filename;
              var i = UnMHT.config.initPref ("int", "unmht.save.dup.init", 2);
              var file2
                = Components.classes ["@mozilla.org/file/local;1"]
                .createInstance (Components.interfaces.nsILocalFile);
              file2.initWithPath (base);
              file2.appendRelativePath (filename2);
              while (file2.exists ()) {
                filename2 = filenameBase + "-" + UnMHT.padDup (i) + ".mht";
                file2
                  = Components.classes ["@mozilla.org/file/local;1"]
                  .createInstance (Components.interfaces.nsILocalFile);
                file2.initWithPath (base);
                file2.appendRelativePath (filename2);
                i ++;
              }
        
              var confirmURL = "chrome://unmht/content/confirm.xul";
              var features = "chrome,dialog,modal,centerscreen";
              var m
                = Components.classes ["@mozilla.org/intl/stringbundle;1"]
                .getService (Components.interfaces.nsIStringBundleService)
                .createBundle ("chrome://unmht/locale/unmht.properties");
              var mtime = new Date ();
              mtime.setTime (file.lastModifiedTime);
              var result = {
                result : 0,
                applyToAll: false,
                rename : false,
                filename : ""
              };
              var param = {
                result: result,
                title: m.GetStringFromName ("overwrite_title"),
                message1: m.GetStringFromName ("overwrite_pre")
                + " " + filename + " "
                + m.GetStringFromName ("overwrite_post"),
                message2: m.GetStringFromName ("overwrite_lastaccess")
                + " " + mtime,
                message3: m.GetStringFromName ("overwrite_post2"),
                accept: m.GetStringFromName ("replace"),
                extra2: m.GetStringFromName ("abort"),
                rename: m.GetStringFromName ("rename"),
                toall: null,
                filename: filename2,
                base: base,
                exist1: m.GetStringFromName ("overwrite_pre"),
                exist2: m.GetStringFromName ("overwrite_post")
              };
              openDialog (confirmURL, "", features, param);
              if (result.result == "abort" || result.result == "no") {
                return;
              }
              if (result.result == "rename") {
                filename = result.filename;
                
                targetFile
                  = Components.classes ["@mozilla.org/file/local;1"]
                  .createInstance (Components.interfaces.nsILocalFile);
                targetFile.initWithPath (base);
                targetFile.appendRelativePath (filename);
              }
            }
          }
        }
      }
    }
    
    if (targetFile) {
      UnMHTUIManager.startSave
        (saveContext,
         targetFile, targetDocument, useCurrentState,
         isScrapBook, UnMHTUIManager.scrapBookData,
         targetSelection, useAdditionalFiles, additionalFiles,
         callback);
    }
  },
  
  /**
   * 選択範囲を保存
   */
  saveSelection : function () {
    var targetDocument;
    if (!UnMHT.isThunderbird) {
      targetDocument = document.popupNode.ownerDocument;
    }
    else {
      var messagepane = document.getElementById ("messagepane");
      targetDocument = messagepane.contentDocument;
    }
    
    var targetSelection = targetDocument.defaultView.getSelection ();
    var targetRange = targetSelection.getRangeAt (0);
    targetDocument = targetRange.startContainer.ownerDocument;
    
    UnMHTUIManager.save (false, targetDocument, null, targetSelection);
  },
    
  /**
   * リンク先を保存
   */
  saveLink : function () {
    var node = document.popupNode;
    if (!node) {
      return;
    }
    var href = node.href;
    if (!href) {
      return;
    }
        
    var iframe = document.createElement ("iframe");
    var f = function (isStopped) {
      iframe.parentNode.removeChild (iframe);
    };
    
    iframe.hidden = true;
    iframe.setAttribute ("src", href);
    iframe.setAttribute ("type", "content");
    iframe.addEventListener
    ("load",
     function () {
      UnMHTUIManager.save (false, iframe.contentDocument, f);
    }, true);
        
    var appcontent = document.getElementById ("appcontent");
    if (appcontent) {
      appcontent.parentNode.insertBefore (iframe, appcontent);
    }
  },
    
  /**
   * まとめて保存
   */
  savemulti : function () {
    var pageURL = "chrome://unmht/content/savemulti.xul";
    var features = "chrome,titlebar,centerscreen,resizable";
        
    var targetWindow
    = Components.classes ["@mozilla.org/appshell/window-mediator;1"]
    .getService (Components.interfaces.nsIWindowMediator)
    .getMostRecentWindow ("navigator:browser");
        
    var param = {
      targetWindow : targetWindow,
      saveFilename : UnMHT.saveFilename
    };
        
    openDialog (pageURL, "", features, param);
  },
    
    
  /**
   * Quick Save
   */
  quicksave : function () {
    var targetDocument;
        
    if (!UnMHT.isThunderbird) {
      var tabbrowser = document.getElementById ("content");
      if (!tabbrowser) {
        tabbrowser = document.getElementById ("browser_content");
      }
      targetDocument = tabbrowser.contentDocument;
    }
    else {
      var messagepane = document.getElementById ("messagepane");
      targetDocument = messagepane.contentDocument;
    }
    
    if (!UnMHT.isSavable (targetDocument)) {
      var m
      = Components.classes ["@mozilla.org/intl/stringbundle;1"]
      .getService (Components.interfaces.nsIStringBundleService)
      .createBundle ("chrome://unmht/locale/unmht.properties");
      alert (m.GetStringFromName ("not_savable"));
      return;
    }
    
    var useCurrentState
    = (UnMHT.quicksaveSaveType == "current");
    var base = UnMHT.quicksaveBase;
    var action = UnMHT.quicksaveAction;
    
    var fail = false;
    try {
      var file
        = Components.classes ["@mozilla.org/file/local;1"]
        .createInstance (Components.interfaces.nsILocalFile);
      file.initWithPath (base);
      if (!file.exists ()) {
        fail = true;
      }
    }
    catch (e) {
      fail = true;
    }
    if (fail || base == "") {
      var deskdir
      = Components
      .classes ["@mozilla.org/file/directory_service;1"]
      .getService (Components.interfaces.nsIProperties)
      .get ("Desk", Components.interfaces.nsIFile);
      base = deskdir.path;
    }
    
    if (UnMHT.enableLocalToSame) {
      var original = targetDocument.location.href;
      if (UnMHT.protocolHandler.isUnMHTURI (original)) {
        var param = UnMHT.protocolHandler.getUnMHTURIParam (original);
        original = param.original;
      }
      
      if (original.match (/^file:\/\/\//)) {
        /* ローカルファイルである */
        var file = UnMHTFileManager.getFileFromURLSpec (original);
        base = file.parent.path;
      }
    }
    
    var filenameBase = UnMHTUIManager.getFilename (targetDocument, false);
    var filename = filenameBase + ".mht";
    
    if (action == "rename") {
      var i = UnMHT.config.initPref ("int", "unmht.save.dup.init", 2);
      var file
        = Components.classes ["@mozilla.org/file/local;1"]
        .createInstance (Components.interfaces.nsILocalFile);
      file.initWithPath (base);
      file.appendRelativePath (filename);
      while (file.exists ()) {
        filename = filenameBase + "-" + UnMHT.padDup (i) + ".mht";
        var file
          = Components.classes ["@mozilla.org/file/local;1"]
          .createInstance
          (Components.interfaces.nsILocalFile);
        file.initWithPath (base);
        file.appendRelativePath (filename);
        i ++;
      }
    }
    else if (action == "ask") {
      var file
      = Components.classes ["@mozilla.org/file/local;1"]
      .createInstance (Components.interfaces.nsILocalFile);
      file.initWithPath (base);
      file.appendRelativePath (filename);
      if (file.exists ()) {
        var filename2 = filename;
        var i = UnMHT.config.initPref ("int", "unmht.save.dup.init", 2);
        var file2
          = Components.classes ["@mozilla.org/file/local;1"]
          .createInstance (Components.interfaces.nsILocalFile);
        file2.initWithPath (base);
        file2.appendRelativePath (filename2);
        while (file2.exists ()) {
          filename2 = filenameBase + "-" + UnMHT.padDup (i) + ".mht";
          file2
            = Components.classes ["@mozilla.org/file/local;1"]
            .createInstance (Components.interfaces.nsILocalFile);
          file2.initWithPath (base);
          file2.appendRelativePath (filename2);
          i ++;
        }
        
        var confirmURL = "chrome://unmht/content/confirm.xul";
        var features = "chrome,dialog,modal,centerscreen";
        var m
          = Components.classes ["@mozilla.org/intl/stringbundle;1"]
          .getService (Components.interfaces.nsIStringBundleService)
          .createBundle ("chrome://unmht/locale/unmht.properties");
        var mtime = new Date ();
        mtime.setTime (file.lastModifiedTime);
        var result = {
          result : 0,
          applyToAll: false,
          rename : false,
          filename : ""
        };
        var param = {
          result: result,
          title: m.GetStringFromName ("overwrite_title"),
          message1: m.GetStringFromName ("overwrite_pre")
          + " " + filename + " "
          + m.GetStringFromName ("overwrite_post"),
          message2: m.GetStringFromName ("overwrite_lastaccess")
          + " " + mtime,
          message3: m.GetStringFromName ("overwrite_post2"),
          accept: m.GetStringFromName ("replace"),
          extra2: m.GetStringFromName ("abort"),
          rename: m.GetStringFromName ("rename"),
          toall: null,
          filename: filename2,
          base: base,
          exist1: m.GetStringFromName ("overwrite_pre"),
          exist2: m.GetStringFromName ("overwrite_post")
        };
        openDialog (confirmURL, "", features, param);
        if (result.result == "abort" || result.result == "no") {
          return;
        }
        if (result.result == "rename") {
          filename = result.filename;
        }
      }
    }
        
    var file
    = Components.classes ["@mozilla.org/file/local;1"]
    .createInstance (Components.interfaces.nsILocalFile);
    file.initWithPath (base);
    file.appendRelativePath (filename);
    
    UnMHTSave.init ();
    var saveContext = new UnMHTSaveContext ();
    
    var targetSelection = null;
    if (UnMHT.enableQuicksaveSelection) {
      targetSelection = targetDocument.defaultView.getSelection ();
      var text = targetSelection.toString ();
      if (text.replace (/[ \t\r\n]/g, "") == "") {
        targetSelection = null;
      }
    }
    
    UnMHTUIManager.startSave
    (saveContext,
     file, targetDocument, useCurrentState,
     false, null,
     targetSelection, false, null,
     null);
  },
    
  /**
   * 保存する際のファイル名を取得する
   *
   * @param  HTMLDocument targetDocument
   *         対象のドキュメント
   * @param  Boolean isScrapBook
   *         ScrapBook 使用か
   */
  getFilename : function (targetDocument, isScrapBook) {
    var filename;
    var title, uri;
        
    filename = "";
    
    if (isScrapBook) {
      uri = UnMHTUIManager.scrapBookData ["source"];
      title = UnMHTUIManager.scrapBookData ["title"];
    }
    else {
      uri = targetDocument.location.href;
      title = targetDocument.title;
    }
    
    if (UnMHT.enableLocalToSame
        && UnMHT.enableLocalToSameAppend) {
      var original = uri;
      if (UnMHT.protocolHandler.isUnMHTURI (original)) {
        var param = UnMHT.protocolHandler.getUnMHTURIParam (original);
        original = param.original;
      }
      if (original.match (/^file:\/\/\//)) {
        /* ローカルファイルである */
        var file = UnMHTFileManager.getFileFromURLSpec (original);
        filename = file.leafName;
        if (filename.match (/^(.+)\.[^\.]+$/)) {
          filename = RegExp.$1;
        }
        
        return filename + UnMHT.localToSameAppendText;
      }
    }
    
    if (uri.match (/[^\?]+\/([^\/\?]+)(\?.+)?(#.+)?$/)) {
      filename = RegExp.$1;
      if (filename.match (/^(.+)\.[^\.]+$/)) {
        filename = RegExp.$1;
      }
    }
    else {
      filename = "index";
    }
        
    if (UnMHT.saveFilename == "title") {
      var tmp = UnMHTFormatter.validateFileName (title);
            
      if (tmp) {
        filename = tmp;
      }
    }
    else if (UnMHT.saveFilename == "custom") {
      var pattern = UnMHT.saveFilenameCustom;
      var am = UnMHT.saveFilenameCustomAM;
      var pm = UnMHT.saveFilenameCustomPM;
            
      filename
      = UnMHTFormatter.format (pattern, title, filename, uri, am, pm);
    }
    else {
      filename = UnMHTFormatter.validateFileName (filename);
    }
        
    return filename;
  },
    
  /**
   * サイトを開く
   */
  openWebsite : function () {
    var tabbrowser = document.getElementById ("content");
    if (!tabbrowser) {
      tabbrowser = document.getElementById ("browser_content");
    }
    var locale = "ja-JP";
    try {
      var name = "general.useragent.locale";
      locale = UnMHT.config.prefBranch.getCharPref (name);
    }
    catch (e) {
    }
    var uri;
    if (locale.match (/^ja/)) {
      uri = "http://www.unmht.org/unmht/";
    }
    else {
      uri = "http://www.unmht.org/unmht/en_index.html";
    }
    if ("addTab" in tabbrowser) {
      var newTab = tabbrowser.addTab (uri);
      tabbrowser.selectedTab = newTab;
    }
    else {
      tabbrowser.contentDocument.location.href = uri;
    }
  },
    
  /**
   * ステータスバー、ツールバーのパネルを表示する
   */
  showPanel : function () {
    UnMHT.getConfigurationFromPreferencesPanel ();
        
    if (UnMHT.enableToolbarPreferences) {
      var style = -1;
      if (UnMHT.config.prefBranch
          .prefHasUserValue ("browser.chrome.toolbar_style")) {
        style
          = UnMHT.config.prefBranch
          .getIntPref ("browser.chrome.toolbar_style");
      }
      var navbar;
      var button;
      var inner;
      navbar = document.getElementById ("nav-bar");
      inner = document.getElementById ("nav-bar-inner");
      if (navbar && inner) {
        if (UnMHT.enableToolbarPreferences) {
          button = document.createElement ("toolbarbutton");
          var id = "";
          if (style != 1) {
            id = "unmht-toolbarbutton-preferences-image";
          }
          else {
            id = "unmht-toolbarbutton-preferences-text";
          }
          button.setAttribute ("id", id);
          button.setAttribute ("oncommand",
                               "UnMHTUIManager.showPreferences (arguments [0], true);");
          button.setAttribute ("class", "toolbarbutton-1");
          button.setAttribute ("status", UnMHT.enableAll);
          if (style != 0) {
            button.setAttribute ("label", "UnMHT");
          }
          button.setAttribute ("tooltiptext", "UnMHT");
                    
          navbar.insertBefore (button, inner);
        }
      }
    }
        
    var panel;
    panel = document.getElementById ("unmht-statusbarpanel-preferences");
    if (panel) {
      panel.hidden = !UnMHT.enableStatusbarPreferences;
    }
        
    var popup;
    popup = document.getElementById ("unmht-statusbar-popup");
    if (!popup) {
      /* Mozilla Suite では mainPopupSet が無いためオーバーレイできない */
            
      var mainWindow = document.getElementById ("main-window");
      if (mainWindow) {
        var targetDocument = mainWindow.ownerDocument;
                
        var popupset = targetDocument.createElement ("popupset");
                
        popup = targetDocument.createElement ("popup");
        popup.id = "unmht-statusbar-popup";
        popup.setAttribute ("position", "after_start");
        popup.setAttribute ("onpopupshowing",
                            "UnMHTUIManager.setStatusbarPopup ();");
                
        var menuitem;
        var menuseparator;
        var label, command;
                
        /* 読み込めていない事があるので読み直す */
        UnMHT.messages
          = Components.classes ["@mozilla.org/intl/stringbundle;1"]
          .getService (Components.interfaces.nsIStringBundleService)
          .createBundle ("chrome://unmht/locale/unmht.properties");
                
        label
          = UnMHT.messages.GetStringFromName ("turnoff");
        command = "UnMHTUIManager.switchDisabled ();";
        menuitem = targetDocument.createElement ("menuitem");
        menuitem.id = "unmht-statusbar-popup-all";
        menuitem.setAttribute ("label", label);
        menuitem.setAttribute ("oncommand", command);
        popup.appendChild (menuitem);
                
        menuseparator = targetDocument.createElement ("menuseparator");
        menuseparator.id = "unmht-statusbar-menuitem-separator";
        popup.appendChild (menuseparator);
                
        label
          = UnMHT.messages.GetStringFromName ("extract");
        Command = "UnMHT.loadMHTFromFocusedWindow (1);";
        menuitem = targetDocument.createElement ("menuitem");
        menuitem.setAttribute ("label", label);
        menuitem.setAttribute ("oncommand", command);
        popup.appendChild (menuitem);
                
        label
          = UnMHT.messages.GetStringFromName ("unextract");
        command = "UnMHT.loadMHTFromFocusedWindow (2);";
        menuitem = targetDocument.createElement ("menuitem");
        menuitem.setAttribute ("label", label);
        menuitem.setAttribute ("oncommand", command);
        popup.appendChild (menuitem);
                
        label
          = UnMHT.messages.GetStringFromName ("showcache");
        command = "UnMHTUIManager.openCache ();";
        menuitem = targetDocument.createElement ("menuitem");
        menuitem.setAttribute ("label", label);
        menuitem.setAttribute ("oncommand", command);
        popup.appendChild (menuitem);
                
        label
          = UnMHT.messages.GetStringFromName ("clearcache");
        command = "UnMHTUIManager.confirmClearCache ();";
        menuitem = targetDocument.createElement ("menuitem");
        menuitem.setAttribute ("label", label);
        menuitem.setAttribute ("oncommand", command);
        popup.appendChild (menuitem);
                
        menuseparator = targetDocument.createElement ("menuseparator");
        menuseparator.id = "unmht-statusbar-menuitem-separator2";
        popup.appendChild (menuseparator);
                
        if (Components.classes ["@mozilla.org/binaryinputstream;1"]
            != undefined) {
          label
            = UnMHT.messages.GetStringFromName ("save_title");
          command = "UnMHTUIManager.save (false);";
          menuitem = targetDocument.createElement ("menuitem");
          menuitem.setAttribute ("label", label);
          menuitem.setAttribute ("oncommand", command);
          popup.appendChild (menuitem);
                    
          menuseparator
            = targetDocument.createElement ("menuseparator");
          menuseparator.id = "unmht-statusbar-menuitem-separator3";
          popup.appendChild (menuseparator);
        }
                
        label
          = UnMHT.messages.GetStringFromName ("openwebsite");
        command = "UnMHTUIManager.openWebsite ();";
        menuitem = targetDocument.createElement ("menuitem");
        menuitem.setAttribute ("label", label);
        menuitem.setAttribute ("oncommand", command);
        popup.appendChild (menuitem);
                
        popupset.appendChild (popup);
                
        mainWindow.appendChild (popupset);
      }
    }
  },
    
  /**
   * パネルのアイコンを、全機能の ON／OFF に合わせて切り替える
   */
  setPanelStatus : function () {
    UnMHT.config.loadPrefBranch ();
        
    var enableAll
    = UnMHT.config
    .initPref ("bool", "unmht.all", true);
        
    var panel;
    if (enableAll) {
      panel
        = document.getElementById ("unmht-statusbarpanel-preferences");
      if (panel) {
        panel.setAttribute ("status", "enabled");
        var text = panel.getAttribute ("tooltiptext");
        if (text.indexOf (UnMHTVersion) == -1) {
          panel.setAttribute ("tooltiptext",
                              text + " " + UnMHTVersion);
        }
      }
      panel
        = document.getElementById ("unmht-toolbarbutton-preferences");
      if (panel) {
        var text = panel.getAttribute ("tooltiptext");
        if (text.indexOf (UnMHTVersion) == -1) {
          panel.setAttribute ("tooltiptext",
                              text + " " + UnMHTVersion);
        }
      }
      panel
        = document.getElementById ("unmht-toolbarbutton-preferences");
      if (panel) {
        panel.setAttribute ("status", "enabled");
      }
      panel
        = document.getElementById ("unmht-toolbarbutton-save");
      if (panel) {
        panel.setAttribute ("status", "enabled");
      }
      panel
        = document
        .getElementById ("unmht-toolbarbutton-preferences-image");
      if (panel) {
        panel.setAttribute ("status", "enabled");
      }
    }
    else {
      panel
      = document.getElementById ("unmht-statusbarpanel-preferences");
      if (panel) {
        panel.setAttribute ("status", "disabled");
      }
      panel
      = document.getElementById ("unmht-toolbarbutton-preferences");
      if (panel) {
        panel.setAttribute ("status", "disabled");
      }
      panel
      = document.getElementById ("unmht-toolbarbutton-save");
      if (panel) {
        panel.setAttribute ("status", "disabled");
      }
      panel
      = document.getElementById ("unmht-toolbarbutton-preferences-image");
      if (panel) {
        panel.setAttribute ("status", "disabled");
      }
    }
  }
};
