/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* Unicode をエスケープ解除できない場合に unescape を定義しなおす */
if (unescape ("%u3042") == "%u3042") {
  unescape = function (text) {
    var converter
    = Components.classes ["@mozilla.org/intl/scriptableunicodeconverter"]
    .getService (Components.interfaces.nsIScriptableUnicodeConverter);
    converter.charset = "utf-8";
        
    text = text
    .replace (/((%[0-9A-Fa-f][0-9A-Fa-f])+)|%u([0-9A-Fa-f][0-9A-Fa-f][0-9A-Fa-f][0-9A-Fa-f])/g,
              function (match, part1, part2, part3) {
                if (part1) {
                  var t = part1
                  .replace (/%([0-9A-Fa-f][0-9A-Fa-f])/g,
                            function (submatch, subpart1) {
                              return String
                              .fromCharCode (parseInt ("0x"
                                                       + subpart1));
                            });
                  try {
                    t = converter.ConvertToUnicode (t);
                  }
                  catch (e) {
                  }
                  return t;
                }
                else {
                  return String
                  .fromCharCode (parseInt ("0x" + part3));
                }
              });
    return text;
  };
}

/**
 * 設定管理
 **/
var UnMHTOptions = {
  prefBranch : null,    /* nsIPrefBranch/nsIPrefBranch2  pref サービス */
  
  /**
   * 初期化処理
   */
  init : function () {
    UnMHTOptions.messages
    = Components.classes ["@mozilla.org/intl/stringbundle;1"]
    .getService (Components.interfaces.nsIStringBundleService)
    .createBundle ("chrome://unmht/locale/unmht.properties");
        
    try {
      var id
        = Components.classes ["@mozilla.org/xre/app-info;1"]
        .getService (Components.interfaces.nsIXULAppInfo).ID;
      if (id == "{3550f703-e582-4d05-9a08-453d09bdfdc6}") {
        /* Thunderbird */
        UnMHTOptions.isThunderbird = true;
      }
    } catch (e) {}
        
    if (UnMHTOptions.isThunderbird) {
      document.getElementById ("tab").selectedTab
      = document.getElementById ("tab_other_tab");
            
      document.getElementById ("tab_general_tab").hidden = true;
      document.getElementById ("tab_filter_tab").hidden = true;
      document.getElementById ("tab_save_tab").hidden = true;
      document.getElementById ("toolbar_group").hidden = true;
      document.getElementById ("all").hidden = true;
    }
        
    UnMHTOptions.loadPrefs (null);
  },
    
  /**
   * 適用
   */
  apply : function () {
    UnMHTOptions.savePrefs (null);
  },
    
  /**
   * 終了処理
   *
   * @return Boolean
   *         ダイアログを閉じるか
   */
  term : function () {
    if (UnMHTOptions.checkFocus ()) {
      return UnMHTOptions.savePrefs (null);
    }
    else {
      return false;
    }
  },
    
  /**
   * 設定を読み込む
   * 設定が無ければ既定値を書き込む
   *
   * @param Object map
   *        設定のマップ
   *        prefs.js から読む場合は null
   *        <String 設定名, String 設定値>
   * @param  String type
   *         設定の種類
   *          "bool"
   *          "char"
   *          "int"
   * @param  String name
   *         設定名
   * @param  Boolean/String/Number value
   *         既定値
   * @return Boolean/String/Number
   *         取得した値
   *         設定が無ければ既定値
   */
  initPref : function (map, type, name, value) {
    if (map == null) {
      if (UnMHTOptions.prefBranch == null) {
        if (Components.interfaces.nsIPrefBranch2) {
          UnMHTOptions.prefBranch
          = Components.classes ["@mozilla.org/preferences-service;1"]
          .getService (Components.interfaces.nsIPrefBranch2);
        }
        else {
          UnMHTOptions.prefBranch
          = Components.classes ["@mozilla.org/preferences-service;1"]
          .getService (Components.interfaces.nsIPrefBranch);
        }
      }
      
      if (UnMHTOptions.prefBranch.prefHasUserValue (name)) {
        ; /* switch のインデント用 */
        switch (type) {
          case "bool":
            value = UnMHTOptions.prefBranch.getBoolPref (name);
            break;
          case "char":
            value = UnMHTOptions.prefBranch.getCharPref (name);
            break;
          case "int":
            value = UnMHTOptions.prefBranch.getIntPref (name);
            break;
        }
      }
      else {
        ; /* switch のインデント用 */
        switch (type) {
          case "bool":
            UnMHTOptions.prefBranch.setBoolPref (name, value);
            break;
          case "char":
            UnMHTOptions.prefBranch.setCharPref (name, value);
            break;
          case "int":
            UnMHTOptions.prefBranch.setIntPref (name, value);
            break;
        }
      }
    }
    else {
      for (var name2 in map) {
        if (name2 == name) {
          ; /* switch のインデント用 */
          switch (type) {
            case "bool":
              value = (map [name2] == "true");
              break;
            case "char":
              value = map [name2];
              break;
            case "int":
              value = parseInt (map [name2]);
              break;
          }
          break;
        }
      }
    }
        
    return value;
  },
    
  /**
   * 設定を読み込む
   *
   * @param Object map
   *        設定のマップ
   *        prefs.js から読む場合は null
   *        <String 設定名, String 設定値>
   */
  loadPrefs : function (map) {
    if (map == null) {
      document.getElementById ("version").value
      += UnMHTVersion;
            
      var button;
            
      button = document
      .getElementById ("unmht_preferences_dialog")
      .getButton ("extra1");
      button.addEventListener
      ("command",
       function () {
        UnMHTOptions.apply ();
      }, true);
            
      button = document
      .getElementById ("unmht_preferences_dialog")
      .getButton ("accept");
      button.addEventListener
      ("mousedown",
       function () {
        arguments [0].explicitOriginalTarget.focus ();
      }, true);
    }
        
    document.getElementById ("all").checked
    = UnMHTOptions.initPref (map, "bool", "unmht.all", true);
        
    document.getElementById ("nolimit").checked
    = UnMHTOptions.initPref (map, "bool", "unmht.nolimit", true);
    document.getElementById ("nolimit_time").value
    = UnMHTOptions.initPref (map, "int",  "unmht.nolimit.time", 0);
    document.getElementById ("fast_extract").checked
    = UnMHTOptions.initPref (map, "bool", "unmht.fast_extract", false);
        
    document.getElementById ("cache_expire_enabled").checked
    = UnMHTOptions.initPref (map, "bool", "unmht.cache_expire.enabled",
                             true);
    document.getElementById ("cache_expire").value
    = UnMHTOptions.initPref (map, "int",  "unmht.cache_expire", 5);
        
    document.getElementById ("skip_ppt_warning").checked
    = UnMHTOptions.initPref (map, "bool", "unmht.skip_ppt_warning",
                             false);

    document.getElementById ("open_multiple").value
    = UnMHTOptions.initPref (map, "char", "unmht.open_multiple",
                             "tabs");
    UnMHTOptions.selectItem (document.getElementById
                             ("open_multiple"));
        
    document.getElementById ("load_image_type").checked
    = UnMHTOptions.initPref (map, "bool", "unmht.load_image_type",
                             true);

    document.getElementById ("use_global_lastdir").checked
    = UnMHTOptions.initPref (map, "bool", "unmht.use_global_lastdir",
                             false);
    
    document.getElementById ("filter_extract").checked
    = UnMHTOptions.initPref (map, "bool", "unmht.filter", false);
    var value
    = UnMHTOptions
    .initPref (map, "char", "unmht.filter.list", "");
    var listbox = document.getElementById ("filter_extract_list");
    var node = listbox.firstChild;
    while (node) {
      var nextNode = node.nextSibling;
      if (node.nodeName == "listitem") {
        listbox.removeChild (node);
      }
      node = nextNode;
    }
    if (value != "") {
      /* 値を解析するだけなので代入はしない */
      value.replace
      (/([^&,]*)&([^&,]*)&([^&,]*),?/g,
       function (matched, type, pattern, r) {
        if  (unescape (r) == "o"
             || unescape (r) == "x") {
          UnMHTOptions
          .addFilterItem ("extract",
                          unescape (type),
                          unescape (pattern),
                          unescape (r),
                          null);
        }
      });
    }
    UnMHTOptions.checkFilter ("extract");

    document.getElementById ("filter_load").checked
    = UnMHTOptions.initPref (map, "bool", "unmht.filter.load", false);
    document.getElementById ("filter_load_offline").checked
    = UnMHTOptions.initPref (map, "bool", "unmht.filter.load.offline", false);
    var value
    = UnMHTOptions
    .initPref (map, "char", "unmht.filter.load.list", "");
    var listbox = document.getElementById ("filter_load_list");
    var node = listbox.firstChild;
    while (node) {
      var nextNode = node.nextSibling;
      if (node.nodeName == "listitem") {
        listbox.removeChild (node);
      }
      node = nextNode;
    }
    if (value != "") {
      /* 値を解析するだけなので代入はしない */
      value.replace
      (/([^&,]*)&([^&,]*)&([^&,]*),?/g,
       function (matched, type, pattern, r) {
        if  (unescape (r) == "o"
             || unescape (r) == "x") {
          UnMHTOptions
          .addFilterItem ("load",
                          unescape (type),
                          unescape (pattern),
                          unescape (r),
                          null);
        }
      });
    }
    UnMHTOptions.checkFilter ("load");
        
    document.getElementById ("save_filename").value
    = UnMHTOptions.initPref (map, "char", "unmht.save.filename",
                             "title");
    UnMHTOptions.selectItem (document.getElementById
                             ("save_filename"));
    
    value
    = UnMHTOptions.initPref (map, "char", "unmht.save.filename.custom",
                             "%FILENAME%");
    document.getElementById ("save_filename_custom").value
    = unescape (value);
    value
    = UnMHTOptions.initPref (map, "char", "unmht.save.filename.custom.am",
                             "a.m.");
    document.getElementById ("save_filename_custom_am").value
    = unescape (value);
    value
    = UnMHTOptions.initPref (map, "char", "unmht.save.filename.custom.pm",
                             "p.m.");
    document.getElementById ("save_filename_custom_pm").value
    = unescape (value);

    document.getElementById ("save_filename_custom_limit").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.save.filename.custom.limit", false);
    value
    = UnMHTOptions.initPref (map, "int",
                             "unmht.save.filename.custom.limit.count",
                             100);
    document.getElementById ("save_filename_custom_limit_count").value
    = value;
    UnMHTOptions.changeSaveFilename ();
    UnMHTOptions.changeFilename ();
    
    document.getElementById ("save_contextmenu").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.save.contextmenu", true);

    document.getElementById ("save_shortcut").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.save.shortcut", false);
    value
    = UnMHTOptions
    .initPref (map, "char", "unmht.save.shortcut.keycode", "VK_S");
    document.getElementById ("save_shortcut_keycode").value
    = unescape (value);
    document.getElementById ("save_shortcut_modifiers_alt").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.save.shortcut.modifiers.alt",
               true);
    document.getElementById ("save_shortcut_modifiers_ctrl").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.save.shortcut.modifiers.ctrl",
               true);
    document.getElementById ("save_shortcut_modifiers_meta").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.save.shortcut.modifiers.meta",
               false);
    document.getElementById ("save_shortcut_modifiers_shift").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.save.shortcut.modifiers.shift",
               false);
    UnMHTOptions.checkSaveShortcut ();

    document.getElementById ("save_shortcut2").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.save.shortcut2", false);
    value
    = UnMHTOptions
    .initPref (map, "char", "unmht.save.shortcut2.keycode", "VK_S");
    document.getElementById ("save_shortcut2_keycode").value
    = unescape (value);
    document.getElementById ("save_shortcut2_modifiers_alt").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.save.shortcut2.modifiers.alt",
               true);
    document.getElementById ("save_shortcut2_modifiers_ctrl").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.save.shortcut2.modifiers.ctrl",
               true);
    document.getElementById ("save_shortcut2_modifiers_meta").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.save.shortcut2.modifiers.meta",
               false);
    document.getElementById ("save_shortcut2_modifiers_shift").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.save.shortcut2.modifiers.shift",
               false);
    UnMHTOptions.checkSaveShortcut2 ();
        
    document.getElementById ("save_inserturl").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.save.inserturl", false);
        
    document.getElementById ("save_insertlink").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.save.insertlink", false);
    value
    = UnMHTOptions
    .initPref (map, "char", "unmht.save.insertlink.pos",
               "bottom");
    document.getElementById ("save_insertlink_pos").value
    = unescape (value);
    UnMHTOptions.selectItem (document.getElementById
                             ("save_insertlink_pos"));
    
    document.getElementById ("save_insertlink_date").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.save.insertlink.date", true);
    document.getElementById ("save_insertlink_version").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.save.insertlink.version", true);
    document.getElementById ("save_insertlink_nolocal").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.save.insertlink.nolocal", false);
    UnMHTOptions.checkInsertLink ();
        
    document.getElementById ("save_show_errors").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.save.show_errors", true);
    document.getElementById ("save_statusbar").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.save.statusbar", false);
        
    document.getElementById ("save_skip_alert_on_deep").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.save.skip_alert_on_deep", false);
    value
    = UnMHTOptions
    .initPref (map, "char", "unmht.save.skip_alert_on_deep.default",
               "save");
    document.getElementById ("save_skip_alert_on_deep_default").value
    = unescape (value);
    UnMHTOptions.selectItem (document.getElementById
                             ("save_skip_alert_on_deep_default"));
    
    UnMHTOptions.checkSkipAlertOnDeep ();
        
    document.getElementById ("save_local_to_same").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.save.local_to_same", false);
    document.getElementById ("save_local_to_same_append").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.save.local_to_same.append", false);
    value
    = UnMHTOptions
    .initPref (map, "chae", "unmht.save.local_to_same.append.text", "");
    document.getElementById ("save_local_to_same_append_text").value
    = unescape (value);
    UnMHTOptions.checkSaveLocalToSame ();
    
    document.getElementById ("save_nolimit").checked
    = UnMHTOptions.initPref (map, "bool", "unmht.save.nolimit", false);
    document.getElementById ("save_nolimit_time").value
    = UnMHTOptions.initPref (map, "int",  "unmht.save.nolimit.time", 0);
    
    document.getElementById ("save_multi_autoclose").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.save.multi.autoclose", false);

    document.getElementById ("save_multi_none").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.save.multi.none", false);
        
    document.getElementById ("save_multi_autoreplace").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.save.multi.autoreplace", false);
    value
    = UnMHTOptions
    .initPref (map, "char", "unmht.save.multi.autoreplace.action",
               "rename");
    document.getElementById ("save_multi_autoreplace_action").value
    = unescape (value);
    UnMHTOptions.selectItem (document.getElementById
                             ("save_multi_autoreplace_action"));
    UnMHTOptions.checkAutoReplace ();
    
    value
    = UnMHTOptions
    .initPref (map, "char", "unmht.sendmail.savetype", "current");
    document.getElementById ("sendmail_savetype").value
    = unescape (value);
    UnMHTOptions.selectItem (document.getElementById
                             ("sendmail_savetype"));
    value
    = UnMHTOptions
    .initPref (map, "char", "unmht.sendmail.app",
               "");
    document.getElementById ("sendmail_app").value
    = unescape (value);
    value
    = UnMHTOptions
    .initPref (map, "char", "unmht.sendmail.argtype",
               "thunderbird");
    document.getElementById ("sendmail_argtype").value
    = unescape (value);
    UnMHTOptions.selectItem (document.getElementById
                             ("sendmail_argtype"));
    value
    = UnMHTOptions
    .initPref (map, "char", "unmht.sendmail.arg",
               escape (UnMHTMail.thunderbird));
    document.getElementById ("sendmail_arg").value
    = unescape (value);
    UnMHTOptions.changeSendmailArgType ();
    document.getElementById ("sendmail_debug").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.sendmail.debug", false);
    document.getElementById ("sendmail_showmenu").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.sendmail.showmenu", true);
    
    value
    = UnMHTOptions
    .initPref (map, "char", "unmht.quicksave.savetype",
               "current");
    document.getElementById ("quicksave_savetype").value
    = unescape (value);
    UnMHTOptions.selectItem (document.getElementById
                             ("quicksave_savetype"));
    
    value
    = UnMHTOptions
    .initPref (map, "char", "unmht.quicksave.base",
               "");
    document.getElementById ("quicksave_base").value
    = unescape (value);
    value
    = UnMHTOptions
    .initPref (map, "char", "unmht.quicksave.autoreplace.action",
               "rename");
    document.getElementById ("quicksave_autoreplace_action").value
    = unescape (value);
    UnMHTOptions.selectItem (document.getElementById
                             ("quicksave_autoreplace_action"));
        
    document.getElementById ("quicksave_selection").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.quicksave.selection", false);
    
    document.getElementById ("save_multi_autoreplace").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.save.multi.autoreplace", false);
        
    document.getElementById ("quicksave_showmenu").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.quicksave.showmenu", true);
    
    document.getElementById ("quicksave_shortcut").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.quicksave.shortcut", false);
    value
    = UnMHTOptions
    .initPref (map, "char", "unmht.quicksave.shortcut.keycode", "VK_Q");
    document.getElementById ("quicksave_shortcut_keycode").value
    = unescape (value);
    document.getElementById ("quicksave_shortcut_modifiers_alt").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.quicksave.shortcut.modifiers.alt",
               true);
    document.getElementById ("quicksave_shortcut_modifiers_ctrl").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.quicksave.shortcut.modifiers.ctrl",
               true);
    document.getElementById ("quicksave_shortcut_modifiers_meta").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.quicksave.shortcut.modifiers.meta",
               false);
    document.getElementById ("quicksave_shortcut_modifiers_shift").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.quicksave.shortcut.modifiers.shift",
               false);
    UnMHTOptions.checkQuicksaveShortcut ();

    document.getElementById ("save_dup_digits").value
    = UnMHTOptions
    .initPref (map, "int", "unmht.save.dup.digits", 1);

    document.getElementById ("save_dup_init").value
    = UnMHTOptions
    .initPref (map, "int", "unmht.save.dup.init", 2);

    document.getElementById ("save_register_download").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.save.register_download", true);
    document.getElementById ("show_option_dialog").checked
    = UnMHTOptions
    .initPref (map, "bool", "unmht.save.show_option_dialog", false);
        
    document.getElementById ("statusbar_preferences").checked
    = UnMHTOptions.initPref (map, "bool", "unmht.statusbar.preferences",
                             true);
    document.getElementById ("toolbar_preferences").checked
    = UnMHTOptions.initPref (map, "bool", "unmht.toolbar.preferences",
                             false);
    document.getElementById ("toolmenu_preferences").checked
    = UnMHTOptions.initPref (map, "bool", "unmht.toolmenu.preferences",
                             true);
        
    sizeToContent ();
  },
    
  /**
   * 設定を保存する
   *
   * @param  nsIFileOutputStream fstream
   *         保存先のファイル
   *         prefs.js に保存する場合は null
   * @param  String type
   *         設定の種類
   *          "bool"
   *          "char"
   *          "int"
   * @param  String name
   *         設定名
   * @param  Boolean/String/Number value
   *         既定値
   */
  setPref : function (fstream, type, name, value) {
    if (fstream == null) {
      if (UnMHTOptions.prefBranch == null) {
        if (Components.interfaces.nsIPrefBranch2) {
          UnMHTOptions.prefBranch
          = Components.classes ["@mozilla.org/preferences-service;1"]
          .getService (Components.interfaces.nsIPrefBranch2);
        }
        else {
          UnMHTOptions.prefBranch
          = Components.classes ["@mozilla.org/preferences-service;1"]
          .getService (Components.interfaces.nsIPrefBranch);
        }
      }
      
      ; /* switch のインデント用 */
      switch (type) {
        case "bool":
          UnMHTOptions.prefBranch.setBoolPref (name, value);
          break;
        case "char":
          UnMHTOptions.prefBranch.setCharPref (name, value);
          break;
        case "int":
          UnMHTOptions.prefBranch.setIntPref (name, value);
          break;
      }
    }
    else {
      var line = escape (name) + "," + escape (value) + "\r\n";
      fstream.write (line, line.length);
    }
  },
    
  /**
   * 設定を保存する
   *
   * @param  nsIFileOutputStream fstream
   *         保存先のファイル
   *         prefs.js に保存する場合は null
   * @return Boolean
   *         成功フラグ
   */
  savePrefs : function (fstream) {
    var value;
        
    if (fstream) {
      UnMHTOptions
        .setPref (fstream, "char", "unmht.version",
                  UnMHTVersion);
    }
        
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.all",
              document.getElementById ("all").checked);
        
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.nolimit",
              document.getElementById ("nolimit").checked);
    value   = document.getElementById ("nolimit_time").value
    UnMHTOptions
    .setPref (fstream, "int",  "unmht.nolimit.time",
              parseInt (value));
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.fast_extract",
              document.getElementById ("fast_extract").checked);
    value   = document.getElementById ("cache_expire").value;
    UnMHTOptions
    .setPref (fstream, "int", "unmht.cache_expire", 
              parseInt (value));
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.cache_expire.enabled",
              document.getElementById ("cache_expire_enabled").checked);

    UnMHTOptions
    .setPref (fstream, "bool", "unmht.skip_ppt_warning",
              document.getElementById ("skip_ppt_warning").checked);

    UnMHTOptions
    .setPref (fstream, "char", "unmht.open_multiple",
              document.getElementById ("open_multiple").value);
        
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.load_image_type",
              document.getElementById ("load_image_type").checked);

    UnMHTOptions
    .setPref (fstream, "bool", "unmht.use_global_lastdir",
              document.getElementById ("use_global_lastdir").checked);
    
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.filter",
              document.getElementById ("filter_extract").checked);
    var listbox = document.getElementById ("filter_extract_list");
    var tmp = "";
    var node = listbox.firstChild;
    while (node) {
      var nextNode = node.nextSibling;
      if (node.nodeName == "listitem") {
        var value = UnMHTOptions.getFilterItem (node);
                
        if (tmp != "") {
          tmp += ",";
        }
                
        tmp
          += escape (value [0])
          + "&" + escape (value [1])
          + "&" + escape (value [2]);
      }
      node = nextNode;
    }
    UnMHTOptions
    .setPref (fstream, "char", "unmht.filter.list",
              tmp);
        
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.filter.load",
              document.getElementById ("filter_load").checked);
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.filter.load.offline",
              document.getElementById ("filter_load_offline").checked);
    var listbox = document.getElementById ("filter_load_list");
    var tmp = "";
    var node = listbox.firstChild;
    while (node) {
      var nextNode = node.nextSibling;
      if (node.nodeName == "listitem") {
        var value = UnMHTOptions.getFilterItem (node);
                
        if (tmp != "") {
          tmp += ",";
        }
                
        tmp
          += escape (value [0])
          + "&" + escape (value [1])
          + "&" + escape (value [2]);
      }
      node = nextNode;
    }
    UnMHTOptions
    .setPref (fstream, "char", "unmht.filter.load.list",
              tmp);
        
    UnMHTOptions
    .setPref (fstream, "char", "unmht.save.filename",
              document.getElementById ("save_filename").value);
    
    value = document.getElementById ("save_filename_custom").value;
    UnMHTOptions
    .setPref (fstream, "char", "unmht.save.filename.custom",
              escape (value));
    value = document.getElementById ("save_filename_custom_am").value;
    UnMHTOptions
    .setPref (fstream, "char", "unmht.save.filename.custom.am",
              escape (value));
    value = document.getElementById ("save_filename_custom_pm").value;
    UnMHTOptions
    .setPref (fstream, "char", "unmht.save.filename.custom.pm",
              escape (value));
    
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.save.filename.custom.limit",
              document.getElementById ("save_filename_custom_limit").checked);
    value = document.getElementById ("save_filename_custom_limit_count").value;
    UnMHTOptions
    .setPref (fstream, "int", "unmht.save.filename.custom.limit.count",
              parseInt (value));

    UnMHTOptions
    .setPref (fstream, "bool", "unmht.save.contextmenu",
              document.getElementById ("save_contextmenu").checked);
        
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.save.shortcut",
              document.getElementById ("save_shortcut").checked);
    value   = document.getElementById ("save_shortcut_keycode").value;
    UnMHTOptions
    .setPref (fstream, "char", "unmht.save.shortcut.keycode",
              escape (value));
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.save.shortcut.modifiers.alt",
              document.getElementById ("save_shortcut_modifiers_alt")
              .checked);
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.save.shortcut.modifiers.ctrl",
              document.getElementById ("save_shortcut_modifiers_ctrl")
              .checked);
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.save.shortcut.modifiers.meta",
              document.getElementById ("save_shortcut_modifiers_meta")
              .checked);
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.save.shortcut.modifiers.shift",
              document.getElementById ("save_shortcut_modifiers_shift")
              .checked);
        
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.save.shortcut2",
              document.getElementById ("save_shortcut2").checked);
    value   = document.getElementById ("save_shortcut2_keycode").value;
    UnMHTOptions
    .setPref (fstream, "char", "unmht.save.shortcut2.keycode",
              escape (value));
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.save.shortcut2.modifiers.alt",
              document.getElementById ("save_shortcut2_modifiers_alt")
              .checked);
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.save.shortcut2.modifiers.ctrl",
              document.getElementById ("save_shortcut2_modifiers_ctrl")
              .checked);
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.save.shortcut2.modifiers.meta",
              document.getElementById ("save_shortcut2_modifiers_meta")
              .checked);
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.save.shortcut2.modifiers.shift",
              document.getElementById ("save_shortcut2_modifiers_shift")
              .checked);
        
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.save.inserturl",
              document.getElementById ("save_inserturl").checked);
        
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.save.insertlink",
              document.getElementById ("save_insertlink").checked);
    value   = document.getElementById ("save_insertlink_pos").value;
    UnMHTOptions
    .setPref (fstream, "char", "unmht.save.insertlink.pos",
              escape (value));
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.save.insertlink.date",
              document.getElementById ("save_insertlink_date").checked);
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.save.insertlink.version",
              document.getElementById ("save_insertlink_version").checked);
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.save.insertlink.nolocal",
              document.getElementById ("save_insertlink_nolocal").checked);
        
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.save.show_errors",
              document.getElementById ("save_show_errors").checked);
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.save.statusbar",
              document.getElementById ("save_statusbar").checked);

    UnMHTOptions
    .setPref (fstream, "bool", "unmht.save.skip_alert_on_deep",
              document.getElementById ("save_skip_alert_on_deep").checked);
    value
    = document.getElementById ("save_skip_alert_on_deep_default").value
    UnMHTOptions
    .setPref (fstream, "char", "unmht.save.skip_alert_on_deep.default",
              escape (value));
        
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.save.local_to_same",
              document.getElementById ("save_local_to_same").checked);
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.save.local_to_same.append",
              document.getElementById ("save_local_to_same_append").checked);
    value
    = document.getElementById ("save_local_to_same_append_text").value
    UnMHTOptions
    .setPref (fstream, "char", "unmht.save.local_to_same.append.text",
              escape (value));
    
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.save.nolimit",
              document.getElementById ("save_nolimit").checked);
    value   = document.getElementById ("save_nolimit_time").value
    UnMHTOptions
    .setPref (fstream, "int",  "unmht.save.nolimit.time",
              parseInt (value));
    
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.save.multi.autoclose",
              document.getElementById ("save_multi_autoclose").checked);

    UnMHTOptions
    .setPref (fstream, "bool", "unmht.save.multi.none",
              document.getElementById ("save_multi_none").checked);
        
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.save.multi.autoreplace",
              document.getElementById ("save_multi_autoreplace").checked);
    value
    = document.getElementById ("save_multi_autoreplace_action").value
    UnMHTOptions
    .setPref (fstream, "char", "unmht.save.multi.autoreplace.action",
              escape (value));
    
    value
    = document.getElementById ("sendmail_savetype").value;
    UnMHTOptions
    .setPref (fstream, "char", "unmht.sendmail.savetype",
              escape (value));
    value
    = document.getElementById ("sendmail_app").value
    UnMHTOptions
    .setPref (fstream, "char", "unmht.sendmail.app",
              escape (value));
    value
    = document.getElementById ("sendmail_argtype").value
    UnMHTOptions
    .setPref (fstream, "char", "unmht.sendmail.argtype",
              escape (value));
    value
    = document.getElementById ("sendmail_arg").value
    UnMHTOptions
    .setPref (fstream, "char", "unmht.sendmail.arg",
              escape (value));
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.sendmail.debug",
              document.getElementById ("sendmail_debug").checked);
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.sendmail.showmenu",
              document.getElementById ("sendmail_showmenu").checked);
    
    value
    = document.getElementById ("quicksave_savetype").value
    UnMHTOptions
    .setPref (fstream, "char", "unmht.quicksave.savetype",
              escape (value));
    value
    = document.getElementById ("quicksave_base").value
    UnMHTOptions
    .setPref (fstream, "char", "unmht.quicksave.base",
              escape (value));
    value
    = document.getElementById ("quicksave_autoreplace_action").value
    UnMHTOptions
    .setPref (fstream, "char", "unmht.quicksave.autoreplace.action",
              escape (value));
        
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.quicksave.selection",
              document.getElementById ("quicksave_selection").checked);
    
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.save.register_download",
              document.getElementById ("save_register_download").checked);
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.save.show_option_dialog",
              document.getElementById ("show_option_dialog").checked);
        
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.quicksave.showmenu",
              document.getElementById ("quicksave_showmenu").checked);
    
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.quicksave.shortcut",
              document.getElementById ("quicksave_shortcut").checked);
    value   = document.getElementById ("quicksave_shortcut_keycode").value;
    UnMHTOptions
    .setPref (fstream, "char", "unmht.quicksave.shortcut.keycode",
              escape (value));
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.quicksave.shortcut.modifiers.alt",
              document.getElementById ("quicksave_shortcut_modifiers_alt")
              .checked);
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.quicksave.shortcut.modifiers.ctrl",
              document.getElementById ("quicksave_shortcut_modifiers_ctrl")
              .checked);
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.quicksave.shortcut.modifiers.meta",
              document.getElementById ("quicksave_shortcut_modifiers_meta")
              .checked);
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.quicksave.shortcut.modifiers.shift",
              document.getElementById ("quicksave_shortcut_modifiers_shift")
              .checked);

    value = document.getElementById ("save_dup_digits").value;
    UnMHTOptions
    .setPref (fstream, "int", "unmht.save.dup.digits",
              parseInt (value, 10));

    value = document.getElementById ("save_dup_init").value;
    UnMHTOptions
    .setPref (fstream, "int", "unmht.save.dup.init",
              parseInt (value, 10));

    UnMHTOptions
    .setPref (fstream, "bool", "unmht.statusbar.preferences",
              document.getElementById ("statusbar_preferences").checked);
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.toolbar.preferences",
              document.getElementById ("toolbar_preferences").checked);
    UnMHTOptions
    .setPref (fstream, "bool", "unmht.toolmenu.preferences",
              document.getElementById ("toolmenu_preferences").checked);
        
    UnMHTOptions
    .setPref (fstream, "char", "unmht.savepref",
              new Date ().getTime ());
        
    if (fstream == null) {
      var prefService
        = Components.classes ["@mozilla.org/preferences-service;1"].
        getService (Components.interfaces.nsIPrefService);
            
      prefService.savePrefFile (null);
    }
        
    return true;
  },
  
  selectSendmailApp : function () {
    var id = "sendmail_app";
    var message
    = UnMHTOptions.messages.GetStringFromName ("sendmail_app_title");
    
    var filePicker
    = Components.classes ["@mozilla.org/filepicker;1"]
    .createInstance (Components.interfaces.nsIFilePicker);
    var flag = 0;
    flag = Components.interfaces.nsIFilePicker.modeOpen;
        
    filePicker.init (window, message,
                     flag);
        
    try {
      var filename = document.getElementById (id).value;
            
      var file
        = Components.classes ["@mozilla.org/file/local;1"]
        .createInstance (Components.interfaces.nsILocalFile);
      file.initWithPath (filename);
      
      filePicker.displayDirectory = file.parent;
    }
    catch (e) {
    }
    
    if (filePicker.show ()
        == Components.interfaces.nsIFilePicker.returnOK) {
      document.getElementById (id).value
      = filePicker.file
      .QueryInterface (Components.interfaces.nsILocalFile).path;
    }
  },
  
  selectQuicksaveBase : function () {
    var id = "quicksave_base";
    var message
    = UnMHTOptions.messages.GetStringFromName ("quicksave_base_title");
        
    var filePicker
    = Components.classes ["@mozilla.org/filepicker;1"]
    .createInstance (Components.interfaces.nsIFilePicker);
    var flag = 0;
    flag = Components.interfaces.nsIFilePicker.modeGetFolder;
    
    filePicker.init (window, message,
                     flag);
        
    try {
      var base = document.getElementById (id).value;
            
      var dir
        = Components.classes ["@mozilla.org/file/local;1"]
        .createInstance (Components.interfaces.nsILocalFile);
      dir.initWithPath (base);
            
      filePicker.displayDirectory = dir;
    }
    catch (e) {
    }
        
    if (filePicker.show ()
        == Components.interfaces.nsIFilePicker.returnOK) {
      document.getElementById (id).value
      = filePicker.file
      .QueryInterface (Components.interfaces.nsILocalFile).path;
    }
  },
  
  /**
   * メニュー, ラヂオボタンの選択肢を表示に反映する
   * (古いバージョンだと value の変更だけでは反映されない)
   *
   */
  selectItem : function (node) {
    var value = node.value;
        
    var nodes = node.getElementsByTagName ("radio");
    for (var i = 0; i < nodes.length; i ++) {
      if (nodes [i].value == value) {
        node.selectedItem = nodes [i];
      }
    }
  },
    
  /**
   * フィルタを追加するボタンのイベント
   *
   * @param  String id
   *         フィルタの対象
   *           "extract"/"load"
   * @param  Boolean modify
   *         修正するか追加するか
   *           true: 修正
   *           false: 追加
   */
  onAddFilter : function (id, modify) {
    var type = document.getElementById ("filter_" + id + "_type").value;
    var pattern = document.getElementById ("filter_" + id + "_pattern").value;
    var r;
    document.getElementById ("filter_" + id + "_illegal").value = "";
        
    if (document.getElementById ("filter_" + id + "_regexp").checked) {
      r = "o";
      try {
        var tmp = "test";
        tmp.search (pattern);
      }
      catch (e) {
        document.getElementById ("filter_" + id + "_illegal").value
          = document.getElementById ("filter_" + id + "_illegal_regexp").value;
                
        return;
      }
    }
    else {
      r = "x";
    }
    
    var listbox = document.getElementById ("filter_" + id + "_list");
    var exist = false;
        
    if (type && pattern) {
      var nextNode = null;
      var node = null;
      if (modify) {
        var node = listbox.firstChild;
        while (node) {
          if (node.nodeName == "listitem") {
            if (node.selected) {
              break;
            }
          }
          node = node.nextSibling;
        }
      }
            
      var node2 = listbox.firstChild;
      while (node2) {
        var nextNode2 = node2.nextSibling;
                
        var value = UnMHTOptions.getFilterItem (node2);
        if (type == value [0] && pattern == value [1]
            && r == value [2]
            && node2 != node) {
          exist = true;
          break;
        }
        node2 = nextNode2;
      }
            
      if (!exist) {
        UnMHTOptions.addFilterItem (id, type, pattern, r, node);
                
        if (node == null) {
          document.getElementById ("filter_" + id + "_pattern").value
            = "";
        }
      }
      else {
        document.getElementById ("filter_" + id + "_illegal").value
          = document.getElementById ("filter_" + id + "_illegal_dup").value;
      }
    }
    else {
      if (pattern == "") {
        document.getElementById ("filter_" + id + "_illegal").value
        = document.getElementById ("filter_" + id + "_illegal_nullpat").value;
      }
      else {
        document.getElementById ("filter_" + id + "_illegal").value
        = document.getElementById ("filter_" + id + "_illegal_nulllink").value;
      }
    }
  },
    
  /**
   * リストの選択項目を上にずらすボタンのイベント
   *
   * @param  String id
   *         フィルタの対象
   *           "extract"/"load"
   */
  onMoveupList : function (id) {
    var listbox = document.getElementById ("filter_" + id + "_list");
    
    var node = listbox.firstChild;
    var nextNode = null;
    var prevNode = null;
    var node1 = null;
    var node2 = null;
    var tmp = "";
    while (node) {
      nextNode = node.nextSibling;
      if (node.nodeName == "listitem") {
        if (node.selected) {
          if (prevNode) {
            node1 = node.firstChild;
            node2 = prevNode.firstChild;
            while (node1 && node2) {
              tmp = node1.getAttribute ("value");
              node1.setAttribute ("value",
                                  node2.getAttribute ("value"));
              node2.setAttribute ("value", tmp);
                            
              if (node1.getAttribute ("class")
                  == "listcell-iconic") {
                tmp = node1.getAttribute ("image");
                node1.setAttribute ("image",
                                    node2
                                    .getAttribute ("image"));
                node2.setAttribute ("image", tmp);
              }
              else {
                tmp = node1.getAttribute ("label");
                node1.setAttribute ("label",
                                    node2
                                    .getAttribute ("label"));
                node2.setAttribute ("label", tmp);
              }
                            
              node1 = node1.nextSibling;
              node2 = node2.nextSibling;
            }
            listbox.toggleItemSelection (node);
            listbox.toggleItemSelection (prevNode);
            prevNode = prevNode;
          }
        }
        else {
          prevNode = node;
        }
      }
      node = nextNode;
    }
  },
    
  /**
   * リストの選択項目を下にずらすボタンのイベント
   *
   * @param  String id
   *         フィルタの対象
   *           "extract"/"load"
   */
  onMovedownList : function (id) {
    var listbox = document.getElementById ("filter_" + id + "_list");
        
    var node = listbox.lastChild;
    var nextNode = null;
    var prevNode = null;
    var node1 = null;
    var node2 = null;
    var tmp = "";
    while (node) {
      prevNode = node.previousSibling;
      if (node.nodeName == "listitem") {
        if (node.selected) {
          if (nextNode) {
            node1 = node.firstChild;
            node2 = nextNode.firstChild;
            while (node1 && node2) {
              tmp = node1.getAttribute ("value");
              node1.setAttribute ("value",
                                  node2.getAttribute ("value"));
              node2.setAttribute ("value", tmp);
                            
              if (node1.getAttribute ("class")
                  == "listcell-iconic") {
                tmp = node1.getAttribute ("image");
                node1.setAttribute ("image",
                                    node2
                                    .getAttribute ("image"));
                node2.setAttribute ("image", tmp);
              }
              else {
                tmp = node1.getAttribute ("label");
                node1.setAttribute ("label",
                                    node2
                                    .getAttribute ("label"));
                node2.setAttribute ("label", tmp);
              }
                            
              node1 = node1.nextSibling;
              node2 = node2.nextSibling;
            }
            listbox.toggleItemSelection (node);
            listbox.toggleItemSelection (nextNode);
            nextNode = nextNode;
          }
        }
        else {
          nextNode = node;
        }
      }
      node = prevNode;
    }
  },
    
  /**
   * フィルタを削除するボタンのイベント
   *
   * @param  String id
   *         フィルタの対象
   *           "extract"/"load"
   */
  onDeleteFilter : function (id) {
    var listbox = document.getElementById ("filter_" + id + "_list");
        
    var node = listbox.firstChild;
    while (node) {
      var nextNode = node.nextSibling;
      if (node.selected) {
        listbox.removeChild (node);
      }
      node = nextNode;
    }
  },
    
  /**
   * フィルタを選択したイベント
   *
   * @param  String id
   *         フィルタの対象
   *           "extract"/"load"
   */
  selectFilter : function (id) {
    var listbox = document.getElementById ("filter_" + id + "_list");
        
    if (listbox.selectedIndex != -1) {
      var value = UnMHTOptions
        .getFilterItem (listbox.selectedItem);
            
      document.getElementById ("filter_" + id + "_type").value
        = value [0];
      document.getElementById ("filter_" + id + "_pattern").value
        = value [1];
      document.getElementById ("filter_" + id + "_regexp").checked
        = value [2] == "o";
            
      document.getElementById ("filter_" + id + "_modify").disabled
        = !document.getElementById ("filter_" + id).checked
        || (listbox.selectedCount > 1);
    }
  },
    
  /**
   * フィルタを取得する
   *
   * @param  Listitem listitem 
   *         対象の項目
   * @return Array
   *         フィルタ
   *           [String パターン,
   *            String 正規表現ならば "o" そうでなければ "x", String リンク先]
   */
  getFilterItem : function (listitem) {
    var listcell1 = listitem.firstChild;
    var listcell2 = listcell1.nextSibling;
    var listcell3 = listcell2.nextSibling;
    return new Array (listcell1.getAttribute ("value"),
                      listcell2.getAttribute ("value"),
                      listcell3.getAttribute ("value"));
  },
    
  /**
   * フィルタを追加する
   *
   * @param  String id
   *         フィルタの対象
   *           "extract"/"load"
   * @param  String type
   *         フィルタの種類
   * @param  String pattern
   *         パターン
   * @param  String r
   *         正規表現フラグ
   *             "o"
   *             "x"
   * @param  XULElement listitem
   *         置き換えるノード
   *         追加する場合は null
   */
  addFilterItem : function (id, type, pattern, r, listitem) {
    var listbox = document.getElementById ("filter_" + id + "_list");
    var listcell;
    var append = false;
        
    if (listitem == null) {
      append = true;
      listitem = document.createElement ("listitem");
            
      listcell = document.createElement ("listcell");
      listitem.appendChild (listcell);
      listcell = document.createElement ("listcell");
      listitem.appendChild (listcell);
      listcell = document.createElement ("listcell");
      listitem.appendChild (listcell);
            
      listitem.addEventListener
        ("mousedown",
         (function (id) {
           return function () {
             UnMHTOptions.onMouseDownFilter (id, arguments [0]);
           };
         })(id), false);
    }
        
    var listcell = listitem.firstChild;
    listcell.setAttribute ("label",
                           document.getElementById
                           ("filter_" + id + "_type_" + type).value);
    listcell.setAttribute ("value", type);
        
    listcell = listcell.nextSibling;
    listcell.setAttribute ("label", pattern);
    listcell.setAttribute ("value", pattern);
        
    listcell = listcell.nextSibling;
    listcell.setAttribute ("value", r);
    listcell.setAttribute ("class", "listcell-iconic");
    if (r == "o") {
      listcell.setAttribute ("image",
                             "chrome://unmht/content/check_o.png");
    }
    else {
      listcell.setAttribute ("image",
                             "chrome://unmht/content/check_x.png");
    }
        
    if (append) {
      listbox.appendChild (listitem);
    }
  },
    
  /**
   * フィルタの項目を押したイベント
   *
   * @param  String id
   *         フィルタの対象
   *           "extract"/"load"
   * @param  Event event
   *         対象のイベント
   */
  onMouseDownFilter : function (id, event) {
    var listitem = event.target;
    if (listitem.nodeName.toLowerCase () != "listitem") {
      return;
    }
        
    var listcell1 = listitem.firstChild;
    var listcell2 = listcell1.nextSibling;
    var listcell3 = listcell2.nextSibling;
        
    var value = UnMHTOptions.getFilterItem (listitem);
        
    if (event.clientX
        < listcell1.boxObject.x + listcell1.boxObject.width) {
      return;
    }
    else if (event.clientX
             < listcell2.boxObject.x + listcell2.boxObject.width) {
      return;
    }
    else if (event.clientX
             < listcell3.boxObject.x + listcell3.boxObject.width) {
      if (event.clientX < listcell3.boxObject.x + 16) {
        value [2] = (value [2] == "o") ? "x" : "o";
      }
    }
        
    document.getElementById ("filter_" + id + "_type").value
    = value [0];
    document.getElementById ("filter_" + id + "_pattern").value
    = value [1];
    document.getElementById ("filter_" + id + "_regexp").checked
    = value [2] == "o";
        
    var listbox = document.getElementById ("filter_" + id + "_list");
    UnMHTOptions.addFilterItem (id,
                                value [0], value [1],
                                value [2], listitem);
  },
    
  /* 親のチェックボックスの状態で子の操作不可を設定する - ここから - */
    
  checkFilter : function (id) {
    document.getElementById ("filter_" + id + "_type").disabled
    = document.getElementById ("filter_" + id + "_type_label").disabled
    = document.getElementById ("filter_" + id + "_pattern_label").disabled
    = document.getElementById ("filter_" + id + "_pattern").disabled
    = document.getElementById ("filter_" + id + "_regexp").disabled
    = document.getElementById ("filter_" + id + "_list").disabled
    = document.getElementById ("filter_" + id + "_add").disabled
    = document.getElementById ("filter_" + id + "_moveup").disabled
    = document.getElementById ("filter_" + id + "_movedown").disabled
    = document.getElementById ("filter_" + id + "_delete").disabled
    = !document.getElementById ("filter_" + id).checked;
    
    if (id == "load") {
      document.getElementById ("filter_" + id + "_offline").disabled
        = !document.getElementById ("filter_" + id).checked;
      
      document.getElementById ("filter_" + id + "_offline_desc").disabled
        = !document.getElementById ("filter_" + id).checked
        || !document.getElementById ("filter_" + id + "_offline").checked;
    }
        
    document.getElementById ("filter_" + id + "_modify").disabled
    = !document.getElementById ("filter_" + id).checked
    || (document.getElementById ("filter_" + id + "_list").selectedCount > 1);
  },
    
  /* 親のチェックボックスの状態で子の操作不可を設定する - ここまで - */
    
  /**
   * 設定のインポート
   */
  importPrefs : function () {
    try {
      var filename;
            
      var filePicker
      = Components.classes ["@mozilla.org/filepicker;1"]
      .createInstance (Components.interfaces.nsIFilePicker);
      filePicker.init (window,
                       document.getElementById ("import_title").value,
                       Components.interfaces.nsIFilePicker.modeOpen);
      filePicker.appendFilter ("Text", "*.txt");
      filePicker.defaultString = "unmhtConfig.txt";
      filePicker.appendFilters (Components.interfaces.nsIFilePicker
                                .filterAll);
            
      var ret = filePicker.show ();
      if (ret == Components.interfaces.nsIFilePicker.returnOK) {
        var file = filePicker.file
          .QueryInterface (Components.interfaces.nsILocalFile);
                
        var fstream
          = Components
          .classes ["@mozilla.org/network/file-input-stream;1"]
          .createInstance (Components.interfaces.nsIFileInputStream);
        var sstream
          = Components
          .classes ["@mozilla.org/scriptableinputstream;1"]
          .createInstance (Components.interfaces
                           .nsIScriptableInputStream);
        fstream.init (file, 0x01, 0444, 0);
        sstream.init (fstream);
        var text = sstream.read (-1);
        sstream.close ();
        fstream.close ();
                
        text = text.replace (/\r\n/g, "\n");
        var map = new Object ();
        var ok = false;
                
        /* ファイルを解析するだけなので代入はしない */
        text.replace
          (/([^\n]+)\n?/g,
           function (matched, part1) {
            if (part1.match (/^(unmht\..+),(.+)$/)) {
              map [unescape (RegExp.$1)] = unescape (RegExp.$2);
              ok = true;
            }
            return "";
          });
                
        if (ok) {
          UnMHTOptions.loadPrefs (map);
                    
          document.getElementById ("import_export_message").value
            = document.getElementById ("import_done").value;
        }
        else {
          document.getElementById ("import_export_message").value
            = document.getElementById ("import_illegal").value;
        }
      }
      else {
        document.getElementById ("import_export_message").value
        = document.getElementById ("import_abort").value;
      }
    }
    catch (e) {
      document.getElementById ("import_export_message").value
      = document.getElementById ("import_fail").value;
    }
  },
    
  /**
   * 設定のエクスポート
   */
  exportPrefs : function () {
    try {
      var filename;
            
      var filePicker
      = Components.classes ["@mozilla.org/filepicker;1"]
      .createInstance (Components.interfaces.nsIFilePicker);
      filePicker.init (window,
                       document.getElementById ("export_title").value,
                       Components.interfaces.nsIFilePicker.modeSave);
      filePicker.appendFilter ("Text", "*.txt");
      filePicker.defaultString = "unmhtConfig.txt";
      filePicker.appendFilters (Components.interfaces.nsIFilePicker
                                .filterAll);
            
      var ret = filePicker.show ();
      if (ret == Components.interfaces.nsIFilePicker.returnOK ||
          ret == Components.interfaces.nsIFilePicker.returnReplace) {
        var file
          = filePicker.file
          .QueryInterface (Components.interfaces.nsILocalFile);
                
        var fstream
          = Components
          .classes ["@mozilla.org/network/file-output-stream;1"]
          .createInstance (Components.interfaces.nsIFileOutputStream);
        fstream.init (file, 0x02 | 0x08 | 0x20, 0664, 0);
                
        UnMHTOptions.savePrefs (fstream);
                
        fstream.close ();
                
        document.getElementById ("import_export_message").value
          = document.getElementById ("export_done").value;
      }
      else {
        document.getElementById ("import_export_message").value
        = document.getElementById ("export_abort").value;
      }
    }
    catch (e) {
      document.getElementById ("import_export_message").value
      = document.getElementById ("export_fail").value;
    }
  },
    
  /**
   * 設定の初期化
   */
  initPrefs : function () {
    var map = new Object ();
    UnMHTOptions.loadPrefs (map);
  },
    
  /**
   * フォーカスされている要素を調べる
   * テキストボックス内でエンターを押してもダイアログが閉じないようにする
   *
   * @return Boolean
   *         1行のテキストボックスにフォーカスが無ければ true
   */
  checkFocus : function () {
    var node = document.commandDispatcher.focusedElement;
        
    if  (node
         && node.nodeName == "html:input") {
      return false;
    }
        
    return true;
  },
    
  /**
   * サイトを開く
   */
  openWebsite : function () {
    var mediator
    = Components.classes ["@mozilla.org/appshell/window-mediator;1"]
    .getService (Components.interfaces.nsIWindowMediator);
    var chromeWindow = mediator.getMostRecentWindow ("navigator:browser");
    if (chromeWindow) {
      chromeWindow.UnMHTUIManager.openWebsite ();
    }
    else {
      var locale = "ja-JP";
      try {
        if (UnMHTOptions.prefBranch == null) {
          if (Components.interfaces.nsIPrefBranch2) {
            UnMHTOptions.prefBranch
              = Components.classes ["@mozilla.org/preferences-service;1"]
              .getService (Components.interfaces.nsIPrefBranch2);
          }
          else {
            UnMHTOptions.prefBranch
              = Components.classes ["@mozilla.org/preferences-service;1"]
              .getService (Components.interfaces.nsIPrefBranch);
          }
        }
        var name = "general.useragent.locale";
        if (UnMHTOptions.prefBranch.prefHasUserValue (name)) {
          locale = UnMHTOptions.prefBranch.getCharPref (name);
        }
      }
      catch (e) {
      }
      if (locale == "ja-JP") {
        window.open ("http://www.unmht.org/unmht/");
      }
      else {
        window.open ("http://www.unmht.org/unmht/en_index.html");
      }
    }
  },
    
  /**
   * MIME Type を修正する
   */
  checkMIME : function () {
    var exists = false;
    document.getElementById ("checkmime_message").value = "";
    try {
      var file
        = Components.classes ["@mozilla.org/file/directory_service;1"]
        .getService (Components.interfaces.nsIProperties)
        .get ("UMimTyp", Components.interfaces.nsIFile);
            
      if (UnMHTOptions.modifyMIME (file)) {
        exists = true;
      }
    }
    catch (e) {
    }
    try {
      var profdir
      = Components.classes ["@mozilla.org/file/directory_service;1"]
      .getService (Components.interfaces.nsIProperties)
      .get ("DefProfRt", Components.interfaces.nsIFile);
      var entries = profdir.directoryEntries;
      while (entries.hasMoreElements ()) {
        file = entries.getNext ();
        try {
          file = file.QueryInterface (Components.interfaces.nsIFile);
          if (file.isDirectory ()) {
            file.append ("mimeTypes.rdf");
            if (file.exists ()) {
              /* mimeTypes.rdf を発見した */
                        
              if (UnMHTOptions.modifyMIME (file)) {
                exists = true;
              }
            }
          }
        }
        catch (e) {
        }
      }
    }
    catch (e) {
    }
        
    if (exists) {
      document.getElementById ("checkmime_message").value
      = document.getElementById ("checkmime_ok").value;
    }
    else {
      document.getElementById ("checkmime_message").value
      = document.getElementById ("checkmime_no").value;
    }
  },
    
  /**
   * mimeTypes.rdf を修正する
   *
   * @param  nsIFile file
   *         mimeTypes.rdf
   */
  modifyMIME : function (file) {
    var rdf
    = Components.classes ["@mozilla.org/rdf/rdf-service;1"]
    .getService (Components.interfaces.nsIRDFService);
            
    var result = false;
        
    var spec
    = Components.classes ["@mozilla.org/network/io-service;1"]
    .getService (Components.interfaces.nsIIOService)
    .getProtocolHandler ("file")
    .QueryInterface (Components.interfaces.nsIFileProtocolHandler)
    .getURLSpecFromFile (file);
        
    var mimeDS = rdf.GetDataSource (spec);
        
    var source = rdf.GetUnicodeResource
    ("urn:mimetype:application/x-maf");
    var property = rdf.GetUnicodeResource
    ("http://home.netscape.com/NC-rdf#fileExtensions");
    var target = rdf.GetLiteral ("mht");
    if (mimeDS.HasAssertion (source, property, target, true)) {
      mimeDS.Unassert (source, property, target);
      result = true;
    }
        
    source = rdf.GetUnicodeResource
    ("urn:mimetype:application/maf");
    property = rdf.GetUnicodeResource
    ("http://home.netscape.com/NC-rdf#fileExtensions");
    target = rdf.GetLiteral ("mht");
    if (mimeDS.HasAssertion (source, property, target, true)) {
      mimeDS.Unassert (source, property, target);
      result = true;
    }
        
    mimeDS.QueryInterface (Components.interfaces.nsIRDFRemoteDataSource)
    .Flush ();
        
    var compregFile = file.parent;
    compregFile.append ("compreg.dat");
    if (compregFile.exists ()) {
      compregFile.remove (true);
    }
        
    return result;
  },
    
  /**
   * 保存のキー変更イベント
   *
   * @param  Event event
   *         対象のイベント
   */
  onSaveShortcutKeyDown : function (event) {
    var keycode = "";
    for (var attribute in Components.interfaces.nsIDOMKeyEvent) {
      if (attribute.match (/^DOM_(VK_.+)$/)) {
        var code = RegExp.$1;
        if (event.keyCode
            == Components.interfaces.nsIDOMKeyEvent [attribute]) {
          keycode = code;
        }
      }
    }
    if (keycode == "VK_SHIFT"
        || keycode == "VK_CONTROL"
        || keycode == "VK_ALT"
        || keycode == "VK_META") {
      return;
    }
        
    if (keycode != "") {
      document.getElementById ("save_shortcut_keycode").value
      = keycode;
      UnMHTOptions.selectItem (document.getElementById
                               ("save_shortcut_keycode"));
            
      document.getElementById ("save_shortcut_modifiers_alt").checked
      = event.altKey;
      document.getElementById ("save_shortcut_modifiers_ctrl").checked
      = event.ctrlKey;
      document.getElementById ("save_shortcut_modifiers_meta").checked
      = event.metaKey;
      document.getElementById ("save_shortcut_modifiers_shift").checked
      = event.shiftKey;
    }
        
    event.preventDefault ();
    event.stopPropagation ();
  },
    
  checkSaveShortcut : function () {
    document.getElementById ("save_shortcut_input").disabled
    = document.getElementById ("save_shortcut_keycode").disabled
    = document.getElementById ("save_shortcut_keycode_menu").disabled
    = document.getElementById ("save_shortcut_modifiers_alt").disabled
    = document.getElementById ("save_shortcut_modifiers_ctrl").disabled
    = document.getElementById ("save_shortcut_modifiers_meta").disabled
    = document.getElementById ("save_shortcut_modifiers_shift").disabled
    = !document.getElementById ("save_shortcut").checked;
  },
  
  /**
   * 保存のキー変更イベント
   *
   * @param  Event event
   *         対象のイベント
   */
  onSaveShortcut2KeyDown : function (event) {
    var keycode = "";
    for (var attribute in Components.interfaces.nsIDOMKeyEvent) {
      if (attribute.match (/^DOM_(VK_.+)$/)) {
        var code = RegExp.$1;
        if (event.keyCode
            == Components.interfaces.nsIDOMKeyEvent [attribute]) {
          keycode = code;
        }
      }
    }
    if (keycode == "VK_SHIFT"
        || keycode == "VK_CONTROL"
        || keycode == "VK_ALT"
        || keycode == "VK_META") {
      return;
    }
        
    if (keycode != "") {
      document.getElementById ("save_shortcut2_keycode").value
      = keycode;
      UnMHTOptions.selectItem (document.getElementById
                               ("save_shortcut2_keycode"));
            
      document.getElementById ("save_shortcut2_modifiers_alt").checked
      = event.altKey;
      document.getElementById ("save_shortcut2_modifiers_ctrl").checked
      = event.ctrlKey;
      document.getElementById ("save_shortcut2_modifiers_meta").checked
      = event.metaKey;
      document.getElementById ("save_shortcut2_modifiers_shift").checked
      = event.shiftKey;
    }
        
    event.preventDefault ();
    event.stopPropagation ();
  },
    
  checkSaveShortcut2 : function () {
    document.getElementById ("save_shortcut2_input").disabled
    = document.getElementById ("save_shortcut2_keycode").disabled
    = document.getElementById ("save_shortcut2_keycode_menu").disabled
    = document.getElementById ("save_shortcut2_modifiers_alt").disabled
    = document.getElementById ("save_shortcut2_modifiers_ctrl").disabled
    = document.getElementById ("save_shortcut2_modifiers_meta").disabled
    = document.getElementById ("save_shortcut2_modifiers_shift").disabled
    = !document.getElementById ("save_shortcut2").checked;
  },
    
  /**
   * 保存のキー変更イベント
   *
   * @param  Event event
   *         対象のイベント
   */
  onQuicksaveShortcutKeyDown : function (event) {
    var keycode = "";
    for (var attribute in Components.interfaces.nsIDOMKeyEvent) {
      if (attribute.match (/^DOM_(VK_.+)$/)) {
        var code = RegExp.$1;
        if (event.keyCode
            == Components.interfaces.nsIDOMKeyEvent [attribute]) {
          keycode = code;
        }
      }
    }
    if (keycode == "VK_SHIFT"
        || keycode == "VK_CONTROL"
        || keycode == "VK_ALT"
        || keycode == "VK_META") {
      return;
    }
        
    if (keycode != "") {
      document.getElementById ("quicksave_shortcut_keycode").value
      = keycode;
      UnMHTOptions.selectItem (document.getElementById
                               ("quicksave_shortcut_keycode"));
            
      document.getElementById ("quicksave_shortcut_modifiers_alt").checked
      = event.altKey;
      document.getElementById ("quicksave_shortcut_modifiers_ctrl").checked
      = event.ctrlKey;
      document.getElementById ("quicksave_shortcut_modifiers_meta").checked
      = event.metaKey;
      document.getElementById ("quicksave_shortcut_modifiers_shift").checked
      = event.shiftKey;
    }
        
    event.preventDefault ();
    event.stopPropagation ();
  },
  
  checkQuicksaveShortcut : function () {
    document.getElementById ("quicksave_shortcut_input").disabled
    = document.getElementById ("quicksave_shortcut_keycode").disabled
    = document.getElementById ("quicksave_shortcut_keycode_menu").disabled
    = document.getElementById ("quicksave_shortcut_modifiers_alt").disabled
    = document.getElementById ("quicksave_shortcut_modifiers_ctrl").disabled
    = document.getElementById ("quicksave_shortcut_modifiers_meta").disabled
    = document.getElementById ("quicksave_shortcut_modifiers_shift").disabled
    = !document.getElementById ("quicksave_shortcut").checked;
  },
    
  checkSkipAlertOnDeep : function () {
    document.getElementById ("save_skip_alert_on_deep_default").disabled
    = !document.getElementById ("save_skip_alert_on_deep").checked;
  },
  
  checkSaveLocalToSame : function () {
    document.getElementById ("save_local_to_same_append").disabled
    = !document.getElementById ("save_local_to_same").checked;
    UnMHTOptions.checkSaveLocalToSameAppend ();
  },
  
  checkSaveLocalToSameAppend : function () {
    document.getElementById ("save_local_to_same_append_text").disabled
    = !document.getElementById ("save_local_to_same").checked
    || !document.getElementById ("save_local_to_same_append").checked;
  },
  
  checkAutoReplace : function () {
    document.getElementById ("save_multi_autoreplace_action").disabled
    = !document.getElementById ("save_multi_autoreplace").checked;
  },
    
  checkInsertLink : function () {
    document.getElementById ("save_insertlink_pos").disabled
    = document.getElementById ("save_insertlink_date").disabled
    = document.getElementById ("save_insertlink_version").disabled
    = document.getElementById ("save_insertlink_nolocal").disabled
    = !document.getElementById ("save_insertlink").checked;
  },
    
  insertCustom : function (text) {
    var textbox = document.getElementById ("save_filename_custom");
    var value = textbox.value;
    var pos = textbox.selectionStart
        
    value = value.substr (0, pos) + text + value.substr (pos);
        
    textbox.value = value;
        
    textbox.focus ();
        
    textbox.selectionStart = textbox.selectionEnd = pos + text.length;
        
    UnMHTOptions.changeFilename ();
  },
  
  changeSendmailArgType : function () {
    if (document.getElementById ("sendmail_argtype").value == "thunderbird") {
      document.getElementById ("sendmail_arg").value = UnMHTMail.thunderbird;
      document.getElementById ("sendmail_arg").disabled = true;
    }
    else if (document.getElementById ("sendmail_argtype").value == "outlook") {
      document.getElementById ("sendmail_arg").value = UnMHTMail.outlook;
      document.getElementById ("sendmail_arg").disabled = true;
    }
    else if (document.getElementById ("sendmail_argtype").value == "winmail") {
      document.getElementById ("sendmail_arg").value = UnMHTMail.winmail;
      document.getElementById ("sendmail_arg").disabled = true;
    }
    else {
      document.getElementById ("sendmail_arg").disabled = false;
    }
  },
  
  changeFilename : function () {
    var TITLE = "no title";
    var FILENAME = "index.html";
    var URL = "http://localhost/index.html";
    var pattern
    = document.getElementById ("save_filename_custom").value;
    var am
    = document.getElementById ("save_filename_custom_am").value;
    var pm
    = document.getElementById ("save_filename_custom_pm").value;
    document.getElementById ("save_filename_custom_sample").value
    = UnMHTFormatter.format (pattern, TITLE, FILENAME, URL, am, pm);
  },
    
  changeSaveFilename : function () {
  },
    
  /**
   * HTML ファイルを MHT に変換
   *
   * @param  nsILocalFile mhtFile
   *         保存先のファイル
   * @param  Array targetFiles
   *         対象のファイル [nsILocalFile, ...]
   *           先頭のファイルは対象の HTML ファイル
   * @param  String subject
   *         Subject の値
   * @param  String dirname
   *         ディレクトリ名
   */
  convertFile : function (mhtFile, targetFiles, subject, dirname) {
    var fstream
    = Components
    .classes ["@mozilla.org/network/file-output-stream;1"]
    .createInstance (Components.interfaces.nsIFileOutputStream);
    try {
      fstream.init (mhtFile, 0x02 | 0x08 | 0x20, 0664, 0);
    }
    catch (e) {
      return false;
    }
        
    converter
    = Components.classes ["@mozilla.org/intl/scriptableunicodeconverter"]
    .getService (Components.interfaces.nsIScriptableUnicodeConverter);
        
    converter.charset = "iso-2022-jp";
    subject = converter.ConvertFromUnicode (subject);
        
    /* ヘッダに必要な情報を設定する */
    var subject
    = "=?iso-2022-jp?B?"
    + btoa (subject)
    + "?=";
    var boundary = "----=_NextPart_000_0000_";
    var hex = [
      "0", "1", "2", "3", "4", "5", "6", "7",
      "8", "9", "A", "B", "C", "D", "E", "F"
      ];
    for (var i = 0; i < 8; i ++) {
      boundary += hex [parseInt (Math.random () * 15)];
    }
    boundary += ".";
    for (var i = 0; i < 8; i ++) {
      boundary += hex [parseInt (Math.random () * 15)];
    }
    var date
    = (new Date ()).toString ()
    .replace (/^([A-Za-z]{3}) /, "$1, ")
    .replace (/ \(.*\)$/, "");
            
    var data = "";
            
    /* 全体のヘッダ */
    data
    = "From: <Saved by UnMHT>\r\n"
    + "Subject: " + subject + "\r\n"
    + "Date: " + date + "\r\n"
    + "MIME-Version: 1.0\r\n"
    + "Content-Type: multipart/related;\r\n"
    + "\tboundary=\"" + boundary + "\";\r\n"
    + "\ttype=\"text/html\"\r\n"
    + "\r\n";
            
    fstream.write (data, data.length);
            
    for (var i = 0; i < targetFiles.length; i ++) {
      var file = targetFiles [i];
                
      data = "--" + boundary + "\r\n";
      fstream.write (data, data.length);
                
      var filetype = "";
                
      if (file.leafName.match (/\.[sx]?html?(\?.+)?(#.+)?$/i)) {
        filetype = "text/html";
      }
      else if (file.leafName.match (/\.css(\?.+)?$/i)) {
        filetype = "text/css";
      }
      else if (file.leafName.match (/\.mht(ml)?(\?.+)?$/i)) {
        filetype = "message/rfc822";
      }
      else if (file.leafName.match (/\.eml(\?.+)?$/i)) {
        filetype = "message/rfc822";
      }
      else if (file.leafName.match (/\.js(\?.+)?$/i)) {
        filetype = "text/javascript";
      }
      else if (file.leafName.match (/\.txt(\?.+)?$/i)) {
        filetype = "text/plain";
      }
      else if (file.leafName.match (/\.jpe?g(\?.+)?$/i)) {
        filetype = "image/jpeg";
      }
      else if (file.leafName.match (/\.gif(\?.+)?$/i)) {
        filetype = "image/gif";
      }
      else if (file.leafName.match (/\.png(\?.+)?$/i)) {
        filetype = "image/png";
      }
      else if (file.leafName.match (/\.bmp(\?.+)?$/i)) {
        filetype = "image/bmp";
      }
      else if (file.leafName.match (/\.ico(\?.+)?$/i)) {
        filetype = "image/x-icon";
      }
      else if (file.leafName.match (/\.svg(\?.+)?$/i)) {
        filetype = "image/svg+xml";
      }
      else if (file.leafName.match (/\.swf(\?.+)?$/i)) {
        filetype = "application/swf";
      }
      else if (file.leafName.match (/\.jar(\?.+)?$/i)) {
        filetype = "application/java-archive";
      }
      else if (file.leafName.match (/\.class(\?.+)?$/i)) {
        filetype = "application/octet-stream";
      }
      else if (file.leafName.match (/\.([A-Za-z0-9])(\?.*)?$/i)) {
        var ext = RegExp.$1;
        var mime
          = Components.classes ["@mozilla.org/mime;1"]
          .getService
          (Components.interfaces.nsIMIMEService);
                
        try {
          filetype
            = mime.getTypeFromExtension (ext);
        }
        catch (e) {
          filetype = "application/octet-stream";
        }
      }
      else {
        filetype = "application/octet-stream";
      }
      
      var location;
      if (i == 0) {
        location = escape (file.leafName);
      }
      else {
        location = escape (dirname + "/" + file.leafName);
      }
            
      data
        = "Content-Type: " + filetype + "\r\n"
        + "Content-Transfer-Encoding: base64\r\n"
        + "Content-Location: " + location + "\r\n"
        + "\r\n";
            
      fstream.write (data, data.length);
            
      var fstream2
        = Components
        .classes ["@mozilla.org/network/file-input-stream;1"]
        .createInstance (Components.interfaces.nsIFileInputStream);
      var bstream
        = Components.classes ["@mozilla.org/binaryinputstream;1"]
        .createInstance (Components.interfaces.nsIBinaryInputStream);
      fstream2.init (file, 0x01, 0444, 0);
      bstream.setInputStream (fstream2);
      var content = btoa (bstream.readBytes (file.fileSize));
      bstream.close ();
      fstream2.close ();
                
      var width = 76;
      data = "";
      for (var j = 0; j < content.length; j += width) {
        data
          += content.substr (j, width) + "\r\n";
      }
            
      fstream.write (data,
                     data.length);
                
      data = "\r\n";
      fstream.write (data, data.length);
    }
        
    data = "--" + boundary + "--\r\n";
    fstream.write (data, data.length);
        
    fstream.close ();
    try {
      mhtFile.lastModifiedTime = targetFiles [0].lastModifiedTime;
    }
    catch (e) {
    }
        
    return true;
  },
    
  /**
   * HTML ファイルを MHT に変換
   */
  convert : function () {
    var filePicker
    = Components.classes ["@mozilla.org/filepicker;1"]
    .createInstance (Components.interfaces.nsIFilePicker);
        
    filePicker.init
    (window,
     UnMHTOptions.messages.GetStringFromName ("convert_title"),
     Components.interfaces.nsIFilePicker.modeOpenMultiple);
        
    var lastDir = UnMHTOptions.getLastOpenDir ();
    if (lastDir) {
      var dir
        = Components.classes ["@mozilla.org/file/local;1"]
        .createInstance (Components.interfaces.nsILocalFile);
      dir.initWithPath (lastDir);
      filePicker.displayDirectory = dir;
    }
        
    var ret = filePicker.show ();
    if (ret == Components.interfaces.nsIFilePicker.returnOK) {
      var files = [];
            
      UnMHTOptions.setLastOpenDir (filePicker.file.parent.path);
      
      var fs = filePicker.files;
      var count = 0;
            
      while (fs.hasMoreElements ()) {
        var f = fs.getNext ();
        var file;
                
        try {
          file
            = f.QueryInterface (Components.interfaces.nsILocalFile);
        }
        catch (e) {
          continue;
        }
                
        if (!file.isFile ()) {
          continue;
        }
                
        files.push (file);
      }
            
      setTimeout
        (function (files, count) {
          UnMHTOptions.convertTimer (files, count);
        }, 100, files, 0);
    }
  },
  
  getLastOpenDir : function () {
    var lastDir;
    
    if (document.getElementById ("use_global_lastdir").checked) {
      if (UnMHTOptions.prefBranch.prefHasUserValue ("browser.open.lastDir")) {
        lastDir
        = UnMHTOptions.prefBranch.getComplexValue
        ("browser.open.lastDir", Components.interfaces.nsILocalFile);
        if (lastDir) {
          lastDir = lastDir.path;
        }
        else {
          lastDir = "";
        }
      }
    }
    else {
      lastDir
      = UnMHTOptions
      .initPref (null, "char", "unmht.save.lastdir", "");
      lastDir = unescape (lastDir);
    }
    
    return lastDir;
  },
  
  setLastOpenDir : function (lastDir) {
    if (document.getElementById ("use_global_lastdir").checked) {
      var dir
        = Components.classes ["@mozilla.org/file/local;1"]
        .createInstance (Components.interfaces.nsILocalFile);
      dir.initWithPath (lastDir);
      
      UnMHTOptions.prefBranch.setComplexValue
      ("browser.open.lastDir", Components.interfaces.nsILocalFile, dir);
    }
    else {
      UnMHTOptions.prefBranch.setCharPref
        ("unmht.save.lastdir", escape (lastDir));
    }
  },
  
  /**
   * ディレクトリ内の HTML ファイルを MHT に変換
   */
  convertDir : function () {
    var filePicker
    = Components.classes ["@mozilla.org/filepicker;1"]
    .createInstance (Components.interfaces.nsIFilePicker);
    
    filePicker.init
    (window,
     UnMHTOptions.messages.GetStringFromName ("convert_title"),
     Components.interfaces.nsIFilePicker.modeGetFolder);
    
    var lastDir = UnMHTOptions.getLastOpenDir ();
    if (lastDir) {
      var dir
        = Components.classes ["@mozilla.org/file/local;1"]
        .createInstance (Components.interfaces.nsILocalFile);
      dir.initWithPath (lastDir);
      filePicker.displayDirectory = dir;
    }
        
    var ret = filePicker.show ();
    if (ret == Components.interfaces.nsIFilePicker.returnOK) {
      var files = [];
      
      UnMHTOptions.setLastOpenDir (filePicker.file.path);
      
      var fs = filePicker.file.directoryEntries;
      var count = 0;

      while (fs.hasMoreElements ()) {
        var f = fs.getNext ();
        var file;

        try {
          file
            = f.QueryInterface (Components.interfaces.nsILocalFile);
        }
        catch (e) {
          continue;
        }

        if (!file.isFile ()) {
          continue;
        }

        files.push (file);
      }

      setTimeout
        (function (files, count) {
          UnMHTOptions.convertTimer (files, count);
        }, 100, files, 0);
    }
  },
  
  convertTimer : function (files, count) {
    if (files.length > 0) {
      var file = files.shift ();
            
      var pre
      = UnMHTOptions.messages.GetStringFromName ("convert_progress_pre");
      var post
      = UnMHTOptions.messages.GetStringFromName ("convert_progress_post");
            
      var message = "";
      if (pre) {
        message += pre + " ";
      }
      message += file.leafName;
      if (pre) {
        message += " " + post;
      }
            
      document.getElementById ("convert_progress").value = message;
            
      var dir = null;
                
      try {
        var d
          = Components.classes ["@mozilla.org/file/local;1"]
          .createInstance (Components.interfaces.nsILocalFile);
        d.initWithPath (file.path.replace (/\.[^\.]+$/, "_files"));
        if (d.exists ()
            && d.isDirectory ()) {
          dir = d;
        }
      }
      catch (e) {
      }
                
      if (dir == null) {
        try {
          var d
          = Components.classes ["@mozilla.org/file/local;1"]
          .createInstance
          (Components.interfaces.nsILocalFile);
          d.initWithPath
          (file.path.replace (/\.[^\.]+$/, ".files"));
          if (d.exists ()
              && d.isDirectory ()) {
            dir = d;
          }
        }
        catch (e) {
        }
      }
                
      var dirname = "";
      var targetFiles = [];
      if (dir) {
        dirname = dir.leafName;
        var entries = dir.directoryEntries;
        while (entries.hasMoreElements ()) {
          var f = entries.getNext ();
          try {
            var file2
              = f.QueryInterface
              (Components.interfaces.nsILocalFile);
          }
          catch (e) {
            continue;
          }
                        
          if (!file2.isFile ()) {
            continue;
          }
                        
          targetFiles.push (file2);
        }
        
        targetFiles.unshift (file);
                
        var mhtFile
          = Components.classes ["@mozilla.org/file/local;1"]
          .createInstance
          (Components.interfaces.nsILocalFile);
        mhtFile.initWithPath
          (file.path.replace (/\.[^\.]+$/, "") + ".mht");
            
        var n = 2;
        while (mhtFile.exists ()) {
          mhtFile
            = Components.classes ["@mozilla.org/file/local;1"]
            .createInstance
            (Components.interfaces.nsILocalFile);
          mhtFile.initWithPath
            (file.path.replace (/\.[^\.]+$/, "") + "-" + n + ".mht");
          n ++;
        }
            
        if (UnMHTOptions.convertFile
            (mhtFile, targetFiles, file.leafName, dirname)) {
          count ++;
        }
      }
      
      setTimeout
      (function (files, count) {
        UnMHTOptions.convertTimer (files, count);
      }, 100, files, count);
    }
    else {
      var pre
      = UnMHTOptions.messages.GetStringFromName ("convert_result_pre");
      var post
      = UnMHTOptions.messages.GetStringFromName ("convert_result_post");
            
      var message = "";
      if (pre) {
        message += pre + " ";
      }
      message += count;
      if (post) {
        message += " " + post;
      }
            
      document.getElementById ("convert_progress").value = message;
    }
  }
};
