/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/**
 * ブラウザのイベントリスナ
 * ロケーションの変更を監視する
 *   Inherits From: nsIWebProgressListener
 */
var UnMHTBrowserProgressListener = {
  /**
   * インターフェースの要求
   *   nsISupports.QueryInterface
   *
   * @param  nsIIDRef iid
   *         インターフェース ID
   * @throws Components.results.NS_NOINTERFACE
   * @return nsIWebProgressListener
   *         this
   */
  QueryInterface : function (iid) {
    if (iid.equals (Components.interfaces.nsISupports)
        || iid.equals (Components.interfaces.nsISupportsWeakReference)
        || iid.equals (Components.interfaces.nsIWebProgressListener)) {
      return this;
    }
        
    throw Components.results.NS_NOINTERFACE;
  },
    
  /**
   * 監視ウィンドウのロケーションが変わった時のイベント
   *   nsIWebProgressListener.onLocationChange
   *
   * @param  nsIWebProgress webProgress
   *         呼び出し元
   * @param  nsIRequest request
   *         状況の変わったリクエスト
   * @param  String location
   *         ロケーション
   */
  onLocationChange : function (webProgress, request, location) {
    var panel;
        
    var targetDocument = null;
    var savable = true;
        
    if (!UnMHT.isThunderbird) {
      var tabbrowser = document.getElementById ("content");
      if (!tabbrowser) {
        tabbrowser = document.getElementById ("browser_content");
      }
      if (!tabbrowser) {
        return;
      }
      
      targetDocument = tabbrowser.contentDocument;
    }
    else {
      var messagepane = document.getElementById ("messagepane");
      targetDocument = messagepane.contentDocument;
            
      var messagepanebox = document.getElementById ("messagepanebox");
      if (messagepanebox.collapsed) {
        savable = false;
      }
    }
    
    if (savable && UnMHT.isSavable (targetDocument)) {
      panel
      = document.getElementById ("unmht-toolbarbutton-save");
      if (panel) {
        panel.setAttribute ("status", "enabled");
      }
      if (UnMHT.isThunderbird) {
        var command = document.getElementById ("unmht-command-save");
        if (command) {
          command.setAttribute ("disabled", "false");
        }
      }
    }
    else {
      panel
      = document.getElementById ("unmht-toolbarbutton-save");
      if (panel) {
        panel.setAttribute ("status", "disabled");
      }
      if (UnMHT.isThunderbird) {
        var command = document.getElementById ("unmht-command-save");
        if (command) {
          command.setAttribute ("disabled", "true");
        }
      }
    }
  },
    
  /**
   * 進行状況が変わった時のイベント
   *   nsIWebProgressListener.onProgressChange
   * 未使用
   */
  onProgressChange: function (webProgress , request,
                              curSelfProgress, maxSelfProgress,
                              curTotalProgress, maxTotalProgress) {
  },
    
  /**
   * プロトコルのセキュリティ設定が変わった時のイベント
   *   nsIWebProgressListener.onSecurityChange
   * 未使用
   */
  onSecurityChange : function (webProgress, request, state) {
  },
    
  /**
   * 状況が変わった時のイベント
   *   nsIWebProgressListener.onStateChange
   * 未使用
   */
  onStateChange : function (webProgress, request, stateFlags, status) {
  },
    
  /**
   * ステータスバーに表示するメッセージが変わった時のイベント
   *   nsIWebProgressListener.onStatusChange
   * 未使用
   */
  onStatusChange : function (webProgress, request, status, message) {
  }
};

/**
 * ヒストリーリスナ
 * リロードイベントを監視する
 *   Inherits From: nsISHistoryListener
 */
function UnMHTSHistoryListener () {
}
UnMHTSHistoryListener.prototype = {
  targetWindow : null, /* Window  対象のウィンドウ */
    
  /**
   * データを開放する
   */
  destruct : function () {
    try {
      this.targetWindow
      .QueryInterface (Components.interfaces.nsIInterfaceRequestor)
      .getInterface (Components.interfaces.nsIWebNavigation)
      .sessionHistory.removeSHistoryListener (this);
    }
    catch (e) {
    }
    this.targetWindow = null;
  },
    
  /**
   * インターフェースの要求
   *   nsISupports.QueryInterface
   *
   * @param  nsIIDRef iid
   *         インターフェース ID
   * @throws Components.results.NS_NOINTERFACE
   * @return nsISHistoryListener
   *         this
   */
  QueryInterface : function (iid) {
    if (iid.equals (Components.interfaces.nsISupports)
        || iid.equals (Components.interfaces.nsISupportsWeakReference)
        || iid.equals (Components.interfaces.nsISHistoryListener)) {
      return this;
    }
        
    throw Components.results.NS_NOINTERFACE;
  },
    
  /**
   * 戻るイベント
   *   nsISHistoryListener.OnHistoryGoBack
   *
   * @param  nsIURI backURI
   *         戻る先の URI
   * @return Boolean
   *         続けるかどうか
   */
  OnHistoryGoBack : function (backURI) {
    return true;
  },
    
  /**
   * 進むイベント
   *   nsISHistoryListener.OnHistoryGoForward
   *
   * @param  nsIURI forwardURI
   *         進む先の URI
   * @return Boolean
   *         続けるかどうか
   */
  OnHistoryGoForward : function (forwardURI) {
    return true;
  },
    
  /**
   * 移動イベント
   *   nsISHistoryListener.OnHistoryGotoIndex
   *
   * @param  Number index
   *         移動先のインデックス
   * @param  nsIURI gotoURI
   *         移動先の URI
   * @return Boolean
   *         続けるかどうか
   */
  OnHistoryGotoIndex : function (index, gotoURI) {
    return true;
  },
    
  /**
   * 項目追加イベント
   *   nsISHistoryListener.OnHistoryNewEntry
   *
   * @param  nsIURI newURI
   *         追加する URI
   */
  OnHistoryNewEntry : function (newURI) {
    return true;
  },
    
  /**
   * 項目削除イベント
   *   nsISHistoryListener.OnHistoryPurge
   *
   * @param  Number index
   *         削除する数
   * @return Boolean
   *         続けるかどうか
   */
  OnHistoryPurge : function (numEntries) {
    return true;
  },
    
  /**
   * リロードイベント
   *   nsISHistoryListener.OnHistoryReload
   * ファイルを展開し直す
   *
   * @param  nsIURI reloadURI
   *         リロードする URI
   * @param  Number reloadFlags
   *         リロード方法
   * @return Boolean
   *         続けるかどうか
   */
  OnHistoryReload : function (reloadURI, reloadFlags) {
    var nav
    = this.targetWindow
    .QueryInterface (Components.interfaces.nsIInterfaceRequestor)
    .getInterface (Components.interfaces.nsIWebNavigation);
        
    if (UnMHT.protocolHandler.isUnMHTURI (reloadURI.spec)) {
      var uri
        = Components.classes ["@mozilla.org/network/standard-url;1"]
        .createInstance (Components.interfaces.nsIURI);
      var param = UnMHT.protocolHandler.getUnMHTURIParam (reloadURI.spec);
      uri.spec = param.original;
            
      UnMHT.loadMHT (uri, this.targetWindow,
                     false,         /* ドキュメント無し */
                     true,          /* 履歴は置き換える */
                     0,             /* 展開は設定に任せる */
                     param.refName, /* 現在開いているファイルを開く */
                     false);        /* キャッシュは使用しない */
      return false;
    }
        
    return true;
  }
};

/**
 * ファイルダウンロードの進行状況のリスナ
 * ウェブ上のファイルを展開する時に使用する
 *   Inherits From: nsIWebProgressListener
 */
function UnMHTProgressListener () {
}
UnMHTProgressListener.prototype = {
  uri : null,          /* nsIURI  開く URI */
  targetWindow : null, /* Window  表示するウィンドウ */
    
  hasDocument : false, /* Boolean  ドキュメントに内容を持っているか */
  replace : false,     /* Boolean  履歴を置き換えるかどうか */
  extract : false,     /* Boolean  展開するかどうか */
  refName : "",        /* String  対象のファイル */
    
  filename : "",       /* String  保存するファイル名 */
  extractName : "",    /* String  展開先のディレクトリの名前 */
    
  /**
   * インターフェースの要求
   *   nsISupports.QueryInterface
   *
   * @param  nsIIDRef iid
   *         インターフェース ID
   * @throws Components.results.NS_NOINTERFACE
   * @return nsIWebProgressListener
   *         this
   */
  QueryInterface : function (iid) {
    if (iid.equals (Components.interfaces.nsISupports)
        || iid.equals (Components.interfaces.nsIWebProgressListener)) {
      return this;
    }
        
    throw Components.results.NS_NOINTERFACE;
  },
    
  /**
   * 監視ウィンドウのロケーションが変わった時のイベント
   *   nsIWebProgressListener.onLocationChange
   * 未使用
   */
  onLocationChange : function (webProgress, request, location) {
  },
    
  /**
   * 進行状況が変わった時のイベント
   *   nsIWebProgressListener.onProgressChange
   * 未使用
   */
  onProgressChange : function (webProgress , request,
                               curSelfProgress, maxSelfProgress,
                               curTotalProgress, maxTotalProgress) {
  },
    
  /**
   * プロトコルのセキュリティ設定が変わった時のイベント
   *   nsIWebProgressListener.onSecurityChange
   * 未使用
   */
  onSecurityChange : function (webProgress, request, state) {
  },
    
  /**
   * 状況が変わった時のイベント
   *   nsIWebProgressListener.onStateChange
   * 終了したらファイルを展開する
   *
   * @param  nsIWebProgress webProgress
   *         呼び出し元
   * @param  nsIRequest request
   *         状況の変わったリクエスト
   * @param  Number stateFlags
   *         変わった状況のフラグ
   * @param  nsresult status
   *         エラーコード
   */
  onStateChange : function (webProgress, request, stateFlags, status) {
    var httpStatus = 200;
    try {
      /* HTTP の場合ステータスをチェックする */
      httpStatus
        = request.QueryInterface (Components.interfaces.nsIHttpChannel)
        .responseStatus;
    }
    catch (e) {
      /* HTTP でない場合はデフォルトの 200 のまま */
    }
        
    if (stateFlags
        & Components.interfaces.nsIWebProgressListener.STATE_STOP) {
      if (httpStatus == 200) {
        /* 転送が終了したら
         * (HTTP の場合は 200 ならば) */
        UnMHT.loadMHTFromFile (this.uri, this.targetWindow,
                               this.hasDocument,
                               this.replace,
                               this.extract,
                               this.refName,
                               this.filename, this.extractName);
        var targetFile = UnMHTFileManager.initFile (this.filename);
        if (targetFile.exists ()) {
          targetFile.remove (true);
        }
      }
      else {
        /* ファイルが見付からなかった場合 */
        if (this.hasDocument) {
          UnMHT.loadMHTFromDocument (this.uri, this.targetWindow,
                                     true,
                                     this.replace,
                                     this.extract,
                                     this.refName);
        }
        else {
          UnMHT.showErrorPage (this.targetWindow, this.uri.spec);
        }
      }
    }
  },
    
  /**
   * ステータスバーに表示するメッセージが変わった時のイベント
   *   nsIWebProgressListener.onStatusChange
   * 未使用
   */
  onStatusChange : function (webProgress, request, status, message) {
  }
};

/**
 * ドキュメントごとの情報
 */
function UnMHTDocumentParam () {
}
UnMHTDocumentParam.prototype = {
  targetDocument : null,
  history_listener : null
};

/**
 * 本体
 */
var UnMHT = {
  protocolHandler : null,        /* arUnMHTMhtmlProtocolHandler
                                  *   unmht プロトコルハンドラ */
  
  messages : null,               /* nsIStringBundle  メッセージ */
  documentParams : new Array (), /* Array  ドキュメントごとの情報 */
  latestParam : null,            /* UnMHTDocumentParam
                                  *   最近使ったドキュメントごとの情報 */
    
  isOld : false,                 /* Boolean  古い Mozilla Suite か */
  isThunderbird : false,         /* Boolean  Thunderbird か */
    
  initialized : false,           /* Boolean  初期化フラグ */
    
  enableAll : false,             /* Boolean  全機能の ON／OFF */
    
  extractors : new Object (),    /* Obejct  展開の情報を保持しておく
                                  * <String 展開したファイル名の正規化名,
                                  *  UnMHTExtractor 展開の情報> */
    
  config : null,                 /* UnMHTConfigManager 設定管理 */
  conv : null,                   /* UnMHTConverter 文字列変換器 */
    
  mode : 1,                      /* Number  動作モード
                                  *   1: Firefox/Mozilla Suite
                                  *   2: Opera/Safari */
  
  loadImageType : 1,            /* Number  Firefox の画像読み込み設定 */
  
  memoryCacheMode : true,       /* Boolean  メモリキャッシュモード */
  
  /* [一般] タブの設定 */
  enableNolimit : false,        /* Boolean  展開中は n 秒経過するまで
                                 *   警告を表示しない*/
  nolimitTime : 0,              /* Number  秒数 */
  enableFastExtract : false,    /* Boolean  展開を高速化する */
    
  cacheExpire : 300000,         /* Number  キャッシュの保持時間 [ミリ秒] */
  enableCacheExpire : false,    /* Boolean  キャッシュ自動削除 */
    
  enableSkipPPTWarning : false, /* Boolean  PowerPoint の警告を飛ばす */
  openMultiple : "",            /* String  複数のファイルを含む MHT を開く時
                                 *   tabs: 全てのページをタブとして開く
                                 *   index: インデックスページを表示する */
  
  enableLoadImageType : false,  /* Boolean  Firefox の画像読み込み設定を
                                 *   使用するか */
  enableUseGlobalLastDir : false, /* Boolean  最後に使用したディレクトリを
                                   *   ブラウザ全体共通に記憶する */
  
  /* [フィルタ] タブの設定 */
    
  enableFilterExtract : false,      /* Boolean  展開フィルタ */
  filterExtractList : new Array (), /* Array  展開フィルタリスト
                                     * [[String 種類,
                                     *   String/RegExp パターン,
                                     *   Boolean 正規表現のフラグ], ...]
                                     * String 種類
                                     *   "hostname":  ホスト名
                                     *   "leafname":  ファイル名
                                     *   "extension": 拡張子
                                     *   "location":  アドレス全部
                                     *   "mimetype":  MIME タイプ */
  
  enableFilterLoad : false,        /* Boolean  読み込みフィルタ */
  enableFilterLoadOffline : false, /* Boolean  オフラインモード */
  filterLoadList : new Array (),   /* Array  読み込みフィルタリスト
                                    * [[String 種類,
                                    *   String/RegExp パターン,
                                    *   Boolean 正規表現のフラグ], ...]
                                    * String 種類
                                    *   "hostname":  ホスト名
                                    *   "leafname":  ファイル名
                                    *   "extension": 拡張子
                                    *   "uri":  URI */
  
  /* [保存]-[オプション 1] タブの設定 */
  saveFilename : "leafname",            /* String
                                         *   保存時のファイル名 */
  enableSaveContextMenu : false,        /* Boolean  コンテキスメニュー */
  enableShowOptionsDialog : false,      /* Boolean  保存ダイアログの前に
                                         *   オプションダイアログを表示する */
    
  enableShowErrors : false,   /* Boolean  保存されなかったファイルを
                               *   表示する */
  
  /* [保存]-[オプション 2] タブの設定 */
  enableSaveStatusbar : false,          /* Boolean  ステータスバーに状況を
                                         *   表示する */
    
  enableSaveShortcut : false,           /* Boolean  ショートカット */
  saveShortcutKeycode : 0,              /* Number  ショートカットの
                                         *   キーコード */
  saveShortcutModifiersAlt : false,     /* Boolean  ショートカットの Alt */
  saveShortcutModifiersCtrl : false,    /* Boolean  ショートカットの Ctrl */
  saveShortcutModifiersMeta : false,    /* Boolean  ショートカットの Meta */
  saveShortcutModifiersShift : false,   /* Boolean  ショートカットの Shift */

  enableSaveShortcut2 : false,           /* Boolean  ショートカット */
  saveShortcut2Keycode : 0,              /* Number  ショートカットの
                                          *   キーコード */
  saveShortcut2ModifiersAlt : false,     /* Boolean  ショートカットの Alt */
  saveShortcut2ModifiersCtrl : false,    /* Boolean  ショートカットの Ctrl */
  saveShortcut2ModifiersMeta : false,    /* Boolean  ショートカットの Meta */
  saveShortcut2ModifiersShift : false,   /* Boolean  ショートカットの Shift */
  
  dupDigits : 1,              /* 同名のファイルが存在した場合に
                               * ファイル名の末尾に n 桁の数字を追加する */
    
  /* [保存]-[メール] タブの設定 */
  
  sendmailSaveType : "",       /* String  保存形式 */
  sendmailApp : "",            /* String  アプリケーション */
  sendmailArg : "",            /* String  引数 */
  sendmailArgType : "",        /* String  引数の種類 */
  enableSendmailDebug : false, /* Boolean  デバッグモード */
  enableSendmailMenu : false,  /* Boolean  メニューを表示する */
  
  /* [保存]-[Quick Save] タブの設定 */
    
  quicksaveSaveType : "current", /* String  Quick Save 保存方法 */
  quicksaveBase : "",            /* String  保存先 */
  quicksaveAction : "rename",    /* String  同じファイルが存在した場合 */
  enableQuicksaveSelection : false, /* Boolean  選択範囲があれば選択範囲を保存する */
  
  enableLocalToSame : false,       /* Boolean  ローカルファイルは元のファイルと
                                    *   同じディレクトリに保存する */
  enableLocalToSameAppend : false, /* Boolean  元のファイル名に以下のテキストを
                                    *   追加したものを使用する */
  localToSameAppendText : "",      /* String  追加するテキスト */
  enableQuicksaveMenu : false,     /* Boolean  メニューを表示する */
    
  enableQuicksaveShortcut : false,         /* Boolean  ショートカット */
  quicksaveShortcutKeycode : 0,            /* Number  ショートカットの
                                            *   キーコード */
  quicksaveShortcutModifiersAlt : false,   /* Boolean  ショートカットの Alt */
  quicksaveShortcutModifiersCtrl : false,  /* Boolean  ショートカットの Ctrl */
  quicksaveShortcutModifiersMeta : false,  /* Boolean  ショートカットの Meta */
  quicksaveShortcutModifiersShift : false, /* Boolean  ショートカットの Shift */
  
  /* [保存]-[カスタムファイル名] タブの設定 */
    
  saveFilenameCustom : "",    /* String  カスタムファイル名テンプレート */
  saveFilenameCustomAM : "",  /* String  カスタムファイル名テンプレート
                               *   a.m. */
  saveFilenameCustomPM : "",  /* String  カスタムファイル名テンプレート
                               *   p.m. */
    
  /* [その他] タブの設定 */
  enableToolbarPreferences : false,   /* Boolean
                                       *   ツールバーにパネルを表示する */
  enableStatusbarPreferences : false, /* Boolean
                                       *   ステータスバーにパネルを表示する */
  enableToolMenu : false,             /* Boolean
                                       *   ツールメニューに UnMHT の
                                       *   メニューを表示する */
    
  /**
   * ファイル名を正規化する
   *
   * @param  String original
   *         ファイル名
   * @return String
   *         正規化したファイル名
   */
  normalizeName : function (original) {
    if (!original) {
      return original;
    }
    for (var i = 0; i < 10; i ++) {
      var last = original;
      original = UnMHT.conv.unescapeEntity (unescape (original));
      if (last == original) {
        break;
      }
    }
        
    return original;
  },
    
  /**
   * ファイル展開器を取得する
   *
   * @param  String original
   *         ファイル名
   * @return UnMHTExtractor
   *         見付かった場合はファイル展開器
   *         見付からなかった場合は null
   */
  getExtractor : function (original) {
    original = UnMHT.normalizeName (original);
    if (original in UnMHT.extractors) {
      return UnMHT.extractors [original];
    }
        
    return null;
  },
    
  /**
   * ファイル展開器を設定する
   *
   * @param  String original
   *         ファイル名
   * @param  UnMHTExtractor extractor
   *         ファイル展開器
   */
  setExtractor : function (original, extractor) {
    original = UnMHT.normalizeName (original);
    UnMHT.extractors [original] = extractor;
  },
    
  /**
   * ドキュメントごとの情報を追加する
   *
   * @param  HTMLDocument targetDocument
   *         対象のドキュメント
   */
  addDocumentParam : function (targetDocument) {
    var param = new UnMHTDocumentParam ();
    param.targetDocument = targetDocument;
    UnMHT.documentParams.push (param);
    UnMHT.latestParam = param;
  },
    
  /**
   * ドキュメントごとの情報を削除する
   *
   * @param  HTMLDocument targetDocument
   *         対象のドキュメント
   */
  deleteDocumentParam : function (targetDocument) {
    for (var i = 0; i < UnMHT.documentParams.length; i ++) {
      var tmp = UnMHT.documentParams [i];
      if (tmp.targetDocument == targetDocument) {
        UnMHT.documentParams.splice (i, 1);
        tmp.targetDocument = null;
        tmp.location_info = null;
        tmp = null;
        break;
      }
    }
    UnMHT.latestParam = null;
  },
    
  /*
   * ドキュメントごとの情報を取得する
   *
   * @param  HTMLDocument targetDocument
   *         対象のドキュメント
   * @return UnMHTDocumentParam
   *         ドキュメントごとの情報
   */
  getDocumentParam : function (targetDocument) {
    var latest = UnMHT.latestParam;
    if (latest
        && latest.targetDocument == targetDocument) {
      return latest;
    }
        
    for (var i = 0; i < UnMHT.documentParams.length; i ++) {
      if (UnMHT.documentParams [i].targetDocument == targetDocument) {
        return UnMHT.documentParams [i];
      }
    }
        
    return null;
  },
    
  /**
   * SeaMonkey でロケールを修正する
   */
  fixLocale : function () {
    var locales
    = ["en-US",
       "cs-CZ",
       "de",
       "en-GB",
       "es-ES",
       "fr",
       "hu-HU",
       "it-IT",
       "ja-JP",
       "pl-PL",
       "pt-BR",
       "ru-RU",
       "tr-TR",
       "uk-UA",
       "vi",
       "zh-CN",
       "zh-TW"];
        
    try {
      var locale = "en-US";
      try {
        var prefBranch
          = Components.classes ["@mozilla.org/preferences-service;1"]
          .getService (Components.interfaces.nsIPrefBranch);
                
        try {
          locale
            = prefBranch.getComplexValue
            ("general.useragent.locale",
             Components.interfaces.nsIPrefLocalizedString).data;
        }
        catch (e) {
          locale
            = prefBranch.getCharPref ("general.useragent.locale");
        }
      }
      catch (e) {
      }
            
      var select = null;
      for (var i = 0; i < locales.length; i ++) {
        if (locales [i] == locale) {
          select = locales [i];
          break;
        }
      }
      if (!select) {
        for (var i = 0; i < locales.length; i ++) {
          if (locales [i].substr (0, 2) == locale.substr (0, 2)) {
            select = locales [i];
            break;
          }
        }
      }
      if (!select) {
        select = locales [0];
      }
      
      if ("nsIChromeRegistrySea" in Components.interfaces) {
        var registry
          = Components.classes ["@mozilla.org/chrome/chrome-registry;1"]
          .getService (Components.interfaces.nsIChromeRegistrySea);
        registry.selectLocaleForPackage (select, "unmht", false);
        registry.selectLocaleForPackage (select, "unmht", true);
      }
    }
    catch (e) {
    }
  },
    
  /**
   * ウィンドウが開かれたイベント
   * 初期化とイベントのフックを行う
   */
  onLoad : function () {
    if (UnMHT.initialized) {
      return;
    }
    UnMHT.initialized = true;
        
    if (Components.classes ["@mozilla.org/pipe;1"] == undefined) {
      UnMHT.memoryCacheMode = false;
    }
    else {
      UnMHT.memoryCacheMode = true;
    }
    
    var commandset = document.getElementById ("mainCommandSet");
    if (!commandset) {
      /* SeaMonkey では mainCommandSet が無い */
      var mainWindow = document.getElementById ("main-window");
      if (mainWindow) {
        commandset = document.createElement ("commandset");
        commandset.id = "mainCommandSet";
        var command;
        command = document.createElement ("command");
        command.id = "unmht-command-save";
        command.setAttribute ("disabled", "true");
        command.setAttribute ("oncommand",
                              "UnMHTUIManager.save (false);");
        commandset.appendChild (command);
        command = document.createElement ("command");
        command.id = "unmht-command-save-multi";
        command.setAttribute ("oncommand",
                              "UnMHTUIManager.savemulti ();");
        commandset.appendChild (command);
        mainWindow.appendChild (commandset);
      }
    }
        
    try {
      var id
      = Components.classes ["@mozilla.org/xre/app-info;1"]
      .getService (Components.interfaces.nsIXULAppInfo).ID;
      if (id == "{3550f703-e582-4d05-9a08-453d09bdfdc6}") {
        /* Thunderbird */
        UnMHT.isThunderbird = true;
      }
    } catch (e) {}
        
    if (!UnMHT.isThunderbird) {
      var tabbrowser = document.getElementById ("content");
      if (!tabbrowser) {
        tabbrowser = document.getElementById ("browser_content");
      }
      if (tabbrowser
          && "addProgressListener" in tabbrowser) {
        tabbrowser.addProgressListener
          (UnMHTBrowserProgressListener,
           Components.interfaces.nsIWebProgress.NOTIFY_LOCATION);
      }
    }
    else {
      var messagepane = document.getElementById ("messagepane");
      if (messagepane) {
        messagepane.addProgressListener
          (UnMHTBrowserProgressListener,
           Components.interfaces.nsIWebProgress.NOTIFY_LOCATION);
      }
    }
        
    if (parseInt (navigator.productSub) <= 20030621) {
      /* 古い Mozilla Suite の場合 */
      UnMHT.isOld = true;
    }
        
    UnMHT.fixLocale ();
        
    UnMHT.messages
    = Components.classes ["@mozilla.org/intl/stringbundle;1"]
    .getService (Components.interfaces.nsIStringBundleService)
    .createBundle ("chrome://unmht/locale/unmht.properties");
        
    UnMHT.conv = new UnMHTConverter ();
        
    UnMHT.config = new UnMHTConfigManager ();
    UnMHT.config.setListener (UnMHT);
    UnMHT.config.prefBranch.setCharPref ("unmht.version",
                                         UnMHTVersion);
    UnMHT.getConfigurationFromPreferencesAll ();
    UnMHT.getConfigurationFromPreferences ();
    
    if (UnMHT.config.isObservable ()) {
      UnMHT.loadImageType
        = UnMHT.config.prefBranch.getIntPref ("permissions.default.image");
      UnMHT.config.prefBranch.addObserver ("permissions.default.image",
                                           UnMHT,
                                           false);
    }
    
    UnMHTFileManager.init ();
    UnMHTUIManager.init ();
            
    UnMHTUIManager.showPanel ();
    UnMHTUIManager.setPanelStatus ();
        
    if (!UnMHT.isThunderbird) {
      window.addEventListener
        ("keydown",
         function () {
          UnMHT.onKeyDown (arguments [0]);
        }, true);
    }
        
    if (!UnMHT.isThunderbird) {
      var popup;
      popup = document.getElementById ("menu_FilePopup");
      if (popup) {
        popup
          .addEventListener
          ("popupshowing",
           function () {
            UnMHTUIManager.setFileMenuPopup (arguments [0]);
          }, false);
      }
    }
        
    if (!UnMHT.isThunderbird) {
      var sidebar = document.getElementById ("sidebar");
      if (sidebar) {
        sidebar
          .addEventListener
          ("DOMContentLoaded",
           function () {
            UnMHT.checkSidebar (arguments [0]);
          }, false);
      }
    }
        
    var menu = document.getElementById ("contentAreaContextMenu");
    if (!menu) {
      menu = document.getElementById ("popup_content");
    }
    if (menu) {
      menu.addEventListener
        ("popupshowing", 
         function () {
          UnMHTUIManager.setContextMenu (arguments [0]);
        }, false);
    }
        
    if (!UnMHT.isThunderbird) {
      var tabbrowser = document.getElementById ("appcontent");
      if (!tabbrowser) {
        /* タブを持たない場合 */
        tabbrowser = document.getElementById ("content");
        if (!tabbrowser) {
          tabbrowser = document.getElementById ("browser_content");
        }
      }
      if (tabbrowser) {
        tabbrowser.addEventListener
        ("DOMContentLoaded",
         function () {
          UnMHT.onDOMContentLoaded (arguments [0]);
        }, false);
      }
      else {
        /* Fennec の場合 */
        var browsers = document.getElementById ("browsers");
        browsers.addEventListener
        ("DOMNodeInserted",
         function (event) {
          event.target.addEventListener
            ("DOMContentLoaded",
             function () {
              UnMHT.onDOMContentLoaded (arguments [0]);
            }, false);
        }, false);
        var node = browsers.firstChild;
        while (node) {
          node.addEventListener
            ("DOMContentLoaded",
             function () {
              UnMHT.onDOMContentLoaded (arguments [0]);
            }, false);
          node = node.nextSibling;
        }
      }
    }
    
    try {
      var types = [];
      types.push ({
          label : UnMHT.messages.GetStringFromName ("filetype")
            + " "
            + UnMHT.messages.GetStringFromName
            ("filetype_current"),
            ext : "*.mht",
            useCurrentState : true,
            useAdditionalFiles : false
            });
      types.push ({
          label : UnMHT.messages.GetStringFromName ("filetype")
            + " "
            + UnMHT.messages.GetStringFromName
            ("filetype_original"),
            ext : "*.mht",
            useCurrentState : false,
            useAdditionalFiles : false
            });
      types.push ({
          label : UnMHT.messages.GetStringFromName ("filetype")
            + " "
            + UnMHT.messages.GetStringFromName
            ("filetype_current_add"),
            ext : "*.mht",
            useCurrentState : true,
            useAdditionalFiles : true
            });
      types.push ({
          label : UnMHT.messages.GetStringFromName ("filetype")
            + " "
            + UnMHT.messages.GetStringFromName
            ("filetype_original_add"),
            ext : "*.mht",
            useCurrentState : false,
            useAdditionalFiles : true
            });
            
      for (var i = 0; i < types.length; i ++) {
        var newSaveBehavior = new InternalSaveBehavior ();
        newSaveBehavior.isComplete = true;
        newSaveBehavior.mandatoryExtension = true;
        newSaveBehavior.isValidForSaveMode = function (saveMode) {
          return saveMode & 0x01;
        };
                
        newSaveBehavior.getFileFilter
          = (function (label, ext) {
              return function (contentType, fileExtension) {
                var result = {
                  title: label,
                  extensionstring: ext};
                return result;
              };
            })(types [i].label, types [i].ext);
                
        newSaveBehavior.getPersistObject
          = (function (useCurrentState, useAdditionalFiles) {
              return function (saveBrowsers) {
                var result;
                result
                = new UnMHTSavePersist
                (useCurrentState, useAdditionalFiles);
                
                return result;
              };
            })(types [i].useCurrentState,
               types [i].useAdditionalFiles);
                
        gInternalSaveBehaviors.splice
          (gInternalSaveBehaviors.length - 1, 0, newSaveBehavior);
      }
    }
    catch (e) {
    }
  },
    
  /**
   * 設定変更イベント
   *   nsIObserver.observe
   *
   * @param  nsISupports subject
   *         不明
   * @param  String topic
   *         通知の対象
   * @param  String data
   *         通知の内容
   */
  observe : function (subject, topic, data) {
    if (topic == "nsPref:changed") {
      UnMHT.loadImageType
      = UnMHT.config.prefBranch.getIntPref ("permissions.default.image");
    }
  },
  
  /**
   * 設定変更イベント
   */
  onConfigChange : function () {
    /* 設定を取得する */
    UnMHT.getConfigurationFromPreferencesAll ();
    UnMHT.getConfigurationFromPreferences ();
    UnMHTUIManager.showPanel ();
    UnMHTUIManager.setPanelStatus ();
  },
    
  /**
   * 全機能の ON／OFF について、設定を読み込む
   */
  getConfigurationFromPreferencesAll : function () {
    UnMHT.config.loadPrefBranch ();
        
    UnMHT.enableAll
    = UnMHT.config
    .initPref ("bool", "unmht.all", true);
    if (UnMHT.isThunderbird) {
      UnMHT.enableAll = true;
    }
  },
    
  /**
   * ステータスバー、ツールバーのパネルについて、設定を読み込む
   */
  getConfigurationFromPreferencesPanel : function () {
    UnMHT.config.loadPrefBranch ();
        
    UnMHT.enableToolbarPreferences
    = UnMHT.config
    .initPref ("bool", "unmht.toolbar.preferences", false);
    if (UnMHT.isThunderbird) {
      UnMHT.enableToolbarPreferences = false;
    }
        
    UnMHT.enableStatusbarPreferences
    = UnMHT.config
    .initPref ("bool", "unmht.statusbar.preferences", true);
  },
    
  /**
   * 各種設定を読み込む
   */
  getConfigurationFromPreferences : function () {
    UnMHT.config.loadPrefBranch ();
    
    UnMHT.enableNolimit
    = UnMHT.config
    .initPref ("bool", "unmht.nolimit", true);
    UnMHT.nolimitTime
    = UnMHT.config
    .initPref ("int",  "unmht.nolimit.time", 0);
    UnMHT.enableFastExtract
    = UnMHT.config
    .initPref ("bool", "unmht.fast_extract", false);
        
    UnMHT.enableFilterExtract
    = UnMHT.config
    .initPref ("bool", "unmht.filter", false);
    UnMHT.filterExtractList = new Array ();
    var tmp
    = UnMHT.config
    .initPref ("char", "unmht.filter.list", "");
    if (tmp != "") {
      /* 値を解析するだけなので代入はしない */
      tmp.replace
        (/([^&,]*)&([^&,]*)&([^&,]*),?/g,
         function (matched, type, pattern, r) {
          type = unescape (type);
          pattern = unescape (pattern);
          r = unescape (r) == "o";
          if (r) {
            pattern = new RegExp (pattern);
          }
                    
          UnMHT.filterExtractList
            .push (new Array (type,
                              pattern,
                              r));
                    
          return "";
        });
    }
        
    UnMHT.enableFilterLoad
    = UnMHT.config
    .initPref ("bool", "unmht.filter.load", false);
    UnMHT.enableFilterLoadOffline
    = UnMHT.config
    .initPref ("bool", "unmht.filter.load.offline", false);
    UnMHT.filterLoadList = new Array ();
    var tmp
    = UnMHT.config
    .initPref ("char", "unmht.filter.load.list", "");
    if (tmp != "") {
      /* 値を解析するだけなので代入はしない */
      tmp.replace
        (/([^&,]*)&([^&,]*)&([^&,]*),?/g,
         function (matched, type, pattern, r) {
          type = unescape (type);
          pattern = unescape (pattern);
          r = unescape (r) == "o";
          if (r) {
            pattern = new RegExp (pattern);
          }
                    
          UnMHT.filterLoadList
            .push (new Array (type,
                              pattern,
                              r));
                    
          return "";
        });
    }
        
    UnMHT.cacheExpire
    = UnMHT.config
    .initPref ("int",  "unmht.cache_expire", 5)
    * 60000;
    UnMHT.enableCacheExpire
    = UnMHT.config
    .initPref ("bool", "unmht.cache_expire.enabled", true);
        
    UnMHT.enableSkipPPTWarning
    = UnMHT.config
    .initPref ("bool", "unmht.skip_ppt_warning", false);
        
    UnMHT.openMultiple
    = UnMHT.config
    .initPref ("char", "unmht.open_multiple", "tabs");
        
    UnMHT.enableLoadImageType
    = UnMHT.config
    .initPref ("bool", "unmht.load_image_type", true);
    
    UnMHT.enableUseGlobalLastDir
    = UnMHT.config
    .initPref ("bool", "unmht.use_global_lastdir", false);
    
    UnMHT.enableLocalToSame
    = UnMHT.config
    .initPref ("bool", "unmht.save.local_to_same", false);
    UnMHT.enableLocalToSameAppend
    = UnMHT.config
    .initPref ("bool", "unmht.save.local_to_same.append", false);
    value
    = UnMHT.config
    .initPref ("char", "unmht.save.local_to_same.append.text", "(modified)");
    UnMHT.localToSameAppendText = unescape (value)
    
    UnMHT.enableQuicksaveSelection
    = UnMHT.config
    .initPref ("bool", "unmht.quicksave.selection", false);
    
    UnMHT.enableQuicksaveShortcut
    = UnMHT.config
    .initPref ("bool", "unmht.quicksave.shortcut", false);
        
    value
    = UnMHT.config
    .initPref ("char", "unmht.quicksave.shortcut.keycode",
               "VK_Q");
    value
    = unescape (value);
    UnMHT.quicksaveShortcutKeycode
    = Components.interfaces.nsIDOMKeyEvent ["DOM_" + value];
        
    UnMHT.quicksaveShortcutModifiersAlt
    = UnMHT.config
    .initPref ("bool",
               "unmht.quicksave.shortcut.modifiers.alt",
               true);
    UnMHT.quicksaveShortcutModifiersCtrl
    = UnMHT.config
    .initPref ("bool",
               "unmht.quicksave.shortcut.modifiers.ctrl",
               true);
    UnMHT.quicksaveShortcutModifiersMeta
    = UnMHT.config
    .initPref ("bool",
               "unmht.quicksave.shortcut.modifiers.meta",
               false);
    UnMHT.quicksaveShortcutModifiersShift
    = UnMHT.config
    .initPref ("bool",
               "unmht.quicksave.shortcut.modifiers.shift",
               false);
    
    UnMHT.saveFilename
    = UnMHT.config
    .initPref ("char", "unmht.save.filename", "title");
        
    var value;

    value
    = UnMHT.config
    .initPref ("char", "unmht.save.filename.custom", "%FILENAME%");
    UnMHT.saveFilenameCustom = unescape (value);
        
    value
    = UnMHT.config
    .initPref ("char", "unmht.save.filename.custom.am", "a.m.");
    UnMHT.saveFilenameCustomAM = unescape (value);
        
    value
    = UnMHT.config
    .initPref ("char", "unmht.save.filename.custom.pm", "p.m.");
    UnMHT.saveFilenameCustomPM = unescape (value);
    
    value
    = UnMHT.config
    .initPref ("char", "unmht.sendmail.savetype", "current");
    UnMHT.sendmailSaveType = unescape (value);
    value
    = UnMHT.config
    .initPref ("char", "unmht.sendmail.app", "");
    UnMHT.sendmailApp = unescape (value);
    value
    = UnMHT.config
    .initPref ("char", "unmht.sendmail.argtype", "thunderbird");
    UnMHT.sendmailArgType = unescape (value);
    value
    = UnMHT.config
    .initPref ("char", "unmht.sendmail.arg", escape (UnMHTMail.thunderbird));
    UnMHT.sendmailArg = unescape (value);
    if (UnMHT.sendmailArgType == "thunderbird") {
      UnMHT.sendmailArg = UnMHTMail.thunderbird;
    }
    else if (UnMHT.sendmailArgType == "outlook") {
      UnMHT.sendmailArg = UnMHTMail.outlook;
    }
    else if (UnMHT.sendmailArgType == "winmail") {
      UnMHT.sendmailArg = UnMHTMail.winmail;
    }
    UnMHT.enableSendmailDebug
    = UnMHT.config
    .initPref ("bool", "unmht.sendmail.debug", false);
    UnMHT.enableSendmailMenu
    = UnMHT.config
    .initPref ("bool", "unmht.sendmail.showmenu", true);
    
    value
    = UnMHT.config
    .initPref ("char", "unmht.quicksave.savetype", "current");
    UnMHT.quicksaveSaveType = unescape (value);
    value
    = UnMHT.config
    .initPref ("char", "unmht.quicksave.base", "");
    UnMHT.quicksaveBase = unescape (value);
    value
    = UnMHT.config
    .initPref ("char", "unmht.quicksave.autoreplace.action", "rename");
    UnMHT.quicksaveAction = unescape (value);
    UnMHT.enableQuicksaveMenu
    = UnMHT.config
    .initPref ("bool", "unmht.quicksave.showmenu", false);
        
    UnMHT.dupDigits
    = UnMHT.config
    .initPref ("int", "unmht.save.dup.digits", 1);
    
    if (UnMHT.isThunderbird) {
      UnMHT.saveFilename = "title";
    }
    
    UnMHT.enableSaveStatusbar
    = UnMHT.config
    .initPref ("bool", "unmht.save.statusbar", false);
        
    UnMHT.enableSaveContextMenu
    = UnMHT.config
    .initPref ("bool", "unmht.save.contextmenu", true);
    
    UnMHT.enableShowOptionsDialog
    = UnMHT.config
    .initPref ("bool", "unmht.save.show_option_dialog", false);

    UnMHT.enableShowErrors
    = UnMHT.config
    .initPref ("bool", "unmht.save.show_errors", true);
    
    UnMHT.enableSaveShortcut
    = UnMHT.config
    .initPref ("bool", "unmht.save.shortcut", false);
        
    value
    = UnMHT.config
    .initPref ("char", "unmht.save.shortcut.keycode",
               "VK_S");
    value
    = unescape (value);
    UnMHT.saveShortcutKeycode
    = Components.interfaces.nsIDOMKeyEvent ["DOM_" + value];
        
    UnMHT.saveShortcutModifiersAlt
    = UnMHT.config
    .initPref ("bool",
               "unmht.save.shortcut.modifiers.alt",
               true);
    UnMHT.saveShortcutModifiersCtrl
    = UnMHT.config
    .initPref ("bool",
               "unmht.save.shortcut.modifiers.ctrl",
               true);
    UnMHT.saveShortcutModifiersMeta
    = UnMHT.config
    .initPref ("bool",
               "unmht.save.shortcut.modifiers.meta",
               false);
    UnMHT.saveShortcutModifiersShift
    = UnMHT.config
    .initPref ("bool",
               "unmht.save.shortcut.modifiers.shift",
               false);
        
    UnMHT.enableSaveShortcut2
    = UnMHT.config
    .initPref ("bool", "unmht.save.shortcut2", false);
        
    value
    = UnMHT.config
    .initPref ("char", "unmht.save.shortcut2.keycode",
               "VK_S");
    value
    = unescape (value);
    UnMHT.saveShortcut2Keycode
    = Components.interfaces.nsIDOMKeyEvent ["DOM_" + value];
        
    UnMHT.saveShortcut2ModifiersAlt
    = UnMHT.config
    .initPref ("bool",
               "unmht.save.shortcut2.modifiers.alt",
               true);
    UnMHT.saveShortcut2ModifiersCtrl
    = UnMHT.config
    .initPref ("bool",
               "unmht.save.shortcut2.modifiers.ctrl",
               true);
    UnMHT.saveShortcut2ModifiersMeta
    = UnMHT.config
    .initPref ("bool",
               "unmht.save.shortcut2.modifiers.meta",
               false);
    UnMHT.saveShortcut2ModifiersShift
    = UnMHT.config
    .initPref ("bool",
               "unmht.save.shortcut2.modifiers.shift",
               false);
    
    UnMHT.enableToolMenu
    = UnMHT.config
    .initPref ("bool", "unmht.toolmenu.preferences", true);
    
    var menu;
    menu = document.getElementById ("unmht-tools-preferences");
    if (menu) {
      menu.hidden = !UnMHT.enableToolMenu;
    }
  },
    
  /**
   * キーが押されたイベント
   *
   * @param  Event event
   *         対象のイベント
   */
  onKeyDown : function (event) {
    if (UnMHT.enableAll) {
      if (UnMHT.enableSaveShortcut) {
        if (UnMHT.saveShortcutKeycode == event.keyCode
            && UnMHT.saveShortcutModifiersAlt == event.altKey
            && UnMHT.saveShortcutModifiersCtrl == event.ctrlKey
            && UnMHT.saveShortcutModifiersMeta == event.metaKey
            && UnMHT.saveShortcutModifiersShift == event.shiftKey) {
                
          var tabbrowser = document.getElementById ("content");
          if (!tabbrowser) {
            tabbrowser = document.getElementById ("browser_content");
          }
          if (tabbrowser) {
            var targetDocument = tabbrowser.contentDocument;
            if (!UnMHT.isSavable (targetDocument)) {
              return;
            }
                
            UnMHTUIManager.save (false);
          }
                
          event.preventDefault ();
          event.stopPropagation ();
        }
      }
      if (UnMHT.enableSaveShortcut2) {
        if (UnMHT.saveShortcut2Keycode == event.keyCode
            && UnMHT.saveShortcut2ModifiersAlt == event.altKey
            && UnMHT.saveShortcut2ModifiersCtrl == event.ctrlKey
            && UnMHT.saveShortcut2ModifiersMeta == event.metaKey
            && UnMHT.saveShortcut2ModifiersShift == event.shiftKey) {
                
          var tabbrowser = document.getElementById ("content");
          if (!tabbrowser) {
            tabbrowser = document.getElementById ("browser_content");
          }
          if (tabbrowser) {
            var targetDocument = tabbrowser.contentDocument;
            if (!UnMHT.isSavable (targetDocument)) {
              return;
            }
                
            UnMHTUIManager.savemulti ();
          }
                
          event.preventDefault ();
          event.stopPropagation ();
        }
      }
      if (UnMHT.enableQuicksaveShortcut) {
        if (UnMHT.quicksaveShortcutKeycode == event.keyCode
            && UnMHT.quicksaveShortcutModifiersAlt == event.altKey
            && UnMHT.quicksaveShortcutModifiersCtrl == event.ctrlKey
            && UnMHT.quicksaveShortcutModifiersMeta == event.metaKey
            && UnMHT.quicksaveShortcutModifiersShift == event.shiftKey) {
                
          var tabbrowser = document.getElementById ("content");
          if (!tabbrowser) {
            tabbrowser = document.getElementById ("browser_content");
          }
          if (tabbrowser) {
            var targetDocument = tabbrowser.contentDocument;
            if (!UnMHT.isSavable (targetDocument)) {
              return;
            }
                
            UnMHTUIManager.quicksave ();
          }
                
          event.preventDefault ();
          event.stopPropagation ();
        }
      }
    }
  },
    
  /**
   * 保存可能かどうかを調べる
   *
   * @param  HTMLDocument targetDocument
   *         対象のドキュメント
   * @return Boolean
   *         保存可能かどうか
   */
  isSavable : function (targetDocument) {
    try {
      if (gIeTab.isIeEngine ()) {
        return false;
      }
    }
    catch (e) {
    }
    
    if (UnMHT.isThunderbird) {
      if (targetDocument.location.href == "about:blank") {
        return false;
      }
      return true;
    }
    else if (targetDocument.contentType.match (/^text/)
             || targetDocument.contentType.match (/\/x(ht)?ml(\+xml)?$/)) {
      return true;
    }
    else {
      return false;
    }
  },
  
  /**
   * サイドバーの内容が ScrapBook かどうかチェック
   * ScrapBook ならばメニューの項目を追加する
   *
   * @param  Event event
   *         対象のイベント
   */
  checkSidebar : function (event) {
    var sidebarDocument = event.target;
    var popup = sidebarDocument.getElementById ("sbPopup");
    var item = sidebarDocument.getElementById ("sbPopupTools");
    if (popup && item) {
      var newItem;
      label
        = UnMHT.messages.GetStringFromName ("save_title");
      newItem = sidebarDocument.createElement ("menuitem");
      newItem.id = "unmht-scrapbook-save";
      newItem.setAttribute ("label", label);
      newItem
        .addEventListener
        ("command",
         function () {
          UnMHTUIManager.save (true);
        }, false);
      popup.insertBefore (newItem, item);
            
      popup
        .addEventListener
        ("popupshowing",
         function () {
          UnMHTUIManager.setScrapBookPopup (arguments [0]);
        }, false);
    }
  },
    
  /**
   * ウィンドウが閉じられたイベント
   * キャッシュの削除を行う
   */
  onUnload : function () {
    if (!UnMHT.initialized) {
      return;
    }
    
    UnMHT.config.resetListener ();
    UnMHT.clearCache (0, false);
    
    UnMHT.initialized = false;
  },
    
  /**
   * body の unload イベント
   * 各種データを削除する
   *
   * @param  HTMLDocument targetDocument
   *         対象のドキュメント
   * @param  Event event
   *         対象のイベント
   */
  onBodyUnload : function (targetDocument, event) {
    var listener;
    var param;
        
    param = UnMHT.getDocumentParam (targetDocument);
    if (!param) {
      return;
    }
        
    listener = param.history_listener;
    if (listener) {
      try {
        listener.destruct ();
      }
      catch (e) {
      }
    }
    param.history_listener = null;
        
    UnMHT.deleteDocumentParam (targetDocument);
  },
    
  /**
   * コンテンツがロードされたイベント
   * mht ファイルの展開、アドレスの変換を行う
   *
   * @param  Event event
   *         対象のイベント
   */
  onDOMContentLoaded : function (event) {
    var targetWindow = event.target.defaultView;
    var targetDocument = targetWindow.document;
        
    if (Components.interfaces.nsIPrefBranch2 == undefined) {
      /* 監視していない場合にのみ設定を取得する */
      UnMHT.getConfigurationFromPreferencesAll ();
    }
        
    if (UnMHT.enableAll) {
      if (targetDocument.location.href.match (/[^\/]\.(mht(ml)?|eml)$/)) {
        /* Content-Type 指定のない mht ファイルだった場合 */
                
        if (targetDocument.location.href.match
            (/^chrome:\/\/ietab\//)) {
          /* IETab がハンドル済み */
          return;
        }
                
        var uri
        = Components.classes ["@mozilla.org/network/standard-url;1"]
        .createInstance (Components.interfaces.nsIURI);
        uri.spec = targetDocument.location.href;
                
        UnMHT.loadMHT (uri, targetWindow,
                       true,  /* 開いているドキュメントを使用する */
                       true,  /* 履歴を置き換える */
                       1,     /* 強制的に展開する */
                       "",    /* ルートドキュメントを開く */
                       true); /* キャッシュを使用する */
      }
      else if (UnMHT.protocolHandler.isUnMHTURI
               (targetDocument.location.href)) {
        /* UnMHT の開いたファイルだった場合 */
                
        UnMHT.modifyRootDocument (targetDocument, targetWindow);
                
        if (UnMHT.openMultiple == "tabs"
            || UnMHT.openMultiple == "first") {
          var param
            = UnMHT.protocolHandler.getUnMHTURIParam
            (targetDocument.location.href);
          var original = param.original;
          var refName = param.refName;
          var extractor;
          if (refName == ""
              && original
              && (extractor = UnMHT.getExtractor (original))) {
            if (extractor.firstFile.headerMap != null
                && "x-unmht-tabs"
                in extractor.firstFile.headerMap) {
              /* 複数のファイルを含む MHT の
               * インデックスを開いた */
              UnMHT.replaceTabs (targetDocument,
                                 targetWindow);
            }
          }
        }
      }
    }
  },
    
  /**
   * UnMHT の開いたファイルのリンクを書き換える
   * 
   * @param  HTMLDocument targetDocument
   *         対象のドキュメント
   * @param  Window targetWindow
   *         対象のウィンドウ
   */
  modifyRootDocument : function (targetDocument, targetWindow) {
    if (!UnMHT.protocolHandler.isUnMHTURI (targetDocument.location.href)) {
      return;
    }
        
    var param
    = UnMHT.protocolHandler.getUnMHTURIParam (targetDocument.location.href);
    var original = param.original;
    var protocol = param.protocol;
    var refName = param.refName;
    if (protocol != "cache"
        && protocol != "error"
        && protocol != "dummy"
        && protocol != "cid") {
      var extractor = null;
      var file = null;
      if (original
          && (extractor = UnMHT.getExtractor (original))) {
        /* 展開済みのファイルだった場合 */
                
        if (refName
            && refName in extractor.refFiles) {
          file = extractor.refFiles [refName];
        }
        else {
          file = extractor.rootFile;
        }
        
        if (file == extractor.rootFile) {
          /* ルートドキュメントなので
           * リロードとアンロードをフックする */
          UnMHT.hookEvent (targetDocument, targetWindow);
        }
                        
        /* base を削除する */
        var base = file.baseDir;
        var nodes = targetDocument.getElementsByTagName ("base");
        for (var i = 0; i < nodes.length;) {
          if (nodes [i].hasAttribute ("href")) {
            nodes [i].parentNode.removeChild (nodes [i]);
          }
          else {
            i ++;
          }
        }
                
        /* リンク先を書き換える */
        nodes = targetDocument.getElementsByTagName ("a");
        for (var i = 0; i < nodes.length; i ++) {
          var path = nodes [i].getAttribute ("href");
          if (!path) {
            continue;
          }
          if (path.substr (0, 7) == "mailto:") {
            /* mailto: は置き換えない */
            continue;
          }
          
          if ("className" in nodes [i]
              && nodes [i].className == "unmht_link_to_original"
              && UnMHT.protocolHandler.isUnMHTURI (path)) {
            var param
              = UnMHT.protocolHandler.getUnMHTURIParam (path);
            var refName = param.refName;
            
            var file2 = extractor.refFiles [refName];
            /* オリジナルへのリンクは元に戻す */
            nodes [i].setAttribute
              ("href",
               file2.contentLocation);
            
            continue;
          }
          
          var list
            = UnMHT.checkFile (extractor, file, path, false);
          
          var newPath = "";
          if (list [0] != -1
              && list [1]) {
            /* ファイルが MHT に含まれており、展開した場合 */
            newPath
              = extractor.rootURI
              + list [1].refName + list [2];
          }
          else {
            /* ファイルが MHT に含まれていないか
             * 展開しなかった場合 */
            /* base を削除したので置き換える */
            newPath = UnMHTFileManager.jointPath (base, path);
          }
                    
          if (path != newPath) {
            nodes [i].setAttribute ("__unmht_href", path);
            nodes [i].setAttribute ("href", newPath);
          }
        }
        /* action を書き換える */
        nodes = targetDocument.getElementsByTagName ("form");
        for (var i = 0; i < nodes.length; i ++) {
          var path = nodes [i].getAttribute ("action");
          if (!path) {
            continue;
          }
                    
          var list
            = UnMHT.checkFile (extractor, file, path, false);
                    
          var newPath = "";
          if (list [0] != -1
              && list [1]) {
            /* ファイルが MHT に含まれており、展開した場合 */
            newPath
              = extractor.rootURI
              + list [1].refName + list [2];
          }
          else {
            /* ファイルが MHT に含まれていないか
             * 展開しなかった場合 */
            /* base を削除したので置き換える */
            newPath = UnMHTFileManager.jointPath (base, path);
          }
          
          if (path != newPath) {
            nodes [i].setAttribute ("__unmht_action", path);
            nodes [i].setAttribute ("action", newPath);
          }
        }
      }
      else {
        /* 未展開のファイルだった場合 */
                
        var uri
          = Components.classes ["@mozilla.org/network/standard-url;1"]
          .createInstance (Components.interfaces.nsIURI);
        uri.spec = original;
        UnMHT.loadMHT (uri, targetWindow,
                       false,  /* ドキュメント無し */
                       true,   /* 履歴を置き換える */
                       0,      /* 展開は設定に任せる */
                       original,  /* 指定されたファイルを開く */
                       false); /* キャッシュは使用しない */
      }
    }
  },
    
  /**
   * インデックスファイルをタブに置き換える
   * 
   * @param  HTMLDocument targetDocument
   *         対象のドキュメント
   * @param  Window targetWindow
   *         対象のウィンドウ
   */
  replaceTabs : function (targetDocument, targetWindow) {
    var nodes = targetDocument.getElementsByTagName ("a");
    var tabbrowser = document.getElementById ("content");
    if (!tabbrowser) {
      tabbrowser = document.getElementById ("browser_content");
    }
    
    if (!("addTab" in tabbrowser)) {
      return;
    }
    
    try {
      for (var i = 0; i < nodes.length; i ++) {
        var node = nodes [i];
        if ("className" in node
            && node.className == "unmht-tabs") {
          tabbrowser.addTab (node.href);
        }
        
        if (UnMHT.openMultiple == "first") {
          break;
        }
      }
            
      var tab = UnMHT.getTabForWindow (targetWindow);
      if (tab) {
        tabbrowser.removeTab (tab);
      }
    }
    catch (e) {
    }
  },
    
  /**
   * 対象のウィンドウを持つ tab オブジェクトを取得する
   * 
   * @param  Window targetWindow
   *         対象のウィンドウ
   * @return Tab
   *         見付からなければ null
   */
  getTabForWindow : function (targetWindow) {
    var tabbrowser = document.getElementById ("content");
    if (!tabbrowser) {
      tabbrowser = document.getElementById ("browser_content");
    }
    if (tabbrowser
        && tabbrowser.mTabContainer) {
      for (var i = 0; i < tabbrowser.mTabContainer.childNodes.length; i ++) {
        var tab = tabbrowser.mTabContainer.childNodes [i];
        if (tab.linkedBrowser
            && tab.linkedBrowser
            .contentWindow == targetWindow) {
          return tab;
        }
      }
            
      if ("mPanelContainer" in tabbrowser
          && "childNodes" in tabbrowser.mPanelContainer) {
        for (var i = 0; i < tabbrowser.mPanelContainer.childNodes.length;
             i ++) {
          var b = tabbrowser.mPanelContainer.childNodes [i];
          if (b.contentWindow == targetWindow) {
            return tabbrowser.mTabContainer.childNodes [i];
          }
        }
      }
    }
        
    return null;
  },
    
  /**
   * リロードとアンロードをフックする
   * 
   * @param  HTMLDocument targetDocument
   *         対象のドキュメント
   * @param  Window targetWindow
   *         対象のウィンドウ
   */
  hookEvent : function (targetDocument, targetWindow) {
    try {
      var nav
      = targetWindow
      .QueryInterface (Components.interfaces.nsIInterfaceRequestor)
      .getInterface (Components.interfaces.nsIWebNavigation);
                                
      var listener = new UnMHTSHistoryListener ();
      listener.targetWindow = targetWindow;
      nav.sessionHistory.addSHistoryListener (listener);
            
      UnMHT.addDocumentParam (targetDocument);
      UnMHT.getDocumentParam (targetDocument).history_listener
      = listener;
            
      targetWindow.addEventListener
      ("unload",
       function () {
        UnMHT.onBodyUnload (targetDocument, arguments [0]);
      }, true);
    }
    catch (e) {
    }
  },
    
  /**
   * ファイルサイズを "," で区切る
   *
   * @param  Number size
   *         ファイルサイズ
   * @return String
   *         "," で区切られたファイルサイズ
   */
  getReadableSize : function (size) {
    var readableSize = "" + size;
        
    if (readableSize.length > 3) {
      readableSize
        = UnMHT.getReadableSize (readableSize
                                 .substr (0, readableSize.length - 3))
        + "," + readableSize.substr (readableSize.length - 3);
    }
        
    return readableSize;
  },
    
  /**
   * 時刻を可読形式に変換する
   *
   * @param  Number time
   *         時刻
   * @return String
   *         時刻の可読形式
   */
  getReadableTime : function (time) {
    time = new Date (time);
    var n = 0;
    var timeString = (time.getYear () + 1900) + "/";
    n = time.getMonth () + 1;
    if (n < 10) {
      timeString += "0";
    }
    timeString += n + "/";
    n = time.getDate ();
    if (n < 10) {
      timeString += "0";
    }
    timeString += n + " ";
    var weekDay = [
      UnMHT.messages.GetStringFromName ("weekday_sun"),
      UnMHT.messages.GetStringFromName ("weekday_mon"),
      UnMHT.messages.GetStringFromName ("weekday_tue"),
      UnMHT.messages.GetStringFromName ("weekday_wed"),
      UnMHT.messages.GetStringFromName ("weekday_thu"),
      UnMHT.messages.GetStringFromName ("weekday_fri"),
      UnMHT.messages.GetStringFromName ("weekday_sat")
      ];
    timeString += "(" + weekDay [time.getDay ()] + ") ";
        
    n = time.getHours ();
    if (n < 10) {
      timeString += "0";
    }
    timeString += n + ":";
    n = time.getMinutes ();
    if (n < 10) {
      timeString += "0";
    }
    timeString += n + ":";
    n = time.getSeconds ();
    if (n < 10) {
      timeString += "0";
    }
    timeString += n + ".";
    n = time.getMilliseconds ();
    if (n < 100) {
      timeString += "0";
    }
    if (n < 10) {
      timeString += "0";
    }
    timeString += n;
        
    return timeString;
  },
    
  /**
   * キャッシュページのヘッダを返す
   *
   * @param  Number entry
   *         キャッシュエントリ数
   * @param  Number size
   *         キャッシュサイズ
   * @return String
   *         キャッシュページのヘッダ
   */
  getCacheHead : function (entry, size) {
    var head;
        
    var readableSize
    = UnMHT.getReadableSize (size);
        
    head
    = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
    + "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\""
    + "          \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">"
    + "<html xmlns=\"http://www.w3.org/1999/xhtml\">"
    + "<head>"
    + "<title>"
    + UnMHT.messages.GetStringFromName ("cache_title")
    + "</title>"
    + "<style type=\"text/css\">"
    + "th {"
    + "text-align: right;"
    + "white-space: nowrap;"
    + "}"
    + "table {"
    + "margin-bottom: 1em;"
    + "}"
    + "table.file {"
    + "margin-bottom: 1em;"
    + "margin-left: 1em;"
    + "}"
    + "</style>"
    + "</head>"
    + "<body>"
    + "<h1>"
    + UnMHT.messages.GetStringFromName ("cache_title")
    + "</h1>"
        
    + "<hr />"
    + "<table border=\"0\">"
        
    + "<tr>"
    + "<th>"
    + UnMHT.messages.GetStringFromName ("cache_entry") + " "
    + "</th>"
    + "<td>"
    + entry
    + "</td>"
    + "</tr>"
        
    + "<tr>"
    + "<th>"
    + UnMHT.messages.GetStringFromName ("cache_size") + " "
    + "</th>"
    + "<td>"
    + readableSize + " "
    + UnMHT.messages.GetStringFromName ("bytes")
    + "</td>"
    + "</tr>"
        
    + "</table>"
    + "<hr />";
        
    return UnMHT.conv.convertFromUnicode (head, "utf-8");
  },
    
  /**
   * キャッシュページのフッタを返す
   *
   * @return String
   *         キャッシュページフッタ
   */
  getCacheFoot : function () {
    var foot;
        
    foot
    = "</body>"
    + "</html>";
        
    return UnMHT.conv.convertFromUnicode (foot, "utf-8");
  },
    
  /**
   * エラーページを返す
   *
   * @param  String location
   *         ファイルの場所
   * @return String
   *         エラーページ
   */
  getErrorPage : function (location) {
    var error;
        
    error
    = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
    + "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\""
    + "          \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">"
    + "<html xmlns=\"http://www.w3.org/1999/xhtml\">"
    + "<head>"
    + "<title>"
    + UnMHT.messages.GetStringFromName ("error_title")
    + "</title>"
    + "<style type=\"text/css\">"
    + "h1 {"
    + "color: #ff0000;"
    + "background-color: inherit;"
    + "}"
    + "div.message {"
    + "font-size: 1em;"
    + "color: #ff0000;"
    + "background-color: inherit;"
    + "}"
    + "</style>"
    + "</head>"
    + "<body>"
    + "<h1>"
    + UnMHT.messages.GetStringFromName ("error_title")
    + "</h1>"
        
    + "<hr />"
        
    + "<div class=\"message\">"
    + UnMHT.messages.GetStringFromName ("error_file_not_found")
    + "<br>"
    + location
    + "</div>"
        
    + "<hr />"
        
    + "</body>"
    + "</html>";
        
    return UnMHT.conv.convertFromUnicode (error, "utf-8");
  },
    
  /**
   * キャッシュ情報を返す
   *
   * @return String
   *         キャッシュ情報の HTML 断片
   */
  getCacheInfo : function () {
    var i, j;
    var s = "";
    var size = 0;
    var extractor;
        
    var urispec;
    var extractors = new Array ();
        
    for (urispec in UnMHT.extractors) {
      extractors.push (UnMHT.extractors [urispec]);
    }
        
    extractors.sort (function (x, y) {
        return y.lastAccessTime - x.lastAccessTime;
      });
        
    for (i = 0; i < extractors.length; i ++) {
      if (i > 0) {
        s += "<hr />";
      }
      extractor = extractors [i];
      size += extractor.size;
            
      var original;
            
      if (UnMHT.protocolHandler.isUnMHTURI (extractor.rootURI)) {
        var param
          = UnMHT.protocolHandler.getUnMHTURIParam
          (extractor.rootURI);
        original = param.original;
                
        if (param.protocol == "file") {
          original
            = UnMHTFileManager.getFilenameFromURLSpec (original);
        }
      }
      else {
        original = "?";
      }
            
      var openedString = UnMHT.getReadableTime (extractor.extractTime);
      var lastAccessString
        = UnMHT.getReadableTime (extractor.lastAccessTime);
            
      var count = "";
            
      count
        = extractor.count + "/" +  extractor.files.length;
            
      s += "<table border=\"0\">"
        + "<tr>"
        + "<th>"
        + "URI: "
        + "</th>"
        + "<td>"
        + "<a href=\"" + extractor.rootURI + "\">"
        + UnMHT.conv.escapeEntity (extractor.rootURI) + "</a>"
        + "</td>"
        + "</tr>"
                
        + "<tr>"
        + "<th>"
        + UnMHT.messages.GetStringFromName ("original_uri") + " "
        + "</th>"
        + "<td>"
        + UnMHT.conv.escapeEntity (original)
        + "</td>"
        + "</tr>"
                
        + "<tr>"
        + "<th>"
        + UnMHT.messages.GetStringFromName ("opened_time") + " "
        + "</th>"
        + "<td>"
        + UnMHT.conv.escapeEntity (openedString)
        + "</td>"
        + "</tr>"
                
        + "<tr>"
        + "<th>"
        + UnMHT.messages.GetStringFromName ("last_access_time") + " "
        + "</th>"
        + "<td>"
        + UnMHT.conv.escapeEntity (lastAccessString)
        + "</td>"
        + "</tr>"
                
        + "<tr>"
        + "<th>"
        + UnMHT.messages.GetStringFromName ("file_count") + " "
        + "</th>"
        + "<td>"
        + UnMHT.conv.escapeEntity (count)
        + "</td>"
        + "</tr>";
                
      if (extractor.date) {
        var d = null;
        try {
          d = new Date (extractor.date);
        }
        catch (e) {
          d = null;
        }
                
        if (d) {
          var t = d.getTime ();
          var savedString;
          if (t) {
            savedString = UnMHT.getReadableTime (t);
          }
          else {
            /* 未対応のフォーマット */
            savedString = extractor.date;
          }
            
          s += "<tr>"
            + "<th>"
            + UnMHT.messages.GetStringFromName ("saved_time") + " "
            + "</th>"
            + "<td>"
            + UnMHT.conv.escapeEntity (savedString)
            + "</td>"
            + "</tr>";
        }
      }
            
      if (extractor.subject) {
        s += "<tr>"
          + "<th>"
          + "Subject: "
          + "</th>"
          + "<td>"
          + UnMHT.conv.escapeEntity (extractor.subject)
          + "</td>"
          + "</tr>";
      }

      if (extractor.location) {
        s += "<tr>"
          + "<th>"
          + "Content-Location: "
          + "</th>"
          + "<td>"
          + UnMHT.conv.escapeEntity (extractor.location)
          + "</td>"
          + "</tr>";
      }
            
      s += "</table>";
            
      for (var j = 0; j < extractor.files.length; j ++) {
        var file = extractor.files [j];
                
        var refName = "";
                
        if (!file.deleted) {
          refName
            = "<a href=\"" + extractor.rootURI + file.refName
            + "\">"
            + UnMHT.conv.escapeEntity (file.refName)
            + "</a>";
        }
        else {
          refName = UnMHT.conv.escapeEntity (file.refName);
        }
                
        var isRoot = "";
                
        if (extractor.rootFile == file) {
          isRoot = " ("
            + UnMHT.messages.GetStringFromName ("root_document")
            + ")";
        }
                
        var deleted = "";
        if (file.deleted) {
          deleted = " style=\"color: #808080;\"";
        }
                
        var charset = "";
                
        if (file.filetype.match (/^text\//)
            && file.charset) {
          charset = "charset=" + file.charset + "; ";
        }
                
        s += "<table border=\"0\" class=\"file\"" + deleted +">"
                    
          + "<tr>"
          + "<th>"
          + UnMHT.messages.GetStringFromName ("ref_name") + " "
          + "</th>"
          + "<td>"
          + refName + isRoot
          + "</td>"
          + "</tr>";
                
        if (file.contentLocation) {
          var location = file.contentLocation;
          location
            = location
            .replace (/^unmht:\/\/\/dummy\.([0-9]+)\//,
                      function (matched, sep) {
                        var sep1 = (sep & 1) ? "//" : "";
                        var sep2 = (sep & 2) ? "//" : "";
                        var sep3 = (sep & 4) ? "/" : "";
                        return "file:" + sep1 + sep2 + sep3;
                      })
            .replace (/^unmht:\/\/\/cid\.0\//, "cid:");
          s += "<tr>"
            + "<th>"
            + "Content-Location: "
            + "</th>"
            + "<td>"
            + UnMHT.conv.escapeEntity (location)
            + "</td>"
            + "</tr>";
          s += "<tr>"
            + "<th>"
            + "leafName: "
            + "</th>"
            + "<td>"
            + UnMHT.conv.escapeEntity (file.leafName)
            + "</td>"
            + "</tr>";
        }
        if (file.contentBase) {
          var location = file.contentBase;
          location
            = location
            .replace (/^unmht:\/\/\/dummy\.([0-9]+)\//,
                      function (matched, sep) {
                        var sep1 = (sep & 1) ? "//" : "";
                        var sep2 = (sep & 2) ? "//" : "";
                        var sep3 = (sep & 4) ? "/" : "";
                        return "file:" + sep1 + sep2 + sep3;
                      })
            .replace (/^unmht:\/\/\/cid\.0\//, "cid:");
          s += "<tr>"
            + "<th>"
            + "Content-Base: "
            + "</th>"
            + "<td>"
            + UnMHT.conv.escapeEntity (location)
            + "</td>"
            + "</tr>";
        }
        if (file.cid) {
          s += "<tr>"
            + "<th>"
            + "Content-ID: "
            + "</th>"
            + "<td>"
            + UnMHT.conv.escapeEntity (file.cid)
            + "</td>"
            + "</tr>";
        }
                
        if (file.disposition) {
          s += "<tr>"
            + "<th>"
            + "Content-Disposition: "
            + "</th>"
            + "<td>"
            + UnMHT.conv.escapeEntity (file.dispositionType)
            + "; filename="
            + UnMHT.conv.escapeEntity (file.disposition)
            + "</td>"
            + "</tr>";
        }
                
        s += "<tr>"
          + "<th>"
          + "Content-Type: "
          + "</th>"
          + "<td>"
          + UnMHT.conv.escapeEntity (charset + file.filetype)
          + "</td>"
          + "</tr>";
                
        if (!file.deleted) {
          var encoding = "";
                    
          if (file.encoding == 1) {
            encoding = "quoted-printable";
          }
          else if (file.encoding == 2) {
            encoding = "base64";
          }
          else if (file.encoding == 3) {
            encoding = "7bit";
          }
          else {
            encoding = "8bit";
          }
 
          var readableSize
            = UnMHT.getReadableSize (file.size);
                    
          s += "<tr>"
            + "<th>"
            + "Content-Encoding: "
            + "</th>"
            + "<td>"
            + UnMHT.conv.escapeEntity (encoding)
            + "</td>"
            + "</tr>";
                    
          if (!UnMHT.memoryCacheMode) {
            s += "<tr>"
              + "<th>"
              + UnMHT.messages.GetStringFromName
              ("native_filaname") + " "
              + "</th>"
              + "<td>"
              + UnMHT.conv.escapeEntity (file.nativeFilename)
              + "</td>"
              + "</tr>";
          }
                    
          s += "<tr>"
            + "<th>"
            + UnMHT.messages.GetStringFromName ("file_size") + " "
            + "</th>"
            + "<td>"
            + readableSize + " "
            + UnMHT.messages.GetStringFromName ("bytes")
            + "</td>"
            + "</tr>";
        }
        else {
          s += "<tr>"
            + "<th>"
            + UnMHT.messages.GetStringFromName ("not_extracted")
            + "</th>"
            + "<td>"
            + " "
            + "</td>"
            + "</tr>";
        }
                
        s += "</table>";
      }
    }
        
    return [UnMHT.conv.convertFromUnicode (s, "utf-8"),
            size, i];
  },
    
  /**
   * mht から展開したファイルの URI を取得する
   * 他の拡張から呼び出される
   *
   * @param  nsIURI contentLocation
   *         対象の URI
   * @param  nsIURI requestOrigin
   *         呼び出し元の URI
   *         アドレスに UnMHT と入っているもの
   * @return nsIURI
   *         展開したファイルの URI
   *         存在しなかった場合は null
   */
  getMHTFileURI : function (contentLocation, requestOrigin) {
    if (!UnMHT.protocolHandler.isUnMHTURI (requestOrigin.spec)) {
      return null;
    }
        
    var param
    = UnMHT.protocolHandler.getUnMHTURIParam (requestOrigin.spec);
    var original = param.original;
    var refName = param.refName;
        
    var extractor = null;
    var file = null;
    if (original
        && (extractor = UnMHT.getExtractor (original))) {
      if (refName
          && refName in extractor.refFiles) {
        file = extractor.refFiles [refName];
      }
      else {
        file = extractor.rootFile;
      }
    }
        
    if (file) {
      var list
      = UnMHT.checkFile (extractor, file, contentLocation.spec, false);
            
      if (list [0] != -1) {
        /* ファイルが MHT に含まれた場合 */
        if (list [1]) {
          /* 展開した場合 */
          var uri
            = Components
            .classes ["@mozilla.org/network/standard-url;1"]
            .createInstance (Components.interfaces.nsIURI);
          uri.spec = param.rootURI + list [1].refName + list [2];
          return uri;
        }
      }
    }
        
    return null;
  }, 
    
  /**
   * mht に含まれるファイルを探す
   *
   * @param  UnMHTExtractor extractor
   *         ファイルの展開部分
   * @param  UnMHTExtractorFile file
   *         呼び出し元のファイル
   * @param  String path
   *         対象のパス
   * @param  Boolean referred
   *         referredBaseDir をチェックするか
   * @return Array
   *         [Number ファイルのインデックス,
   *          UnMHTExtractorFile 対象のファイル,
   *          String アンカー文字列]
   *         見付からなければインデックスが -1
   *         見付かっても削除されていればファイルが null
   */
  checkFile : function (extractor, file, path, referred) {
    var jointedPath = UnMHTFileManager.jointPath (file.baseDir, path);
    var jointedPath2 = "";
    var jointedPath3 = "";
    var jointedPath4 = "";
    path = UnMHT.normalizeName (path);
    jointedPath = UnMHT.normalizeName (jointedPath);
    
    if (path == "about:blank") {
      return [-1, "", ""];
    }
    
    if (referred && file.referredBaseDir != "") {
      /* HTML ファイルから参照されたものの場合 */
            
      /* 参照元のファイルからの相対パスとみなす */
      jointedPath2
        = UnMHTFileManager.jointPath (file.referredBaseDir, path);
      jointedPath2 = UnMHT.normalizeName (jointedPath2);
    }
        
    var i, file2, filename;
        
    var refName = "";
        
    var anchor = "";
    if (path.match (/^([^#]*)(#.+)$/)) {
      path = RegExp.$1;
      anchor = RegExp.$2;
    }
    if (anchor && path == "") {
      /* 同じファイル */
      for (i = 0; i < extractor.files.length; i ++) {
        if (file == extractor.files [i]) {
          if (file.deleted) {
            return [i, null, anchor];
          }
                    
          return [i, file, anchor];
        }
      }
    }
        
    if (UnMHT.mode == 1
        && UnMHT.protocolHandler.isUnMHTURI (path)) {
      var param = UnMHT.protocolHandler.getUnMHTURIParam (path);
      
      var original = param.original; 
      var refName = param.refName;
      if (refName == "") {
        refName = "index.html";
      }
            
      if (original == extractor.original) {
        for (i = 0; i < extractor.files.length; i ++) {
          file2 = extractor.files [i];
          if (file2.refName == refName) {
            if (file2.deleted) {
              return [i, null, anchor];
            }
            return [i, file2, anchor];
          }
        }
      }
            
      var absPath = "/" + param.path;
      jointedPath3 = UnMHTFileManager.jointPath (file.baseDir, absPath);
      jointedPath3 = UnMHT.normalizeName (jointedPath3);
      if (referred && file.referredBaseDir != "") {
        /* HTML ファイルから参照されたものの場合 */
            
        /* 参照元のファイルからの相対パスとみなす */
        jointedPath4
          = UnMHTFileManager.jointPath (file.referredBaseDir,
                                        absPath);
        jointedPath4 = UnMHT.normalizeName (jointedPath4);
      }
      else {
        jointedPath4 = "";
      }
    }
    
    /* URI に対応するファイルが存在する場合 */
    for (i = 0; i < extractor.files.length; i ++) {
      file2 = extractor.files [i];
      
      filename = UnMHT.normalizeName (file2.location);
      if (filename) {
        if (path == filename
            || jointedPath == filename
            || (referred && jointedPath2 == filename)
            || jointedPath3 == filename
            || (referred && jointedPath4 == filename)) {
          if (file2.deleted) {
            return [i, null, anchor];
          }
                    
          return [i, file2, anchor];
        }
      }
    }
        
    /* ファイル名のみ対応するファイルが存在する場合 */
    for (i = 0; i < extractor.files.length; i ++) {
      file2 = extractor.files [i];
      
      filename = UnMHT.normalizeName (file2.leafName);
      if (filename) {
        var path2 = path.replace (/\?.+/, "");
        if (path2.length >= filename.length + 1
            && path2.substr (path2.length - filename.length,
                             filename.length) == filename
            && path2.substr (path2.length - filename.length - 1,
                             1).match (/[\?\\:]/)) {
          if (file2.deleted) {
            return [i, null, anchor];
          }
          
          return [i, file2, anchor];
        }
      }
    }
    
    /* 変換済みの Content-ID の場合 */
    var cid = "";
    if (UnMHT.mode == 1) {
      if (path.match (/unmht:\/\/\/cid\.0\/(.+)/)) {
        cid = RegExp.$1;
      }
    }
    else if (UnMHT.mode == 2) {
      if (path.match (/^data:text\/plain,cid:(.+)$/)) {
        cid = RegExp.$1;
      }
    }
    
    if (cid) {
      for (i = 0; i < extractor.files.length; i ++) {
        file2 = extractor.files [i];
        if (cid == file2.cid) {
          if (file2.deleted) {
            return [i, null, anchor];
          }
                    
          return [i, file2, anchor];
        }
      }
    }
    
    if (UnMHT.mode == 1) {
      var path2 = path;
      try {
        if (UnMHT.protocolHandler.isUnMHTURI (path)) {
          var param = UnMHT.protocolHandler.getUnMHTURIParam (path);
          if (param.protocol == "dummy") {
            path2 = param.original
              .replace (/^dummy:/, "file:");
          }
        }
        path2 = unescape (path2);
        path2 = UnMHT.conv.convertToUnicode (path2, "UTF-8");
      }
      catch (e) {
      }
            
      if (path2 != path) {
        return UnMHT.checkFile (extractor, file, path2, referred);
      }
            
      var upath = UnMHT.conv.unescapeEntity (path);
      if (upath != path) {
        return UnMHT.checkFile (extractor, file, upath, referred);
      }
    }
    
    return [-1, "", ""];
  },
    
  /**
   * 展開したファイルの情報を取得する
   *
   * @param  String uri
   *         unmht: 形式のファイルの URI
   * @return UnMHTExtractorFile
   *         ファイルの情報
   *         見付からなかった場合は null
   */
  getFile : function (uri) {
    if (!UnMHT.protocolHandler.isUnMHTURI (uri)) {
      return null;
    }
    
    var param
    = UnMHT.protocolHandler.getUnMHTURIParam (uri);
    var original = param.original;
    var refName = param.refName;
    
    var extractor = null;
    var file = null;
    if (original
        && (extractor = UnMHT.getExtractor (original))) {
      extractor.lastAccessTime = (new Date ()).getTime ();
      if (refName) {
        if (refName in extractor.refFiles) {
          file = extractor.refFiles [refName];
        }
        else {
          return null;
        }
      }
      else {
        file = extractor.rootFile;
      }
    }      
    
    return file;
  },

  /**
   * unmht 形式の URI からのパスを展開する
   *
   * @param  String uri
   *         unmht: 形式のファイルの URI
   * @param  String spec
   *         相対パス
   * @return String
   *         ファイルの URI
   *         見付からなかった場合は null
   */
  resolveURI : function (uri, spec) {
    if (!UnMHT.protocolHandler.isUnMHTURI (uri)) {
      return null;
    }
    
    var param
    = UnMHT.protocolHandler.getUnMHTURIParam (uri);
    var original = param.original;
    var refName = param.refName;
    
    var extractor = null;
    var file = null;
    if (original
        && (extractor = UnMHT.getExtractor (original))) {
      extractor.lastAccessTime = (new Date ()).getTime ();
      if (refName) {
        if (refName in extractor.refFiles) {
          file = extractor.refFiles [refName];
        }
        else {
          return null;
        }
      }
      else {
        file = extractor.rootFile;
      }
      
      if (file && spec) {
        var list
          = UnMHT.checkFile (extractor, file, spec, false);
        if (list [0] != -1) {
          return extractor.rootURI + list [1].refName + list [2];
        }
        else {
          if (!UnMHT.protocolHandler.isUnMHTURI (spec)) {
            return UnMHT.resolveURI (spec, "");
          }
        }
      }
    }
    
    return null;
  },
    
  /**
   * mht 内のファイル名を置き換える
   *
   * @param  Number contentType
   *         コンテントの種類
   * @param  nsIURI contentLocation
   *         対象の URI
   * @param  nsIURI requestOrigin
   *         呼び出し元の URI
   *         アドレスに UnMHT と入っているもの
   * @param  HTMLElement context
   *         呼び出し元
   * @return Boolean
   *         許可するかどうか
   */
  shouldLoad : function (contentType, contentLocation, requestOrigin, context) {
    if (context == null) {
      /* 古い Mozilla Suite でのアンカーによる移動の場合
       * フィルタ抜きに許可する */
      return true;
    }
    if (UnMHT.enableLoadImageType
        && UnMHT.loadImageType == 2
        && contentType == Components.interfaces.nsIContentPolicy.TYPE_IMAGE) {
      return false;
    }
    
    if (context instanceof XULElement) {
      /* アンカーによる移動の場合
       * フィルタ抜きに許可する */
      return true;
    }
    
    if (context instanceof HTMLDocument
        || context instanceof HTMLStyleElement) {
      /* CSS からの参照の場合 */
      if (UnMHT.enableFastExtract) {
        /* CSS を無視する場合、拒否する */
        return false;
      }
      else {
        /* それ以外の場合、許可する */
        return UnMHT.checkExternal (contentLocation);
      }
    }
        
    if (context.getAttribute ("__unmht_modified") == "1") {
      /* setTimeout の修正からの読み込みの場合、許可する */
      return true;
    }
        
    /* 対応する展開のチェック */
    if (UnMHT.protocolHandler.isUnMHTURI (requestOrigin.spec)) {
      var param
      = UnMHT.protocolHandler.getUnMHTURIParam (requestOrigin.spec);
      var original = param.original;
      var refName = param.refName;
            
      var extractor = null;
            
      extractor = UnMHT.getExtractor (original);
            
      if (extractor) {
        /* 対応する展開があった場合 */
        var attribute = "src";
        
        if (refName
            && refName in extractor.refFiles) {
          file = extractor.refFiles [refName];
        }
        else {
          file = extractor.rootFile;
        }
        
        var spec = contentLocation.spec;
        
        if (UnMHT.protocolHandler.isUnMHTURI (spec)) {
          var param2
            = UnMHT.protocolHandler.getUnMHTURIParam (spec);
          if (param2.original == original) {
            /* もう変換済み */
            return true;
          }
        }
        
        var list
          = UnMHT.checkFile (extractor, file, spec, false);
        if (list [0] != -1) {
          /* ファイルが MHT に含まれた場合 */
          if (list [1]) {
            /* 展開した場合、置き換える */
            var newpath
              = extractor.rootURI + list [1].refName + list [2];
            if (context instanceof HTMLEmbedElement) {
              context.ownerDocument.defaultView.setTimeout
                (function (node, attr, name) {
                  var newNode = node.cloneNode (true);
                                
                  newNode.setAttribute
                    ("__unmht_modified", "1");
                  var tmp = "__unmht_" + attribute;
                  var value = newNode.getAttribute (attr);
                  if (!newNode.hasAttribute (tmp)) {
                    newNode.setAttribute (tmp, value);
                  }
                  newNode.setAttribute (attr, name);
                                    
                  var parentNode = node.parentNode;
                  var nextSibling = node.nextSibling;
                                    
                  if (parentNode) {
                    parentNode.removeChild (node);
                                    
                    if (nextSibling) {
                      parentNode.insertBefore
                        (newNode, nextSibling);
                    }
                    else {
                      parentNode.appendChild
                        (newNode);
                    }
                  }
                }, 100, context, attribute, newpath);
              return false;
            }
            else {
              if (!UnMHT.isOld) {
                context.setAttribute ("__unmht_modified", "1");
                context.setAttribute ("__unmht_" + attribute,
                                      context.getAttribute
                                      (attribute));
                context.setAttribute (attribute, newpath);
              }
              context.ownerDocument.defaultView.setTimeout
                (function (node, attr, name) {
                  node.setAttribute ("__unmht_modified", "1");
                  var tmp = "__unmht_" + attribute;
                  var value = node.getAttribute (attr);
                  if (!node.hasAttribute (tmp)) {
                    node.setAttribute (tmp, value);
                  }
                  node.setAttribute (attr, name);
                }, 100, context, attribute, newpath);
              return false;
            }
          }
          else {
            /* 展開しなかった場合、削除する */
            if (!UnMHT.isOld) {
              context.removeAttribute (attribute);
            }
            context.ownerDocument.defaultView.setTimeout
              (function (node, attr) {
                try {
                  context.removeAttribute (attr);
                }
                catch (e) {
                }
              }, 100, context, attribute);
            return false;
          }
        }
        
        /* 対応するものが見付からなかった場合 */
            
        if (spec.match (/^unmht:\/\/\/dummy\./)
            || spec.match (/^file:\/\//)) {
          /* 参照先はローカルファイルのなので拒否する  */
          if (!UnMHT.isOld) {
            context.removeAttribute (attribute);
          }
          context.ownerDocument.defaultView.setTimeout
            (function (node, attr) {
              try {
                context.removeAttribute (attr);
              }
              catch (e) {
              }
            }, 100, context, attribute);
          return false;
        }
            
        if (spec.indexOf (requestOrigin.spec) == 0) {
          /* unmht のパスからの相対になっている */
          var newpath;
          var sub = spec.substr (requestOrigin.spec.length);
          if (sub
              && sub.match (/^\?/)) {
            /* 元のアドレスにクエリ文字列を追加しただけ */
            newpath
              = file.location.replace (/\?.*$/, "") + sub;
            contentLocation.spec = newpath;
                    
            return UnMHT.shouldLoad (contentType, contentLocation,
                                     requestOrigin, context);
          }
          else {
            newpath
              = UnMHTFileManager.jointPath (file.baseDir, sub);
          }
                
          if (context instanceof HTMLEmbedElement) {
            context.ownerDocument.defaultView.setTimeout
              (function (node, attr, name) {
                var newNode = node.cloneNode (true);
                            
                newNode.setAttribute ("__unmht_modified", "1");
                var tmp = "__unmht_" + attribute;
                var value = newNode.getAttribute (attr);
                if (!newNode.hasAttribute (tmp)) {
                  newNode.setAttribute (tmp, value);
                }
                newNode.setAttribute (attr, name);
                                
                var parentNode = node.parentNode;
                var nextSibling = node.nextSibling;
                                
                if (parentNode) {
                  parentNode.removeChild (node);
                                
                  if (nextSibling) {
                    parentNode.insertBefore
                      (newNode, nextSibling);
                  }
                  else {
                    parentNode.appendChild
                      (newNode);
                  }
                }
              }, 100, context, attribute, newpath);
            return false;
          }
          else {
            if (!UnMHT.isOld) {
              context.setAttribute ("__unmht_modified", "1");
              context.setAttribute ("__unmht_" + attribute,
                                    context.getAttribute
                                    (attribute));
              context.setAttribute (attribute, newpath);
            }
            context.ownerDocument.defaultView.setTimeout
              (function (node, attr, name) {
                node.setAttribute ("__unmht_modified", "1");
                var tmp = "__unmht_" + attribute;
                var value = node.getAttribute (attr);
                if (!node.hasAttribute (tmp)) {
                  node.setAttribute (tmp, value);
                }
                node.setAttribute (attr, name);
              }, 100, context, attribute, newpath);
            return false;
          }
        }
      }
    }
        
    return UnMHT.checkExternal (contentLocation);
  },
  
  /**
   * 外部読み込みをチェックする
   *
   * @param  nsIURI contentLocation
   *         対象の URI
   * @return Boolean
   *         許可するかどうか
   */
  checkExternal : function (contentLocation) {
    if (contentLocation.spec.match (/^unmht:\/\//)) {
      return true;
    }
    
    if (UnMHT.enableFilterLoad) {
      if (UnMHT.enableFilterLoadOffline) {
        return false;
      }
      
      var host = "", leafName = "", extension = "";
      var uri = contentLocation.spec;
      
      try {
        host = contentLocation.host;
      }
      catch (e) {
      }
      
      try {
        if (contentLocation.path.match (/\/([^\/]+)$/)) {
          leafName = RegExo.$1;
          leafName = leafName.replace (/\?.+$/, "");
          if (leafName.match (/(\.[A-Za-z0-9]+)$/)) {
            extension = RegExp.$1;
          }
        }
      }
      catch (e) {
      }
      
      for (var j = 0; j < UnMHT.filterLoadList.length; j ++) {
        var text = "";
        if (UnMHT.filterLoadList [j][0] == "host") {
          text = host;
        }
        else if (UnMHT.filterLoadList [j][0] == "leafname") {
          text = leafName;
        }
        else if (UnMHT.filterLoadList [j][0] == "extension") {
          text = extension;
        }
        else if (UnMHT.filterLoadList [j][0] == "uri") {
          text = uri;
        }
        
        var pos = -1;
        if (UnMHT.filterLoadList [j][2]) {
          pos = text.search (UnMHT.filterLoadList [j][1]);
        }
        else {
          pos = text.indexOf (UnMHT.filterLoadList [j][1]);
        }
        
        if (pos != -1) {
          return false;
        }
      }
    }
    
    return true;
  },
    
  /**
   * 展開先のディレクトリの名前を作成する
   *
   * @return String
   *         展開先のディレクトリの名前
   */
  createExtractName : function () {
    return new Date ().getTime () + "_"
    + Math.floor (Math.random () * 1000);
  },
    
  /**
   * mht ファイルを展開して開く
   *
   * @param  nsIURI uri
   *         開く URI
   * @param  Window targetWindow
   *         対象のウィンドウ
   * @param  Boolean hasDocument
   *         ドキュメントに内容を持っているか
   * @param  Boolean replace
   *         履歴を置き換えるかどうか
   * @param  Number extract
   *         展開するか
   *           0: 設定にまかせる
   *           1: 展開する
   *           2: 展開しない
   * @param  String refName
   *         対象のファイル
   * @param  Boolean useCache
   *         キャッシュを使用するか
   */
  loadMHT : function (uri, targetWindow,
                      hasDocument, replace, extract, refName,
                      useCache) {
    if (Components.interfaces.nsIPrefBranch2 == undefined) {
      /* 監視していない場合にのみ設定を取得する */
      UnMHT.getConfigurationFromPreferencesAll ();
      UnMHT.getConfigurationFromPreferences ();
    }
        
    if (extract == 0) {
      extract = UnMHT.enableAll;
    }
    else if (extract == 1) {
      extract = true;
    }
    else if (extract == 2) {
      extract = false;
    }
        
    if (uri.scheme == "file") {
      /* file プロトコルの場合、そのファイルから開く */
      var filename = UnMHTFileManager.getFilenameFromURLSpec (uri.spec);
      UnMHT.loadMHTFromFile (uri, targetWindow,
                             hasDocument,
                             replace,
                             extract,
                             refName,
                             filename, "");
    }
    else {
      /* それ以外の場合、ウェブから開く */
      UnMHT.loadMHTFromWeb (uri, targetWindow,
                            hasDocument,
                            replace,
                            extract,
                            refName,
                            useCache);
    }
  },
    
  /**
   * ファイルから mht ファイルを展開して開く
   *
   * @param  nsIURI uri
   *         開く URI (本来の URI)
   * @param  Window targetWindow
   *         対象のウィンドウ
   * @param  Boolean hasDocument
   *         ドキュメントに内容を持っているか
   * @param  Boolean replace
   *         履歴を置き換えるかどうか
   * @param  Boolean extract
   *         展開するか
   * @param  String filename
   *         開くファイル名 (一旦キャッシュに保存したものの場合もある)
   * @param  String extractName
   *         展開するディレクトリ
   * @param  String refName
   *         対象のファイル
   */
  loadMHTFromFile : function (uri, targetWindow,
                              hasDocument, replace, extract, refName,
                              filename, extractName) {
    var text = UnMHTFileManager.readFile (filename);
    if (text == "") {
      if (hasDocument) {
        UnMHT.loadMHTFromDocument (uri, targetWindow,
                                   true,
                                   replace,
                                   extract,
                                   refName);
      }
      else {
        UnMHT.showErrorPage (targetWindow, uri.spec);
      }
      return;
    }
        
    var extractor = new UnMHTExtractor ();
    var urlspec
    = extractor.extractMHT (uri.spec, text, false, extractName, extract);
    if (urlspec) {
      if (!UnMHT.memoryCacheMode
          && UnMHT.getExtractor (extractor.original)) {
        var dir, dirname;
        dirname = UnMHTFileManager.cacheDirectory
          + UnMHTFileManager.separator
          + UnMHT.extractors [extractor.original].extractName;
        dir = UnMHTFileManager.initFile (dirname);
        if (dir && dir.exists ()) {
          dir.remove (true);
        }
      }
            
      UnMHT.setExtractor (extractor.original, extractor);
            
      var nav
        = targetWindow
        .QueryInterface (Components.interfaces.nsIInterfaceRequestor)
        .getInterface (Components.interfaces.nsIWebNavigation);
            
      var flag = 0;
      if (replace) {
        flag = nav.LOAD_FLAGS_REPLACE_HISTORY;
      }
            
      if (refName
          && refName in extractor.refFiles) {
        urlspec += refName;
      }
            
      nav.loadURI (urlspec,
                   flag, null, null, null);
    }
  },
    
  /**
   * ドキュメントから mht ファイルを展開して開く
   *
   * @param  nsIURI uri
   *         開く URI
   * @param  Window targetWindow
   *         対象のウィンドウ
   * @param  Boolean hasDocument
   *         ドキュメントに内容を持っているか
   * @param  Boolean replace
   *         履歴を置き換えるかどうか
   * @param  Boolean extract
   *         展開するか
   * @param  String refName
   *         対象のファイル
   */
  loadMHTFromDocument : function (uri, targetWindow,
                                  hasDocument, replace, extract, refName) {
    var text = "";
    var nodes = targetWindow.document.getElementsByTagName ("pre");
    for (var i = 0; i < nodes.length; i ++) {
      var node = nodes [i].firstChild;
      while (node) {
        text += node.nodeValue;
        node = node.nextSibling;
      }
    }
        
    if (text == "") {
      UnMHT.showErrorPage (targetWindow, uri.spec);
      return;
    }
        
    var extractor = new UnMHTExtractor ();
    var urlspec
    = extractor.extractMHT (targetWindow.document.location.href,
                            text, true, "", extract);
    if (urlspec) {
      if (!UnMHT.memoryCacheMode
          && UnMHT.getExtractor (extractor.original)) {
        var dir, dirname;
        dirname = UnMHTFileManager.cacheDirectory
          + UnMHTFileManager.separator
          + UnMHT.extractors [extractor.original].extractName;
        dir = UnMHTFileManager.initFile (dirname);
        if (dir && dir.exists ()) {
          dir.remove (true);
        }
      }
            
      UnMHT.setExtractor (extractor.original, extractor);
            
      var nav
        = targetWindow
        .QueryInterface (Components.interfaces.nsIInterfaceRequestor)
        .getInterface (Components.interfaces.nsIWebNavigation);
            
      var flag = 0;
      if (replace) {
        flag = nav.LOAD_FLAGS_REPLACE_HISTORY;
      }
            
      if (refName
          && refName in extractor.refFiles) {
        urlspec += refName;
      }
            
      nav.loadURI (urlspec,
                   flag, null, null, null);
    }
  },
    
  /**
   * ウェブのファイルから mht ファイルを展開して開く
   *
   * @param  nsIURI uri
   *         開く URI
   * @param  Window targetWindow
   *         対象のウィンドウ
   * @param  Boolean hasDocument
   *         ドキュメントに内容を持っているか
   * @param  Boolean replace
   *         履歴を置き換えるかどうか
   * @param  Boolean extract
   *         展開するか
   * @param  String refName
   *         対象のファイル
   * @param  Boolean useCache
   *         キャッシュを使用するか
   */
  loadMHTFromWeb : function (uri, targetWindow,
                             hasDocument, replace, extract, refName,
                             useCache) {
    var extractName = "";
    if (UnMHT.memoryCacheMode) {
      var leafName 
        = new Date ().getTime () + "_"
        + Math.floor (Math.random () * 1000);
      var filename
        = UnMHTFileManager.cacheDirectory
        + UnMHTFileManager.separator + leafName;
    }
    else {
      /* 展開するディレクトリを作成する */
      extractName = UnMHT.createExtractName ();
      var extractDirectory
      = UnMHTFileManager.cacheDirectory
      + UnMHTFileManager.separator + extractName;
      UnMHTFileManager.createDirectory (extractDirectory);
            
      var filename = extractDirectory
      + UnMHTFileManager.separator + "cache";
    }
        
    var targetFile = UnMHTFileManager.initFile (filename);
        
    var listener = new UnMHTProgressListener ();
        
    listener.uri = uri;
    listener.targetWindow = targetWindow;
        
    listener.hasDocument = hasDocument;
    listener.replace = replace;
    listener.extract = extract;
    listener.refName = refName;
        
    listener.filename = filename;
    listener.extractName = extractName;
        
    /* ファイルを保存する */
    var webBrowserPersist
    = Components
    .classes ["@mozilla.org/embedding/browser/nsWebBrowserPersist;1"]
    .createInstance (Components.interfaces.nsIWebBrowserPersist);
    if (useCache) {
      webBrowserPersist.persistFlags = 0;
    }
    else {
      webBrowserPersist.persistFlags
      = webBrowserPersist.PERSIST_FLAGS_BYPASS_CACHE;
    }
    webBrowserPersist.progressListener = listener;
    try {
      webBrowserPersist.saveURI (uri, null, targetFile);
    }
    catch (e) {
      try {
        webBrowserPersist.saveURI (uri, null, null, null, null,
                                   targetFile);
      }
      catch (e) {
        if (hasDocument) {
          UnMHT.loadMHTFromDocument (uri, targetWindow,
                                     true,
                                     replace,
                                     extract,
                                     refName);
        }
        else {
          UnMHT.showErrorPage (targetWindow, uri.spec);
        }
      }
    }
  },
    
  /**
   * フォーカスのあるウィンドウから mht ファイルを開く
   *
   * @param  Number extract
   *         展開するか
   *           0: 設定にまかせる
   *           1: 展開する
   *           2: 展開しない
   */
  loadMHTFromFocusedWindow : function (extract) {
    var targetWindow = document.commandDispatcher.focusedWindow;
    if (targetWindow == window) {
      targetWindow = content;
    }
        
    var uri
    = Components.classes ["@mozilla.org/network/standard-url;1"]
    .createInstance (Components.interfaces.nsIURI);
    uri.spec = targetWindow.document.location.href;
        
    if (UnMHT.protocolHandler.isUnMHTURI (uri.spec)) {
      uri.spec = UnMHT.protocolHandler.deUnMHTURI (uri.spec);
    }
        
    UnMHT.loadMHT (uri, targetWindow,
                   true,    /* 開いているドキュメントを使用する */
                   false,   /* 履歴はそのまま */
                   extract, /* 展開は引数次第 */
                   "",      /* ルートドキュメントを開く */
                   true);   /* キャッシュを使用する */
  },
    
  /**
   * エラーメッセージを表示する
   *
   * @param  Window targetWindow
   *         対象のウィンドウ
   * @param  String location
   *         ファイルの URI
   */
  showErrorPage : function (targetWindow, location) {
    var nav
    = targetWindow
    .QueryInterface (Components.interfaces.nsIInterfaceRequestor)
    .getInterface (Components.interfaces.nsIWebNavigation);
        
    nav.loadURI (
      this.getErrorPageURI(location),
      0, null, null, null);
  },
    
  /**
   * エラーページの URI を返す
   *
   * @param  String location
   *         ファイルの URI
   */
  getErrorPageURI : function (location) {
    location = UnMHT.protocolHandler.enUnMHTURI (location);
    location = location.replace (/^unmht:/, "error:");
    location = UnMHT.protocolHandler.enUnMHTURI (location);
        
    return location;
  },
    
  /**
   * キャッシュを削除する
   *
   * @param  Number expire
   *         ミリ秒単位の経過時間
   *         0 なら即削除
   * @param  Boolean force
   *         強制削除
   */
  clearCache : function (expire, force) {
    if (!UnMHT.enableCacheExpire && !force) {
      return;
    }
        
    if (UnMHT.memoryCacheMode) {
      var now = new Date ().getTime ();

      for (var urispec in UnMHT.extractors) {
        var extractor = UnMHT.extractors [urispec];
            
        if (now < extractor.lastAccessTime + expire) {
          /* 最終アクセスから指定時間経過していない */
          continue;
        }
                
        UnMHT.extractors [urispec] = null;
        delete UnMHT.extractors [urispec];
      }
            
      if (expire == 0) {
        UnMHTFileManager.clearCacheDirectory (expire, false);
      }
    }
    else {
      UnMHTFileManager.clearCacheDirectory (expire, true);
    }
  },
  
  padDup : function (n) {
    var s = n + "";
    
    while (s.length < UnMHT.dupDigits) {
      s = "0" + s;
    }
    
    return s;
  }
};

var UnMHT_isThunderbird = false;
try {
  if (Components.classes ["@mozilla.org/xre/app-info;1"]
      .getService (Components.interfaces.nsIXULAppInfo).ID
      == "{3550f703-e582-4d05-9a08-453d09bdfdc6}") {
    UnMHT_isThunderbird = true;
  }
}
catch (e) {
}

if (UnMHT_isThunderbird) {
  /* Thunderbird */
  UnMHT.protocolHandler = {
    enUnMHTURI : function (uri) {
      return uri;
    },
    isUnMHTURI : function (uri) {
      return false;
    },
    deUnMHTURI : function (uri) {
      return uri;
    },
    getUnMHTURIParam : function (uri) {
      return null;
    }
  };
}
else {
  UnMHT.protocolHandler
    = Components.classes ["@mozilla.org/network/protocol;1?name=unmht"]
    .getService (Components.interfaces.arIUnMHTProtocolHandler);
}

/* 古い Mozilla Suite では最初のイベントリスナが無視されるので 2 つ登録する */
window.addEventListener ("load",
                         function () {
                           UnMHT.onLoad ();
                         }, false);
window.addEventListener ("load",
                         function () {
                           UnMHT.onLoad ();
                         }, false);

window.addEventListener ("unload",
                         function () {
                           UnMHT.onUnload ();
                         }, false);
window.addEventListener ("unload",
                         function () {
                           UnMHT.onUnload ();
                         }, false);

